/*------------------------------------------------------------------------------------------------------
  
  File        : CImg.h 
  Description : Main file of `CImg : The C++ Template Image Library'.
  Author      : David Tschumperl
  Institution : Initially developped at ODYSSEE Lab, INRIA Sophia Antipolis.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  ----------------------------------------------------------------------------------------------------*/

#ifndef cimg_include
#define cimg_include
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <cmath>
#include <cstring>
#include <ctime>

/*-------------------------------------------------------------


  Detect and set CImg configuration flags

  
  If compilation flags are not adapted to your system, you can
  set them to correct values, before including
  the file CImg.h.


  -------------------------------------------------------------*/

typedef unsigned int   uint;              //!< small typename for \a unsigned \a int.
typedef unsigned char  uchar;             //!< small typename for \a unsigned \a char.
typedef unsigned short ushort;            //!< small typename for \a unsigned \a short.
struct CImgStats;
struct CImgDisplay;
struct CImgException;
template<typename T=float> struct CImg;
template<typename T=float> struct CImgl;
template<typename T=float> struct CImgROI;

// Overcome VisualC++ 6.0 and DMC compilers namespace bug
#if ( defined(_MSC_VER) && _MSC_VER<=1200 ) || defined(__DMC__)
#define std
#endif

// Autodetection of the current OS
#ifndef cimg_OS
#if defined(sun) || defined(__sun)        
// Solaris
#define cimg_OS            0
#ifndef cimg_display_type
#define cimg_display_type  1
#endif
#elif defined(linux) || defined(__linux) || defined(__CYGWIN__)
// Linux
#define cimg_OS            1
#ifndef cimg_display_type
#define cimg_display_type  1
#endif
#ifndef cimg_color_term
#define cimg_color_term
#endif
#elif defined(_WIN32) || defined(__WIN32__)
// Windows
#define cimg_OS            2
#ifndef cimg_display_type
#define cimg_display_type  2
#endif
#elif defined(__MACOSX__) || defined(__APPLE__)
// Mac OS X
#define cimg_OS            3
#ifndef cimg_display_type
#define cimg_display_type  1
#endif
#else 
// Unknow configuration
#define cimg_OS           -1
#ifndef cimg_display_type
#define cimg_display_type  0
#endif
#endif
#endif
// Other configuration flags
#ifndef cimg_debug
#define cimg_debug         1
#endif

// Architecture-dependent includes
#if cimg_OS!=2
#include <sys/time.h>
#include <unistd.h>
#else
#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400     // Overcome bug in macro definition in winuser.h (allow the use of TrackMouseEvent() )
#endif
#include <windows.h>
#endif
#if cimg_display_type==1
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <pthread.h>
static pthread_mutex_t* cimg_mutex = NULL;
static pthread_t*       cimg_event_thread = NULL;
static CImgDisplay*     cimg_wins[1024];
static Display*         cimg_displayX11 = NULL;
static volatile uint    cimg_nb_wins = 0;
static volatile bool    cimg_thread_finished = false;
static uint             cimg_bits = 0;
static GC*              cimg_gc = NULL;
#endif

/*----------------------------------------------------



  Definition of the CImg Exception Classes



  -------------------------------------------------*/

#if cimg_debug>=1
#if cimg_display_type!=2
#define cimg_exception_print(str) std::fprintf(stderr,"<CImg Error> %s",str);
#else
#define cimg_exception_print(str) MessageBox(NULL,str,"<CImg Error>",MB_OK);
#endif
#else
#define cimg_exception_print(str)
#endif
#define cimg_exception_err(etype)                                       \
  char tmp[1024];                                                       \
  va_list ap;                                                           \
  va_start(ap,format);                                                  \
  std::vsprintf(message,format,ap);                                     \
  va_end(ap);                                                           \
  std::sprintf(tmp,"<Error cause> %s \n\nGeneral : %s\n\n",             \
               message,etype);                                          \
  cimg_exception_print(tmp)

//! This class is used to throw general exceptions, and is the base class of all CImg exceptions.
struct CImgException {
  char message[1024];
  CImgException() { message[0]='\0'; }
  CImgException(const char *format,...) {
    cimg_exception_err("This error has been generated by a 'CImgException' throw, corresponding to a general exception problem."); 
  }
};
//! This class is used to throw an exception related to a non suitable instance encountered in a library function call.
struct CImgInstanceException : CImgException { 
  CImgInstanceException(const char *format,...) {
    cimg_exception_err("This error has been generated by a 'CImgInstanceException' throw.\n\
The instance passed through the function above has a bad structure (perhaps an empty image, list or display object ?)");
  }};
//! This class is used to throw an exception related to invalid arguments encountered in a library function call.
struct CImgArgumentException : CImgException { 
  CImgArgumentException(const char *format,...) { 
    cimg_exception_err("This error has been generated by a 'CImgArgumentException' throw.\n\
At least one argument passed to the function above has been considered as not valid.");
  }};
//! This class is used to throw an exception related to Input/Output file problems encountered in a library function call.
struct CImgIOException : CImgException { 
  CImgIOException(const char *format,...) {
    cimg_exception_err("This error has been generated by a 'CImgIOException' throw.\n\
When trying to load or save a file, the function above has encountered a problem.");
  }};
//! This class is used to throw an exception related to display problems encountered in a library function call.
struct CImgDisplayException : CImgException {
  CImgDisplayException(const char *format,...) {
    cimg_exception_err("This error has been generated by a 'CImgDisplayException' throw.\n\
When trying to operate on a CImgDisplay instance, the function above encountered a problem."); 
  }};


/*----------------------------------------------------



  Definition of the LAPACK layer functions



  -------------------------------------------------*/

#ifdef cimg_lapack
extern "C"
{
  extern void dgeev_(char *JOBVL,char *JOBVR, int *N,double *A,int *LDA,double *WR,double *WI,double *VL,int *LDVL,
                     double *VR,int *LDVR,double *WORK,int *LWORK,int *INFO);
  extern void dsyev_(char *JOBZ,char *UPLO,int *N,double *A,int *LDA,double *W,double *WORK,int *LWORK,int *INFO);
  extern void dgetrf_(int *M,int *N,double *A,int *LDA,int *IPIV,int *INFO);
  extern void dgetri_(int *N,double *A,int *LDA,int *IPIV,double *WORK,int *LWORK,int *INFO);
}
#else
inline void cimg_nolapack() { 
  throw CImgException("a LAPACK call : A LAPACK function has been required, but the LAPACK library\
haven't been linked.\nPlease define the compilation flag '#define cimg_lapack' before including 'CImg.h' and link your code with LAPACK."); 
}
inline void dgeev_(char *JOBVL,char *JOBVR, int *N,double *A,int *LDA,double *WR,double *WI,double *VL,int *LDVL,
                   double *VR,int *LDVR,double *WORK,int  *LWORK,int *INFO) { cimg_nolapack(); }
inline void dsyev_(char *JOBZ, char *UPLO, int *N, double *A, int *LDA, double *W, double *WORK, int *LWORK, int *INFO) { cimg_nolapack(); }
inline void dgetrf_(int *M,int *N,double *A,int *LDA,int *IPIV,int *INFO) { cimg_nolapack(); }
inline void dgetri_(int *N,double *A,int *LDA,int *IPIV,double *WORK,int *LWORK,int *INFO) { cimg_nolapack(); }
#endif

/*-----------------------------------------------------------------------------------



  Define useful CImg macros (use only with a perfect knowledge of what it's doing !)



  ---------------------------------------------------------------------------------*/

//! Describe the program function. It will be displayed when calling the programm with option '-h'
#define cimg_usage(usage) cimg::option((char*)NULL,(uint)argc,(char**)argv,(char*)NULL,(char*)usage)
//! Get an option from the command line. The type of this macro is the one of the default value.
#define cimg_option(name,defaut,usage) cimg::option((char*)name,(uint)argc,(char**)argv,defaut,(char*)usage)

// Macros used for dynamic debugging messages
#define cimg_test(x) if(!(x).width || !(x).height || !(x).depth || !(x).dim || !(x).data) \
    throw CImgInstanceException("(Instance error) : File '%s', Line %d, CImg image '%s' = (%d,%d,%d,%d,%p) is an empty image", \
                                __FILE__,__LINE__,#x,(x).width,(x).height,(x).depth,(x).dim,(x).data)
#define cimgl_test(x) if(!(x).size || !(x).data) \
    throw CImgInstanceException("(Instance error) : File '%s', Line %d, CImgl list '%s' = (%d,%p) is an empty list", \
                                __FILE__,__LINE__,#x,(x).size,(x).data)
#define cimg_test_scalar(x) if(!(x).width || !(x).height || !(x).depth || (x).dim!=1 || !(x).data) \
    throw CImgInstanceException("(Instance error) : File '%s', Line %d, CImg image '%s' = (%d,%d,%d,%d,%p) is not a scalar image", \
                                __FILE__,__LINE__,#x,(x).width,(x).height,(x).depth,(x).dim,(x).data)
#define cimg_test_matrix(x) if(!(x).width || !(x).height || (x).depth!=1 || (x).dim!=1 || !(x).data) \
    throw CImgInstanceException("(Instance error) : File '%s', Line %d, CImg image '%s' = (%d,%d,%d,%d,%p) is not a matrix", \
                                __FILE__,__LINE__,#x,(x).width,(x).height,(x).depth,(x).dim,(x).data)
#define cimg_test_square(x) if(!(x).width || !(x).height || (x).depth!=1 || (x).dim!=1 || (x).width!=(x).height || !(x).data) \
    throw CImgInstanceException("(Instance error) : File '%s', Line %d, CImg image '%s' = (%d,%d,%d,%d,%p) is not a square matrix", \
                                __FILE__,__LINE__,#x,(x).width,(x).height,(x).depth,(x).dim,(x).data)

//@{
//! Macros used for neighborhood definitions and manipulations (used with loops such as 'map3x3')
#define CImg_3x3(I,T)     T I##pp,I##cp,I##np=0,I##pc,I##cc,I##nc=0,I##pn,I##cn,I##nn=0
#define CImg_5x5(I,T)     T I##bb,I##pb,I##cb,I##nb=0,I##ab=0,I##bp,I##pp,I##cp,I##np=0,I##ap=0, \
  I##bc,I##pc,I##cc,I##nc=0,I##ac=0,I##bn,I##pn,I##cn,I##nn=0,I##an=0, \
  I##ba,I##pa,I##ca,I##na=0,I##aa=0
#define CImg_3x3x3(I,T)   T I##ppp,I##cpp,I##npp=0,I##pcp,I##ccp,I##ncp=0,I##pnp,I##cnp,I##nnp=0,\
  I##ppc,I##cpc,I##npc=0,I##pcc,I##ccc,I##ncc=0,I##pnc,I##cnc,I##nnc=0,\
  I##ppn,I##cpn,I##npn=0,I##pcn,I##ccn,I##ncn=0,I##pnn,I##cnn,I##nnn=0

#define cimg_squaresum3x3(I) ( I##pp*I##pp + I##cp*I##cp + I##np*I##np + \
                               I##pc*I##pc + I##cc*I##cc + I##nc*I##nc + \
                               I##pn*I##pn + I##cn*I##cn + I##nn*I##nn )
#define cimg_squaresum5x5(I) ( I##bb*I##bb + I##pb*I##pb + I##cb*I##cb + I##nb*I##nb + I##ab*I##ab + \
                               I##bp*I##bp + I##pp*I##pp + I##cp*I##cp + I##np*I##np + I##ap*I##ap + \
                               I##bc*I##bc + I##pc*I##pc + I##cc*I##cc + I##nc*I##nc + I##ac*I##ac + \
                               I##bn*I##bn + I##pn*I##pn + I##cn*I##cn + I##nn*I##nn + I##an*I##an + \
                               I##ba*I##ba + I##pa*I##pa + I##ca*I##ca + I##na*I##na + I##aa*I##aa )
#define cimg_squaresum3x3x3(I) ( I##ppp*I##ppp + I##cpp*I##cpp + I##npp*I##npp + \
                                 I##pcp*I##pcp + I##ccp*I##ccp + I##ncp*I##ncp + \
                                 I##pnp*I##pnp + I##cnp*I##cnp + I##nnp*I##nnp + \
                                 I##ppc*I##ppc + I##cpc*I##cpc + I##npc*I##npc + \
                                 I##pcc*I##pcc + I##ccc*I##ccc + I##ncc*I##ncc + \
                                 I##pnc*I##pnc + I##cnc*I##cnc + I##nnc*I##nnc + \
                                 I##ppn*I##ppn + I##cpn*I##cpn + I##npn*I##npn + \
                                 I##pcn*I##pcn + I##ccn*I##ccn + I##ncn*I##ncn + \
                                 I##pnn*I##pnn + I##cnn*I##cnn + I##nnn*I##nnn )

#define cimg_corr3x3(I,m)   ( I##pp*(m)[0][0]+I##cp*(m)[1][0]+I##np*(m)[2][0] \
  + I##pc*(m)[0][1]+I##cc*(m)[1][1]+I##nc*(m)[2][1] \
  + I##pn*(m)[0][2]+I##cn*(m)[1][2]+I##nn*(m)[2][2] )
#define cimg_conv3x3(I,m)   ( I##pp*(m)[2][2]+I##cp*(m)[1][2]+I##np*(m)[0][2] \
  + I##pc*(m)[2][1]+I##cc*(m)[1][1]+I##nc*(m)[0][1] \
  + I##pn*(m)[2][0]+I##cn*(m)[1][0]+I##nn*(m)[0][0] )
#define cimg_corr5x5(I,m)   ( I##bb*(m)[0][0]+I##pb*(m)[1][0]+I##cb*(m)[2][0]+I##nb*(m)[3][0]+I##ab*(m)[4][0] \
  + I##bp*(m)[0][1]+I##pp*(m)[1][1]+I##cp*(m)[2][1]+I##np*(m)[3][1]+I##ap*(m)[4][1] \
  + I##bc*(m)[0][2]+I##pc*(m)[1][2]+I##cc*(m)[2][2]+I##nc*(m)[3][2]+I##ac*(m)[4][2] \
  + I##bn*(m)[0][3]+I##pn*(m)[1][3]+I##cn*(m)[2][3]+I##nn*(m)[3][3]+I##an*(m)[4][3] \
  + I##ba*(m)[0][4]+I##pa*(m)[1][4]+I##ca*(m)[2][4]+I##na*(m)[3][4]+I##aa*(m)[4][4] )
#define cimg_conv5x5(I,m)   ( I##bb*(m)[4][4]+I##pb*(m)[3][4]+I##cb*(m)[2][4]+I##nb*(m)[1][4]+I##ab*(m)[0][4] \
  + I##bp*(m)[4][3]+I##pp*(m)[3][3]+I##cp*(m)[2][3]+I##np*(m)[1][3]+I##ap*(m)[0][3] \
  + I##bc*(m)[4][2]+I##pc*(m)[3][2]+I##cc*(m)[2][2]+I##nc*(m)[1][2]+I##ac*(m)[0][2] \
  + I##bn*(m)[4][1]+I##pn*(m)[3][1]+I##cn*(m)[2][1]+I##nn*(m)[1][1]+I##an*(m)[0][1] \
  + I##ba*(m)[4][0]+I##pa*(m)[3][0]+I##ca*(m)[2][0]+I##na*(m)[1][0]+I##aa*(m)[0][0] )
#define cimg_corr3x3x3(I,m) ( I##ppp*(m)[0][0][0]+I##cpp*(m)[1][0][0]+I##npp*(m)[2][0][0] \
  + I##pcp*(m)[0][1][0]+I##ccp*(m)[1][1][0]+I##ncp*(m)[2][1][0] \
  + I##pnp*(m)[0][2][0]+I##cnp*(m)[1][2][0]+I##nnp*(m)[2][2][0] \
  + I##ppc*(m)[0][0][1]+I##cpc*(m)[1][0][1]+I##npc*(m)[2][0][1] \
  + I##pcc*(m)[0][1][1]+I##ccc*(m)[1][1][1]+I##ncc*(m)[2][1][1] \
  + I##pnc*(m)[0][2][1]+I##cnc*(m)[1][2][1]+I##nnc*(m)[2][2][1] \
  + I##ppn*(m)[0][0][2]+I##cpn*(m)[1][0][2]+I##npn*(m)[2][0][2] \
  + I##pcn*(m)[0][1][2]+I##ccn*(m)[1][1][2]+I##ncn*(m)[2][1][2] \
  + I##pnn*(m)[0][2][2]+I##cnn*(m)[1][2][2]+I##nnn*(m)[2][2][2] )
#define cimg_conv3x3x3(I,m) ( I##ppp*(m)[2][2][2]+I##cpp*(m)[1][2][2]+I##npp*(m)[0][2][2] \
  + I##pcp*(m)[2][1][2]+I##ccp*(m)[1][1][2]+I##ncp*(m)[0][1][2] \
  + I##pnp*(m)[2][0][2]+I##cnp*(m)[1][0][2]+I##nnp*(m)[0][0][2] \
  + I##ppc*(m)[2][2][1]+I##cpc*(m)[1][2][1]+I##npc*(m)[0][2][1] \
  + I##pcc*(m)[2][1][1]+I##ccc*(m)[1][1][1]+I##ncc*(m)[0][1][1] \
  + I##pnc*(m)[2][0][1]+I##cnc*(m)[1][0][1]+I##nnc*(m)[0][0][1] \
  + I##ppn*(m)[2][2][0]+I##cpn*(m)[1][2][0]+I##npn*(m)[0][2][0] \
  + I##pcn*(m)[2][1][0]+I##ccn*(m)[1][1][0]+I##ncn*(m)[0][1][0] \
  + I##pnn*(m)[2][0][0]+I##cnn*(m)[1][0][0]+I##nnn*(m)[0][0][0] )

#define cimg_get3x3(img,x,y,z,v,I)    I##pp=(img)(p##x,p##y,z,v), I##cp=(img)(x,p##y,z,v), I##np=(img)(n##x,p##y,z,v), \
  I##pc=(img)(p##x,   y,z,v), I##cc=(img)(x,   y,z,v), I##nc=(img)(n##x,   y,z,v), \
  I##pn=(img)(p##x,n##y,z,v), I##cn=(img)(x,n##y,z,v), I##nn=(img)(n##x,n##y,z,v)

#define cimg_get5x5(img,x,y,z,v,I)    I##bb=(img)(b##x,b##y,z,v), I##pb=(img)(p##x,b##y,z,v), I##cb=(img)(x,b##y,z,v), I##nb=(img)(n##x,b##y,z,v), I##ab=(img)(a##x,b##y,z,v), \
  I##bp=(img)(b##x,p##y,z,v), I##pp=(img)(p##x,p##y,z,v), I##cp=(img)(x,p##y,z,v), I##np=(img)(n##x,p##y,z,v), I##ap=(img)(a##x,p##y,z,v), \
  I##bc=(img)(b##x,   y,z,v), I##pc=(img)(p##x,   y,z,v), I##cc=(img)(x,   y,z,v), I##nc=(img)(n##x,   y,z,v), I##ac=(img)(a##x,   y,z,v), \
  I##bn=(img)(b##x,n##y,z,v), I##pn=(img)(p##x,n##y,z,v), I##cn=(img)(x,n##y,z,v), I##nn=(img)(n##x,n##y,z,v), I##an=(img)(a##x,n##y,z,v), \
  I##ba=(img)(b##x,a##y,z,v), I##pa=(img)(p##x,a##y,z,v), I##ca=(img)(x,a##y,z,v), I##na=(img)(n##x,a##y,z,v), I##aa=(img)(a##x,a##y,z,v)

#define cimg_get3x3x3(img,x,y,z,v,I)  I##ppp=(img)(p##x,p##y,p##z,v), I##cpp=(img)(x,p##y,p##z,v), I##npp=(img)(n##x,p##y,p##z,v), \
  I##pcp=(img)(p##x,   y,p##z,v), I##ccp=(img)(x,   y,p##z,v), I##ncp=(img)(n##x,   y,p##z,v), \
  I##pnp=(img)(p##x,n##y,p##z,v), I##cnp=(img)(x,n##y,p##z,v), I##nnp=(img)(n##x,n##y,p##z,v), \
  I##ppc=(img)(p##x,p##y,   z,v), I##cpc=(img)(x,p##y,   z,v), I##npc=(img)(n##x,p##y,   z,v), \
  I##pcc=(img)(p##x,   y,   z,v), I##ccc=(img)(x,   y,   z,v), I##ncc=(img)(n##x,   y,   z,v), \
  I##pnc=(img)(p##x,n##y,   z,v), I##cnc=(img)(x,n##y,   z,v), I##nnc=(img)(n##x,n##y,   z,v), \
  I##ppn=(img)(p##x,p##y,n##z,v), I##cpn=(img)(x,p##y,n##z,v), I##npn=(img)(n##x,p##y,n##z,v), \
  I##pcn=(img)(p##x,   y,n##z,v), I##ccn=(img)(x,   y,n##z,v), I##ncn=(img)(n##x,   y,n##z,v), \
  I##pnn=(img)(p##x,n##y,n##z,v), I##cnn=(img)(x,n##y,n##z,v), I##nnn=(img)(n##x,n##y,n##z,v)

#define cimg_3x3to5x5(I,u) u##bb=I##pp,u##cb=I##cp,u##ab=I##np,u##bc=I##pc,u##cc=I##cc,u##ac=I##nc,u##ba=I##pn,u##ca=I##cn,u##aa=I##nn,\
  u##pb=0.5*(u##bb+u##cb),u##nb=0.5*(u##cb+u##ab),u##pc=0.5*(u##bc+u##cc),u##nc=0.5*(u##cc+u##ac),u##pa=0.5*(u##ba+u##ca),u##na=0.5*(u##ca+u##aa),\
  u##bp=0.5*(u##bb+u##bc),u##bn=0.5*(u##bc+u##ba),u##cp=0.5*(u##cb+u##cc),u##cn=0.5*(u##cc+u##ca),u##ap=0.5*(u##ab+u##ac),u##an=0.5*(u##ac+u##aa),\
  u##pp=0.5*(u##bp+u##cp),u##np=0.5*(u##cp+u##ap),u##pn=0.5*(u##bn+u##cn),u##nn=0.5*(u##cn+u##an)
//@}

//@{
//! Macros used to define various image loops
#define cimg_map(img,ptr)         for (ptr=(img).data+(img).size()-1; ptr>=(img).data; ptr--)
#define cimgl_map(list,l)         for (uint l=0; l<(list).size; l++)
#define cimg_mapoff(img,off)      for (uint off=0; off<(img).size(); off++)
#define cimg_mapX(img,x)          for (int x=0; x<(int)((img).width); x++)
#define cimg_mapY(img,y)          for (int y=0; y<(int)((img).height);y++)
#define cimg_mapZ(img,z)          for (int z=0; z<(int)((img).depth); z++)
#define cimg_mapV(img,v)          for (int v=0; v<(int)((img).dim);   v++)
#define cimg_mapXY(img,x,y)       cimg_mapY(img,y) cimg_mapX(img,x)
#define cimg_mapXZ(img,x,z)       cimg_mapZ(img,z) cimg_mapX(img,x)
#define cimg_mapYZ(img,y,z)       cimg_mapZ(img,z) cimg_mapY(img,y)
#define cimg_mapXV(img,x,v)       cimg_mapV(img,v) cimg_mapX(img,x)
#define cimg_mapYV(img,y,v)       cimg_mapV(img,v) cimg_mapY(img,y)
#define cimg_mapZV(img,z,v)       cimg_mapV(img,v) cimg_mapZ(img,z)
#define cimg_mapXYZ(img,x,y,z)    cimg_mapZ(img,z) cimg_mapXY(img,x,y)
#define cimg_mapXYV(img,x,y,v)    cimg_mapV(img,v) cimg_mapXY(img,x,y)
#define cimg_mapXZV(img,x,z,v)    cimg_mapV(img,v) cimg_mapXZ(img,x,z)
#define cimg_mapYZV(img,y,z,v)    cimg_mapV(img,v) cimg_mapYZ(img,y,z)
#define cimg_mapXYZV(img,x,y,z,v) cimg_mapV(img,v) cimg_mapXYZ(img,x,y,z)
#define cimg_imapX(img,x,n)       for (int x=n; x<(int)((img).width-n); x++)
#define cimg_imapY(img,y,n)       for (int y=n; y<(int)((img).height-n); y++)
#define cimg_imapZ(img,z,n)       for (int z=n; z<(int)((img).depth-n); z++)
#define cimg_imapV(img,v,n)       for (int v=n; v<(int)((img).dim-n); v++)
#define cimg_imapXY(img,x,y,n)    cimg_imapY(img,y,n) cimg_imapX(img,x,n)
#define cimg_imapXYZ(img,x,y,z,n) cimg_imapZ(img,z,n) cimg_imapXY(img,x,y,n)
#define cimg_bmapX(img,x,n)       for (int x=0; x<(int)((img).width);  x==(n)-1?(x=(img).width-(n)): x++)
#define cimg_bmapY(img,y,n)       for (int y=0; y<(int)((img).height); y==(n)-1?(x=(img).height-(n)):y++)
#define cimg_bmapZ(img,z,n)       for (int z=0; z<(int)((img).depth);  z==(n)-1?(x=(img).depth-(n)): z++)
#define cimg_bmapV(img,v,n)       for (int v=0; v<(int)((img).dim);    v==(n)-1?(x=(img).dim-(n)):   v++)
#define cimg_bmapXY(img,x,y,n)    cimg_mapY(img,y) for (int x=0; x<(int)((img).width); (y<(n) || y>=(int)((img).height)-(n))?x++:\
  ((x<(n)-1 || x>=(int)((img).width)-(n))?x++:(x=(img).width-(n))))
#define cimg_bmapXYZ(img,x,y,z,n) cimg_mapYZ(img,y,z) for (int x=0; x<(int)((img).width); (y<(n) || y>=(int)((img).height)-(n) || z<(n) || z>=(int)((img).depth)-(n))?x++:\
  ((x<(n)-1 || x>=(int)((img).width)-(n))?x++:(x=(img).width-(n))))
#define cimg_3mapX(img,x)         for (int x=0,p##x=0,n##x=1; n##x<(int)((img).width)  || x==--n##x;  p##x=x++,n##x++)
#define cimg_3mapY(img,y)         for (int y=0,p##y=0,n##y=1; n##y<(int)((img).height) || y==--n##y;  p##y=y++,n##y++)
#define cimg_3mapZ(img,z)         for (int z=0,p##z=0,n##z=1; n##z<(int)((img).depth)  || z==--n##z;  p##z=z++,n##z++)
#define cimg_3mapXY(img,x,y)      cimg_3mapY(img,y) cimg_3mapX(img,x)
#define cimg_3mapXZ(img,x,z)      cimg_3mapZ(img,z) cimg_3mapX(img,x)
#define cimg_3mapYZ(img,y,z)      cimg_3mapZ(img,z) cimg_3mapY(img,y)
#define cimg_3mapXYZ(img,x,y,z)   cimg_3mapZ(img,z) cimg_3mapXY(img,x,y)
#define cimg_5mapX(img,x)         for (int b##x=0,p##x=0,x=0,n##x=1,a##x=2; a##x<(int)((img).width)  || n##x==--a##x || x==(a##x=--n##x); b##x=p##x,p##x=x++,n##x++,a##x++)
#define cimg_5mapY(img,y)         for (int b##y=0,p##y=0,y=0,n##y=1,a##y=2; a##y<(int)((img).height) || n##y==--a##y || y==(a##y=--n##y); b##y=p##y,p##y=y++,n##y++,a##y++)
#define cimg_5mapZ(img,z)         for (int b##z=0,p##z=0,z=0,n##z=1,a##z=2; a##z<(int)((img).depth)  || n##z==--a##z || z==(a##z=--n##z); b##z=p##z,p##z=z++,n##z++,a##z++)
#define cimg_5mapXY(img,x,y)      cimg_5mapY(img,y) cimg_5mapX(img,x)
#define cimg_5mapXZ(img,x,z)      cimg_5mapZ(img,z) cimg_5mapX(img,x)
#define cimg_5mapYZ(img,y,z)      cimg_5mapZ(img,z) cimg_5mapY(img,y)
#define cimg_5mapXYZ(img,x,y,z)   cimg_5mapZ(img,z) cimg_5mapXY(img,x,y)

#define cimg_map3x3(img,x,y,z,v,I) cimg_3mapY(img,y) for (int n##x=1, p##x=(int)( \
  I##cp=I##pp=(img)(0,p##y,z,v), I##cc=I##pc=(img)(0,y,z,v), I##cn=I##pn=(img)(0,n##y,z,v)), \
  x=p##x=0; \
  (n##x<(int)((img).width) && ( \
  I##np=(img)(n##x,p##y,z,v), I##nc=(img)(n##x,y,z,v), I##nn=(img)(n##x,n##y,z,v),\
  1)) || x==--n##x; \
  I##pp=I##cp,I##pc=I##cc,I##pn=I##cn,I##cp=I##np,I##cc=I##nc,I##cn=I##nn, \
  p##x=x++,n##x++ )

#define cimg_map5x5(img,x,y,z,v,I) cimg_5mapY(img,y) for (int a##x=2, n##x=1, b##x=(int)( \
  I##cb=I##pb=I##bb=(img)(0,b##y,z,v), I##cp=I##pp=I##bp=(img)(0,p##y,z,v), I##cc=I##pc=I##bc=(img)(0,y,z,v), \
  I##cn=I##pn=I##bn=(img)(0,n##y,z,v), I##ca=I##pa=I##ba=(img)(0,a##y,z,v), \
  I##nb=(img)(n##x,b##y,z,v), I##np=(img)(n##x,p##y,z,v), I##nc=(img)(n##x,y,z,v), I##nn=(img)(n##x,n##y,z,v), I##na=(img)(n##x,a##y,z,v)), \
  x=0, p##x=b##x=0; \
  (a##x<(int)((img).width) && ( \
  I##ab=(img)(a##x,b##y,z,v), I##ap=(img)(a##x,p##y,z,v), I##ac=(img)(a##x,y,z,v), I##an=(img)(a##x,n##y,z,v), I##aa=(img)(a##x,a##y,z,v), \
  1)) || n##x==--a##x || x==(a##x=--n##x); \
  I##bb=I##pb,I##bp=I##pp,I##bc=I##pc,I##bn=I##pn,I##ba=I##pa,I##pb=I##cb,I##pp=I##cp,I##pc=I##cc,I##pn=I##cn,I##pa=I##ca, \
  I##cb=I##nb,I##cp=I##np,I##cc=I##nc,I##cn=I##nn,I##ca=I##na,I##nb=I##ab,I##np=I##ap,I##nc=I##ac,I##nn=I##an,I##na=I##aa, \
  b##x=p##x, p##x=x++, n##x++, a##x++ )

#define cimg_map3x3x3(img,x,y,z,v,I) cimg_3mapYZ(img,y,z) for (int n##x=1, p##x=(int)( \
  I##cpp=I##ppp=(img)(0,p##y,p##z,v), I##ccp=I##pcp=(img)(0,y,p##z,v), I##cnp=I##pnp=(img)(0,n##y,p##z,v), \
  I##cpc=I##ppc=(img)(0,p##y,   z,v), I##ccc=I##pcc=(img)(0,y,   z,v), I##cnc=I##pnc=(img)(0,n##y,   z,v), \
  I##cpn=I##ppn=(img)(0,p##y,n##z,v), I##ccn=I##pcn=(img)(0,y,n##z,v), I##cnn=I##pnn=(img)(0,n##y,n##z,v)), \
  x=p##x=0; \
  (n##x<(int)((img).width) && ( \
  I##npp=(img)(n##x,p##y,p##z,v), I##ncp=(img)(n##x,y,p##z,v), I##nnp=(img)(n##x,n##y,p##z,v), \
  I##npc=(img)(n##x,p##y,   z,v), I##ncc=(img)(n##x,y,   z,v), I##nnc=(img)(n##x,n##y,   z,v), \
  I##npn=(img)(n##x,p##y,n##z,v), I##ncn=(img)(n##x,y,n##z,v), I##nnn=(img)(n##x,n##y,n##z,v), \
  1)) || x==--n##x; \
  I##ppp=I##cpp,I##pcp=I##ccp,I##pnp=I##cnp,I##cpp=I##npp,I##ccp=I##ncp,I##cnp=I##nnp, \
  I##ppc=I##cpc,I##pcc=I##ccc,I##pnc=I##cnc,I##cpc=I##npc,I##ccc=I##ncc,I##cnc=I##nnc, \
  I##ppn=I##cpn,I##pcn=I##ccn,I##pnn=I##cnn,I##cpn=I##npn,I##ccn=I##ncn,I##cnn=I##nnn, \
  p##x=x++, n##x++ )
//@}
// Macros used to create inline code in some functions
#define cimg_deriche_map(x0,y0,z0,k0,nb,offset,T) { \
  ima = ptr(x0,y0,z0,k0); \
  I2 = *ima; ima+=offset; I1 = *ima; ima+=offset; \
  Y2 = *(Y++) = sumg0*I2; Y1 = *(Y++) = g0*I1 + sumg1*I2; \
  for (i=2; i<(nb); i++) { I1 = *ima; ima+=offset; Y0 = *(Y++) = a1*I1 + a2*I2 + b1*Y1 + b2*Y2; I2=I1; Y2=Y1; Y1=Y0; } \
    ima-=offset; I2 = *ima; Y2 = Y1 = parity*sumg1*I2; *ima = (T)(*(--Y)+Y2); \
    ima-=offset; I1 = *ima; *ima = (T)(*(--Y)+Y1); \
    for (i=(nb)-3; i>=0; i--) { Y0=a3*I1+a4*I2+b1*Y1+b2*Y2; ima-=offset; I2=I1; I1=*ima; *ima=(T)(*(--Y)+Y0); Y2=Y1; Y1=Y0; } \
}
#define cimg_inr_case(Tf,sign,pixsize,Ts,Td) \
  if (fopt[6]==pixsize && fopt[4]==Tf && fopt[5]==sign) \
  { \
    Ts *xval, *val = new Ts[fopt[0]*fopt[3]]; \
    const int cimg_endian = cimg::CPU_endian(); \
    cimg_mapYZ(dest,y,z) \
    { \
      cimg::fread(val,pixsize/8,fopt[0]*fopt[3],file); \
        if (fopt[7]!=cimg_endian) cimg::bswap(val,fopt[0]*fopt[3],pixsize/8); \
        xval = val; cimg_mapX(dest,x) cimg_mapV(dest,k) dest(x,y,z,k) = (Td)*(xval++); \
    } \
    delete[] val; \
  } else
#define cimg_raw_case(Ts) \
  if (!cimg::strcasecmp(#Ts,tmp2)) for (uint l=0; l<n; l++) \
  { \
    Ts *buf; \
    const bool cimg_endian = cimg::CPU_endian(); \
    j=0; while((i=fgetc(file))!='\n') tmp[j++]=(char)i; tmp[j]='\0'; std::sscanf(tmp,"%d %d %d %d",&w,&h,&z,&k); \
    buf = new Ts[w*h*z*k]; cimg::fread(buf,sizeof(Ts),w*h*z*k,file); \
    if (!cimg_endian) cimg::bswap(buf,w*h*z*k,sizeof(Ts)); \
      CImg<T> idest(w,h,z,k); cimg_mapoff(idest,off) idest[off] = (T)(buf[off]); idest.swap(res[l]); \
      delete[] buf; \
  } else


/*----------------------------------------



  Definition of the namespace "cimg"



  --------------------------------------*/

//! This namespace some low-level functions and variables of the CImg library.
namespace cimg
{
#if cimg_OS==0 || cimg_OS==1
  // Keycodes for Unix & Solaris
  const uint keyESC        = 9;
  const uint keyF1         = 67;
  const uint keyF2         = 68;
  const uint keyF3         = 69;
  const uint keyF4         = 70;
  const uint keyF5         = 71;
  const uint keyF6         = 72;
  const uint keyF7         = 73;
  const uint keyF8         = 74;
  const uint keyF9         = 75;
  const uint keyF10        = 76;
  const uint keyF11        = 95;
  const uint keyF12        = 96;
  const uint keyPAUSE      = 110;
  const uint key1          = 10;
  const uint key2          = 11;
  const uint key3          = 12;
  const uint key4          = 13;
  const uint key5          = 14;
  const uint key6          = 15;
  const uint key7          = 16;
  const uint key8          = 17;
  const uint key9          = 18;
  const uint key0          = 19;
  const uint keyBACKSPACE  = 22;
  const uint keyINSERT     = 106;
  const uint keyHOME       = 97;
  const uint keyPAGEUP     = 99;
  const uint keyTAB        = 23;
  const uint keyQ          = 24;
  const uint keyW          = 25;
  const uint keyE          = 26;
  const uint keyR          = 27;
  const uint keyT          = 28;
  const uint keyY          = 29;
  const uint keyU          = 30;
  const uint keyI          = 31;
  const uint keyO          = 32;
  const uint keyP          = 33;
  const uint keyDELETE     = 107;
  const uint keyEND        = 103;
  const uint keyPAGEDOWN   = 105;
  const uint keyCAPSLOCK   = 66;
  const uint keyA          = 38;
  const uint keyS          = 39;
  const uint keyD          = 40;
  const uint keyF          = 41;
  const uint keyG          = 42;
  const uint keyH          = 43;
  const uint keyJ          = 44;
  const uint keyK          = 45;
  const uint keyL          = 46;
  const uint keyENTER      = 36;
  const uint keySHIFTLEFT  = 50;
  const uint keyZ          = 52;
  const uint keyX          = 53;
  const uint keyC          = 54;
  const uint keyV          = 55;
  const uint keyB          = 56;
  const uint keyN          = 57;
  const uint keyM          = 58;
  const uint keySHIFTRIGHT = 62;
  const uint keyARROWUP    = 98;
  const uint keyCTRLLEFT   = 37;
  const uint keyAPPLEFT    = 115;
  const uint keySPACE      = 65;
  const uint keyALTGR      = 113;
  const uint keyAPPRIGHT   = 116;
  const uint keyMENU       = 117;
  const uint keyCTRLRIGHT  = 109;
  const uint keyARROWLEFT  = 100;
  const uint keyARROWDOWN  = 104;
  const uint keyARROWRIGHT = 102;

#elif cimg_OS==3 
  // Keycodes for MacOS X
  const uint keyESC        = 61;
  const uint keyF1         = 130;
  const uint keyF2         = 128;
  const uint keyF3         = 107;
  const uint keyF4         = 126;
  const uint keyF5         = 104;
  const uint keyF6         = 105;
  const uint keyF7         = 106;
  const uint keyF8         = 108;
  const uint keyF9         = 109;
  const uint keyF10        = 117;
  const uint keyF11        = 111;
  const uint keyF12        = 119;
  const uint keyPAUSE      = 110;
  const uint key1          = 26;
  const uint key2          = 27;
  const uint key3          = 28;
  const uint key4          = 29;
  const uint key5          = 31;
  const uint key6          = 30;
  const uint key7          = 34;
  const uint key8          = 36;
  const uint key9          = 33;
  const uint key0          = 37;
  const uint keyBACKSPACE  = 59;
  const uint keyINSERT     = 122;
  const uint keyHOME       = 123;
  const uint keyPAGEUP     = 124;
  const uint keyTAB        = 56;
  const uint keyA          = 20;
  const uint keyZ          = 21;
  const uint keyE          = 22;
  const uint keyR          = 23;
  const uint keyT          = 25;
  const uint keyY          = 24;
  const uint keyU          = 40;
  const uint keyI          = 42;
  const uint keyO          = 39;
  const uint keyP          = 43;
  const uint keyDELETE     = 125;
  const uint keyEND        = 127;
  const uint keyPAGEDOWN   = 129;
  const uint keyCAPSLOCK   = 65;
  const uint keyQ          = 8;
  const uint keyS          = 9;
  const uint keyD          = 10;
  const uint keyF          = 11;
  const uint keyG          = 13;
  const uint keyH          = 12;
  const uint keyJ          = 46;
  const uint keyK          = 48;
  const uint keyL          = 45;
  const uint keyM          = 49;
  const uint keyENTER      = 44;
  const uint keySHIFTLEFT  = 64;
  const uint keyW          = 14;
  const uint keyX          = 15;
  const uint keyC          = 16;
  const uint keyV          = 17;
  const uint keyB          = 19;
  const uint keyN          = 53;
  const uint keySHIFTRIGHT = 131;
  const uint keyARROWUP    = 70;
  const uint keyCTRLLEFT   = 62;
  const uint keyAPPLEFT    = 63;
  const uint keySPACE      = 57;
  const uint keyALTGR      = 132;
  const uint keyAPPRIGHT   = 134;
  const uint keyMENU       = 117;
  const uint keyCTRLRIGHT  = 133;
  const uint keyARROWLEFT  = 67;
  const uint keyARROWDOWN  = 69;
  const uint keyARROWRIGHT = 68;

#else
  // Keycodes for Windows-OS
  //@{
  //!\name Keycodes.
  //! Keycodes are used to detect keyboard events occuring on display windows \c CImgDisplay.
  //! The field \c key of the \c CImgDisplay structure is updated at real-time with the corresponding keycode
  //! of the pressed key (or 0 if no key has been pressed). The keycodes values are given by the variables
  //! whose names are of the form <tt>cimg::key*</tt>. Above is the keycode for the 'ESC' key, but
  //! almost all keycodes thus are defined.
  const uint keyESC        = 27;
  const uint keyF1         = 112;
  const uint keyF2         = 113;
  const uint keyF3         = 114;
  const uint keyF4         = 115;
  const uint keyF5         = 116;
  const uint keyF6         = 117;
  const uint keyF7         = 118;
  const uint keyF8         = 119;
  const uint keyF9         = 120;
  const uint keyF10        = 121;
  const uint keyF11        = 122;
  const uint keyF12        = 123;
  const uint keyPAUSE      = 19;
  const uint key1          = 49;
  const uint key2          = 50;
  const uint key3          = 51;
  const uint key4          = 52;
  const uint key5          = 53;
  const uint key6          = 54;
  const uint key7          = 55;
  const uint key8          = 56;
  const uint key9          = 57;
  const uint key0          = 48;
  const uint keyBACKSPACE  = 8;
  const uint keyINSERT     = 45;
  const uint keyHOME       = 36;
  const uint keyPAGEUP     = 33;
  const uint keyTAB        = 9;
  const uint keyQ          = 81;
  const uint keyW          = 87;
  const uint keyE          = 69;
  const uint keyR          = 82;
  const uint keyT          = 84;
  const uint keyY          = 89;
  const uint keyU          = 85;
  const uint keyI          = 73;
  const uint keyO          = 79;
  const uint keyP          = 80;
  const uint keyDELETE     = 8;
  const uint keyEND        = 35;
  const uint keyPAGEDOWN   = 34;
  const uint keyCAPSLOCK   = 20;
  const uint keyA          = 65;
  const uint keyS          = 83;
  const uint keyD          = 68;
  const uint keyF          = 70;
  const uint keyG          = 71;
  const uint keyH          = 72;
  const uint keyJ          = 74;
  const uint keyK          = 75;
  const uint keyL          = 76;
  const uint keyENTER      = 13;
  const uint keySHIFTLEFT  = 16;
  const uint keyZ          = 90;
  const uint keyX          = 88;
  const uint keyC          = 67;
  const uint keyV          = 86;
  const uint keyB          = 66;
  const uint keyN          = 78;
  const uint keyM          = 77;
  const uint keySHIFTRIGHT = 16;
  const uint keyARROWUP    = 38;
  const uint keyCTRLLEFT   = 17;
  const uint keyAPPLEFT    = 91;
  const uint keySPACE      = 32;
  const uint keyALTGR      = 17;
  const uint keyAPPRIGHT   = 92;
  const uint keyMENU       = 93;
  const uint keyCTRLRIGHT  = 17;
  const uint keyARROWLEFT  = 37;
  const uint keyARROWDOWN  = 40;
  const uint keyARROWRIGHT = 39;
#endif
  //@}
  //! Definition of the mathematical constant PI
  const double PI = 3.14159265358979323846;
  const uint lblock = 1024;
  
#ifdef cimg_color_term
  const char t_normal[9]  = {0x1b,'[','0',';','0',';','0','m','\0'};
  const char t_red[11]    = {0x1b,'[','4',';','3','1',';','5','9','m','\0'};
  const char t_bold[5]    = {0x1b,'[','1','m','\0'};
  const char t_purple[11] = {0x1b,'[','0',';','3','5',';','5','9','m','\0'};
#else
  const char t_normal[1]  = {'\0'};
  static const char *t_red = t_normal, *t_bold = t_normal, *t_purple = t_normal;
#endif
  
  // Definition of a 7x11x1x3 font, used to return a default CImgl font for drawing text.
  const uint font7x11[7*11*256/8] = {0x00000000,
      0x00000000,0x00000000,0x00000002,0x04081020,0x00800000,0x24489000,0x00000000,0x000000a1,
      0x4f8a7e50,0xa0000002,0x0f287030,0x50a78200,0x00008695,0x454552c2,0x00000002,0x0a143193,
      0x19f00000,0x04081000,0x00000000,0x00001841,0x02040810,0x20203006,0x02020408,0x1020410c,
      0x000010d8,0xc1400000,0x00000000,0x01021f08,0x10200000,0x00000000,0x00018302,0x08000000,
      0x0007c000,0x00000000,0x00000000,0x0060c000,0x0000820c,0x1040820c,0x10400000,0xe2244891,
      0x22380000,0x00061408,0x102043e0,0x00000070,0x10208208,0x1e000000,0x03c0810c,0x0408e000,
      0x0000020c,0x2891f040,0x80000000,0xf1020701,0x02380000,0x0001c410,0x2c6488e0,0x0000003e,
      0x08104102,0x08000000,0x00e22487,0x11223800,0x00000711,0x223c0823,0x80000000,0x0060c000,
      0x060c0000,0x00000306,0x00003060,0x41000000,0x0218c180,0xc0400000,0x000007f0,0x1fc00000,
      0x00000010,0x180c18c2,0x00000000,0x3c440820,0x80020000,0x0000f229,0xd4afa038,0x00000002,
      0x0a1444f9,0x14100000,0x00788913,0xc4489e00,0x000001e4,0x10204040,0x78000000,0x1e224489,
      0x12278000,0x0000f902,0x0788103e,0x00000007,0xc8103c40,0x81000000,0x001e4102,0x34244780,
      0x00000112,0x244f9122,0x44000000,0x0f840810,0x2043e000,0x00003810,0x2040811c,0x00000002,
      0x248a1828,0x48880000,0x00102040,0x810207c0,0x0000019b,0x36ab56a9,0x42000000,0x044c9d2a,
      0x4c991000,0x00003c85,0x0a14284f,0x00000001,0xe2244f10,0x20400000,0x000f2142,0x850a13c0,
      0xc0c000f1,0x12278911,0x21000000,0x01e4080e,0x0204f000,0x00003f88,0x10204081,0x00000000,
      0x89122448,0x911c0000,0x00082891,0x22285040,0x00000041,0x9325ab64,0xc8800000,0x020a2282,
      0x0a228200,0x00001051,0x14102040,0x80000000,0xfc082082,0x083f0000,0x00e10204,0x08102040,
      0x81c01010,0x20202040,0x40808080,0x70204081,0x02040810,0xe0008105,0x1b228200,0x00000000,
      0x00000000,0x00003f80,0x20200000,0x00000000,0x00000000,0x0e023c89,0x11f00000,0x2040b192,
      0x24489e00,0x00000003,0xc8102040,0x78000000,0x811e4489,0x1223c000,0x000000e2,0x27c8101e,
      0x00000071,0x0fc40810,0x20400000,0x00003c89,0x12244781,0x1c008102,0xe6489122,0x44000001,
      0x001c0810,0x20408000,0x00040070,0x20408102,0x04700102,0x04491c28,0x48880000,0x0e040810,
      0x20408100,0x00000002,0xded93264,0xc9000000,0x000b9922,0x44891000,0x00000038,0x89122447,
      0x00000000,0x02c64891,0x22788100,0x00000f22,0x448911e0,0x40800000,0xb9920408,0x10000000,
      0x00078818,0x0c08e000,0x0000083e,0x20408101,0xc0000000,0x01122448,0x933a0000,0x00001051,
      0x22285040,0x00000000,0x8326ad56,0xc8800000,0x00042486,0x0c248400,0x00000020,0xa24450a0,
      0x831c0000,0x00f81041,0x041f0000,0x00308102,0x0c081020,0x40600204,0x08102040,0x81020400,
      0x60204081,0x82040810,0xe0000000,0x00399c00,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04001020,
      0x40810000,0x00107142,0x85070400,0x000000c2,0x041c1020,0xf0000000,0x213c4891,0xe4200000,
      0x00010511,0x47c21f08,0x00000081,0x02000000,0x00408100,0x001c40c1,0x62428302,0x38480000,
      0x00000000,0x00000000,0x0f215aa5,0x6a13c000,0x00007871,0x23e00000,0x00000000,0x00028a28,
      0x28280000,0x00000001,0xf8102000,0x00000000,0x0007c000,0x00000000,0x07113a44,0x70000000,
      0x1fc00000,0x00000000,0x00000000,0x41410000,0x00000000,0x00000408,0x7c2043e0,0x0000003c,
      0x30f00000,0x00000000,0x01c10700,0x00000000,0x00820000,0x00000000,0x00000000,0x01122448,
      0x933a4080,0x0007ce9d,0x1a142850,0xa1400000,0x0000c180,0x00000000,0x00000000,0x00000010,
      0x60000604,0x08000000,0x00000000,0x30912180,0x00000000,0x0000000a,0x0a0a28a0,0x00000032,
      0x28505952,0xf8400000,0x01914282,0xe8514700,0x00000e38,0xba0e34be,0x10000000,0x00100040,
      0x8208111e,0x40404142,0x889f2282,0x00004102,0x0a1444f9,0x1410000c,0x241050a2,0x27c8a080,
      0x0028a082,0x85113e45,0x04000480,0x04142889,0xf2282000,0x082820a1,0x444f9141,0x00000001,
      0xc60c2c50,0xe2700000,0x001e4102,0x04040782,0x0c2021f2,0x040f1020,0x7c000020,0x8f902078,
      0x8103e000,0x0c247c81,0x03c4081f,0x00009003,0xe4081e20,0x40f80002,0x021f0810,0x204087c0,
      0x000410f8,0x40810204,0x3e0000c2,0x47c20408,0x1021f000,0x09003e10,0x2040810f,0x80000001,
      0xe2245c91,0x22780000,0xa288993a,0x54993220,0x00080879,0x0a142850,0x9e000010,0x43c850a1,
      0x4284f000,0x03091e42,0x850a1427,0x80000a28,0xf2142850,0xa13c0001,0x200790a1,0x428509e0,
      0x00000000,0x8490c184,0x90800000,0x01f66954,0xa966f800,0x01010891,0x22448911,0xc0000104,
      0x44891224,0x488e0000,0x61222448,0x91224470,0x00048011,0x22448912,0x23800002,0x09051141,
      0x02040800,0x0000040f,0x112244f1,0x0000000e,0x2448a142,0x444b8000,0x202001c0,0x4791223e,
      0x00004100,0x0e023c89,0x11f0000c,0x24007011,0xe4488f80,0x0028a003,0x808f2244,0x7c000005,
      0x001c0479,0x1223e000,0x082820e0,0x23c8911f,0x00000000,0x1d853e91,0x21b00000,0x00003c81,
      0x02040782,0x0c202001,0xc44f9020,0x3c000041,0x000e227c,0x8101e000,0x0c240071,0x13e4080f,
      0x000000a0,0x03889f20,0x40780002,0x02003810,0x20408100,0x00041001,0xc0810204,0x080000c2,
      0x400e0408,0x10204000,0x000a0070,0x20408102,0x000000e1,0x21c44891,0x22380000,0xa2801732,
      0x44891220,0x00080800,0x71122448,0x8e000020,0x80038891,0x22447000,0x0309001c,0x44891223,
      0x80000a28,0x00e22448,0x911c0000,0x01400711,0x224488e0,0x00000000,0x2003f000,0x04000000,
      0x0001e4ca,0x99327800,0x01010011,0x22448933,0xa0000410,0x00891224,0x499d0000,0x61200448,
      0x91224ce8,0x00000500,0x22448912,0x67400004,0x10020a24,0x450a0831,0xc002040b,0x19224489,
      0xe204000a,0x00828911,0x42820c70,0x00000000,0x00000000,0x00000002,0x04081020,0x00800000,
      0x24489000,0x00000000,0x000000a1,0x4f8a7e50,0xa0000002,0x0f287030,0x50a78200,0x00008695,
      0x454552c2,0x00000002,0x0a143193,0x19f00000,0x04081000,0x00000000,0x00001841,0x02040810,
      0x20203006,0x02020408,0x1020410c,0x000010d8,0xc1400000,0x00000000,0x01021f08,0x10200000,
      0x00000000,0x00018302,0x08000000,0x0007c000,0x00000000,0x00000000,0x0060c000,0x0000820c,
      0x1040820c,0x10400000,0xe2244891,0x22380000,0x00061408,0x102043e0,0x00000070,0x10208208,
      0x1e000000,0x03c0810c,0x0408e000,0x0000020c,0x2891f040,0x80000000,0xf1020701,0x02380000,
      0x0001c410,0x2c6488e0,0x0000003e,0x08104102,0x08000000,0x00e22487,0x11223800,0x00000711,
      0x223c0823,0x80000000,0x0060c000,0x060c0000,0x00000306,0x00003060,0x41000000,0x0218c180,
      0xc0400000,0x000007f0,0x1fc00000,0x00000010,0x180c18c2,0x00000000,0x3c440820,0x80020000,
      0x0000f229,0xd4afa038,0x00000002,0x0a1444f9,0x14100000,0x00788913,0xc4489e00,0x000001e4,
      0x10204040,0x78000000,0x1e224489,0x12278000,0x0000f902,0x0788103e,0x00000007,0xc8103c40,
      0x81000000,0x001e4102,0x34244780,0x00000112,0x244f9122,0x44000000,0x0f840810,0x2043e000,
      0x00003810,0x2040811c,0x00000002,0x248a1828,0x48880000,0x00102040,0x810207c0,0x0000019b,
      0x36ab56a9,0x42000000,0x044c9d2a,0x4c991000,0x00003c85,0x0a14284f,0x00000001,0xe2244f10,
      0x20400000,0x000f2142,0x850a13c0,0xc0c000f1,0x12278911,0x21000000,0x01e4080e,0x0204f000,
      0x00003f88,0x10204081,0x00000000,0x89122448,0x911c0000,0x00082891,0x22285040,0x00000041,
      0x9325ab64,0xc8800000,0x020a2282,0x0a228200,0x00001051,0x14102040,0x80000000,0xfc082082,
      0x083f0000,0x00e10204,0x08102040,0x81c01010,0x20202040,0x40808080,0x70204081,0x02040810,
      0xe0008105,0x1b228200,0x00000000,0x00000000,0x00003f80,0x20200000,0x00000000,0x00000000,
      0x0e023c89,0x11f00000,0x2040b192,0x24489e00,0x00000003,0xc8102040,0x78000000,0x811e4489,
      0x1223c000,0x000000e2,0x27c8101e,0x00000071,0x0fc40810,0x20400000,0x00003c89,0x12244781,
      0x1c008102,0xe6489122,0x44000001,0x001c0810,0x20408000,0x00040070,0x20408102,0x04700102,
      0x04491c28,0x48880000,0x0e040810,0x20408100,0x00000002,0xded93264,0xc9000000,0x000b9922,
      0x44891000,0x00000038,0x89122447,0x00000000,0x02c64891,0x22788100,0x00000f22,0x448911e0,
      0x40800000,0xb9920408,0x10000000,0x00078818,0x0c08e000,0x0000083e,0x20408101,0xc0000000,
      0x01122448,0x933a0000,0x00001051,0x22285040,0x00000000,0x8326ad56,0xc8800000,0x00042486,
      0x0c248400,0x00000020,0xa24450a0,0x831c0000,0x00f81041,0x041f0000,0x00308102,0x0c081020,
      0x40600204,0x08102040,0x81020400,0x60204081,0x82040810,0xe0000000,0x00399c00,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x04001020,0x40810000,0x00107142,0x85070400,0x000000c2,0x041c1020,
      0xf0000000,0x213c4891,0xe4200000,0x00010511,0x47c21f08,0x00000081,0x02000000,0x00408100,
      0x001c40c1,0x62428302,0x38480000,0x00000000,0x00000000,0x0f215aa5,0x6a13c000,0x00007871,
      0x23e00000,0x00000000,0x00028a28,0x28280000,0x00000001,0xf8102000,0x00000000,0x0007c000,
      0x00000000,0x07113a44,0x70000000,0x1fc00000,0x00000000,0x00000000,0x41410000,0x00000000,
      0x00000408,0x7c2043e0,0x0000003c,0x30f00000,0x00000000,0x01c10700,0x00000000,0x00820000,
      0x00000000,0x00000000,0x01122448,0x933a4080,0x0007ce9d,0x1a142850,0xa1400000,0x0000c180,
      0x00000000,0x00000000,0x00000010,0x60000604,0x08000000,0x00000000,0x30912180,0x00000000,
      0x0000000a,0x0a0a28a0,0x00000032,0x28505952,0xf8400000,0x01914282,0xe8514700,0x00000e38,
      0xba0e34be,0x10000000,0x00100040,0x8208111e,0x40404142,0x889f2282,0x00004102,0x0a1444f9,
      0x1410000c,0x241050a2,0x27c8a080,0x0028a082,0x85113e45,0x04000480,0x04142889,0xf2282000,
      0x082820a1,0x444f9141,0x00000001,0xc60c2c50,0xe2700000,0x001e4102,0x04040782,0x0c2021f2,
      0x040f1020,0x7c000020,0x8f902078,0x8103e000,0x0c247c81,0x03c4081f,0x00009003,0xe4081e20,
      0x40f80002,0x021f0810,0x204087c0,0x000410f8,0x40810204,0x3e0000c2,0x47c20408,0x1021f000,
      0x09003e10,0x2040810f,0x80000001,0xe2245c91,0x22780000,0xa288993a,0x54993220,0x00080879,
      0x0a142850,0x9e000010,0x43c850a1,0x4284f000,0x03091e42,0x850a1427,0x80000a28,0xf2142850,
      0xa13c0001,0x200790a1,0x428509e0,0x00000000,0x8490c184,0x90800000,0x01f66954,0xa966f800,
      0x01010891,0x22448911,0xc0000104,0x44891224,0x488e0000,0x61222448,0x91224470,0x00048011,
      0x22448912,0x23800002,0x09051141,0x02040800,0x0000040f,0x112244f1,0x0000000e,0x2448a142,
      0x444b8000,0x202001c0,0x4791223e,0x00004100,0x0e023c89,0x11f0000c,0x24007011,0xe4488f80,
      0x0028a003,0x808f2244,0x7c000005,0x001c0479,0x1223e000,0x082820e0,0x23c8911f,0x00000000,
      0x1d853e91,0x21b00000,0x00003c81,0x02040782,0x0c202001,0xc44f9020,0x3c000041,0x000e227c,
      0x8101e000,0x0c240071,0x13e4080f,0x000000a0,0x03889f20,0x40780002,0x02003810,0x20408100,
      0x00041001,0xc0810204,0x080000c2,0x400e0408,0x10204000,0x000a0070,0x20408102,0x000000e1,
      0x21c44891,0x22380000,0xa2801732,0x44891220,0x00080800,0x71122448,0x8e000020,0x80038891,
      0x22447000,0x0309001c,0x44891223,0x80000a28,0x00e22448,0x911c0000,0x01400711,0x224488e0,
      0x00000000,0x2003f000,0x04000000,0x0001e4ca,0x99327800,0x01010011,0x22448933,0xa0000410,
      0x00891224,0x499d0000,0x61200448,0x91224ce8,0x00000500,0x22448912,0x67400004,0x10020a24,
      0x450a0831,0xc002040b,0x19224489,0xe204000a,0x00828911,0x42820c70,0x00000000,0x00000000,
      0x00000002,0x04081020,0x00800000,0x24489000,0x00000000,0x000000a1,0x4f8a7e50,0xa0000002,
      0x0f287030,0x50a78200,0x00008695,0x454552c2,0x00000002,0x0a143193,0x19f00000,0x04081000,
      0x00000000,0x00001841,0x02040810,0x20203006,0x02020408,0x1020410c,0x000010d8,0xc1400000,
      0x00000000,0x01021f08,0x10200000,0x00000000,0x00018302,0x08000000,0x0007c000,0x00000000,
      0x00000000,0x0060c000,0x0000820c,0x1040820c,0x10400000,0xe2244891,0x22380000,0x00061408,
      0x102043e0,0x00000070,0x10208208,0x1e000000,0x03c0810c,0x0408e000,0x0000020c,0x2891f040,
      0x80000000,0xf1020701,0x02380000,0x0001c410,0x2c6488e0,0x0000003e,0x08104102,0x08000000,
      0x00e22487,0x11223800,0x00000711,0x223c0823,0x80000000,0x0060c000,0x060c0000,0x00000306,
      0x00003060,0x41000000,0x0218c180,0xc0400000,0x000007f0,0x1fc00000,0x00000010,0x180c18c2,
      0x00000000,0x3c440820,0x80020000,0x0000f229,0xd4afa038,0x00000002,0x0a1444f9,0x14100000,
      0x00788913,0xc4489e00,0x000001e4,0x10204040,0x78000000,0x1e224489,0x12278000,0x0000f902,
      0x0788103e,0x00000007,0xc8103c40,0x81000000,0x001e4102,0x34244780,0x00000112,0x244f9122,
      0x44000000,0x0f840810,0x2043e000,0x00003810,0x2040811c,0x00000002,0x248a1828,0x48880000,
      0x00102040,0x810207c0,0x0000019b,0x36ab56a9,0x42000000,0x044c9d2a,0x4c991000,0x00003c85,
      0x0a14284f,0x00000001,0xe2244f10,0x20400000,0x000f2142,0x850a13c0,0xc0c000f1,0x12278911,
      0x21000000,0x01e4080e,0x0204f000,0x00003f88,0x10204081,0x00000000,0x89122448,0x911c0000,
      0x00082891,0x22285040,0x00000041,0x9325ab64,0xc8800000,0x020a2282,0x0a228200,0x00001051,
      0x14102040,0x80000000,0xfc082082,0x083f0000,0x00e10204,0x08102040,0x81c01010,0x20202040,
      0x40808080,0x70204081,0x02040810,0xe0008105,0x1b228200,0x00000000,0x00000000,0x00003f80,
      0x20200000,0x00000000,0x00000000,0x0e023c89,0x11f00000,0x2040b192,0x24489e00,0x00000003,
      0xc8102040,0x78000000,0x811e4489,0x1223c000,0x000000e2,0x27c8101e,0x00000071,0x0fc40810,
      0x20400000,0x00003c89,0x12244781,0x1c008102,0xe6489122,0x44000001,0x001c0810,0x20408000,
      0x00040070,0x20408102,0x04700102,0x04491c28,0x48880000,0x0e040810,0x20408100,0x00000002,
      0xded93264,0xc9000000,0x000b9922,0x44891000,0x00000038,0x89122447,0x00000000,0x02c64891,
      0x22788100,0x00000f22,0x448911e0,0x40800000,0xb9920408,0x10000000,0x00078818,0x0c08e000,
      0x0000083e,0x20408101,0xc0000000,0x01122448,0x933a0000,0x00001051,0x22285040,0x00000000,
      0x8326ad56,0xc8800000,0x00042486,0x0c248400,0x00000020,0xa24450a0,0x831c0000,0x00f81041,
      0x041f0000,0x00308102,0x0c081020,0x40600204,0x08102040,0x81020400,0x60204081,0x82040810,
      0xe0000000,0x00399c00,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
      0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04001020,0x40810000,0x00107142,
      0x85070400,0x000000c2,0x041c1020,0xf0000000,0x213c4891,0xe4200000,0x00010511,0x47c21f08,
      0x00000081,0x02000000,0x00408100,0x001c40c1,0x62428302,0x38480000,0x00000000,0x00000000,
      0x0f215aa5,0x6a13c000,0x00007871,0x23e00000,0x00000000,0x00028a28,0x28280000,0x00000001,
      0xf8102000,0x00000000,0x0007c000,0x00000000,0x07113a44,0x70000000,0x1fc00000,0x00000000,
      0x00000000,0x41410000,0x00000000,0x00000408,0x7c2043e0,0x0000003c,0x30f00000,0x00000000,
      0x01c10700,0x00000000,0x00820000,0x00000000,0x00000000,0x01122448,0x933a4080,0x0007ce9d,
      0x1a142850,0xa1400000,0x0000c180,0x00000000,0x00000000,0x00000010,0x60000604,0x08000000,
      0x00000000,0x30912180,0x00000000,0x0000000a,0x0a0a28a0,0x00000032,0x28505952,0xf8400000,
      0x01914282,0xe8514700,0x00000e38,0xba0e34be,0x10000000,0x00100040,0x8208111e,0x40404142,
      0x889f2282,0x00004102,0x0a1444f9,0x1410000c,0x241050a2,0x27c8a080,0x0028a082,0x85113e45,
      0x04000480,0x04142889,0xf2282000,0x082820a1,0x444f9141,0x00000001,0xc60c2c50,0xe2700000,
      0x001e4102,0x04040782,0x0c2021f2,0x040f1020,0x7c000020,0x8f902078,0x8103e000,0x0c247c81,
      0x03c4081f,0x00009003,0xe4081e20,0x40f80002,0x021f0810,0x204087c0,0x000410f8,0x40810204,
      0x3e0000c2,0x47c20408,0x1021f000,0x09003e10,0x2040810f,0x80000001,0xe2245c91,0x22780000,
      0xa288993a,0x54993220,0x00080879,0x0a142850,0x9e000010,0x43c850a1,0x4284f000,0x03091e42,
      0x850a1427,0x80000a28,0xf2142850,0xa13c0001,0x200790a1,0x428509e0,0x00000000,0x8490c184,
      0x90800000,0x01f66954,0xa966f800,0x01010891,0x22448911,0xc0000104,0x44891224,0x488e0000,
      0x61222448,0x91224470,0x00048011,0x22448912,0x23800002,0x09051141,0x02040800,0x0000040f,
      0x112244f1,0x0000000e,0x2448a142,0x444b8000,0x202001c0,0x4791223e,0x00004100,0x0e023c89,
      0x11f0000c,0x24007011,0xe4488f80,0x0028a003,0x808f2244,0x7c000005,0x001c0479,0x1223e000,
      0x082820e0,0x23c8911f,0x00000000,0x1d853e91,0x21b00000,0x00003c81,0x02040782,0x0c202001,
      0xc44f9020,0x3c000041,0x000e227c,0x8101e000,0x0c240071,0x13e4080f,0x000000a0,0x03889f20,
      0x40780002,0x02003810,0x20408100,0x00041001,0xc0810204,0x080000c2,0x400e0408,0x10204000,
      0x000a0070,0x20408102,0x000000e1,0x21c44891,0x22380000,0xa2801732,0x44891220,0x00080800,
      0x71122448,0x8e000020,0x80038891,0x22447000,0x0309001c,0x44891223,0x80000a28,0x00e22448,
      0x911c0000,0x01400711,0x224488e0,0x00000000,0x2003f000,0x04000000,0x0001e4ca,0x99327800,
      0x01010011,0x22448933,0xa0000410,0x00891224,0x499d0000,0x61200448,0x91224ce8,0x00000500,
      0x22448912,0x67400004,0x10020a24,0x450a0831,0xc002040b,0x19224489,0xe204000a,0x00828911,
      0x42820c70};

  // Return stringification of a standart C++ type.
  const char* const bool_st    = "bool";
  const char* const uchar_st   = "uchar";
  const char* const char_st    = "char";
  const char* const ushort_st  = "ushort";
  const char* const short_st   = "short";
  const char* const uint_st    = "uint";
  const char* const int_st     = "int";
  const char* const float_st   = "float";
  const char* const double_st  = "double";
  const char* const unknown_st = "unknown";
  template<typename t> const char* template_type(const t& element) { return unknown_st; }
  inline const char* template_type(const bool&   element) { return bool_st;   }
  inline const char* template_type(const uchar&  element) { return uchar_st;  }
  inline const char* template_type(const char&   element) { return char_st;   }
  inline const char* template_type(const ushort& element) { return ushort_st; }
  inline const char* template_type(const short&  element) { return short_st;  }
  inline const char* template_type(const uint&   element) { return uint_st;   }
  inline const char* template_type(const int&    element) { return int_st;    }
  inline const char* template_type(const float&  element) { return float_st;  }
  inline const char* template_type(const double& element) { return double_st; }
  
  //! Return CPU endian type (true = Little-endian (Intel), false= Big-endian).
  inline const bool CPU_endian() { const int x=1; return ((uchar*)&x)[0]?true:false; }
 
  //! Display informations about CImg compilation variables, on the standart error output (stderr).
  inline void info() {
    std::fprintf(stderr,"\n %sCImg Library%s, compiled %s ( %s ) with the following flags :\n\n",
                 t_red,t_normal,__DATE__,__TIME__);
    std::fprintf(stderr,"  > Architecture   : %s%-12s%s %s(cimg_OS=%d)\n%s",
                 t_bold,
                 cimg_OS==0?"Solaris":(cimg_OS==1?"Linux":(cimg_OS==2?"Windows":(cimg_OS==3?"Mac OS X":"Unknown"))),
                 t_normal,t_purple,cimg_OS,t_normal);
    std::fprintf(stderr,"  > Display type   : %s%-12s%s %s(cimg_display_type=%d)%s\n",
                 t_bold,cimg_display_type==0?"No":(cimg_display_type==1?"X11":(cimg_display_type==2?"WindowsGDI":"Unknown")),t_normal,t_purple,cimg_display_type,t_normal);
#ifdef cimg_color_terminal
    std::fprintf(stderr,"  > Color terminal : %s%-12s%s %s(cimg_color_term defined)%s\n",t_bold,"Yes",t_normal,t_purple,t_normal);
#else
    std::fprintf(stderr,"  > Color terminal : %-12s (cimg_color_term undefined)\n","No");
#endif
#ifdef cimg_lapack
    std::fprintf(stderr,"  > Using LAPACK   : %s%-12s%s %s(cimg_lapack defined)%s\n",t_bold,"Yes",t_normal,t_purple,t_normal);
#else
    std::fprintf(stderr,"  > Using LAPACK   : %-12s (cimg_lapack undefined)\n","No");
#endif
    std::fprintf(stderr,"  > Debug messages : %s%-12s%s %s(cimg_debug=%d)%s\n",t_bold,cimg_debug==2?"High":(cimg_debug==1?"Yes":"No"),
                 t_normal,t_purple,cimg_debug,t_normal);
    std::fprintf(stderr,"\n");
  }

#if cimg_debug>=1
  static void warn(const bool cond,const char *format,...) {
    if (cond) {
      va_list ap;
      va_start(ap,format);
      std::fprintf(stderr,"<CImg Warning> ");
      vfprintf(stderr,format,ap);
      std::fputc('\n',stderr);
      va_end(ap);
    }
  }
#else
  inline void warn(const bool cond,const char *format,...) {}
#endif
#ifdef min
#undef min
#undef max
#undef abs
#endif
        
  //! Get the value of the system timer with a millisecond precision.
  inline long int time() {
#if cimg_OS==0 || cimg_OS==1 || cimg_OS==3
    struct timeval st_time;
    gettimeofday(&st_time,NULL);
    return (long int)(st_time.tv_usec/1000 + st_time.tv_sec*1000);
#elif cimg_OS==2
    static SYSTEMTIME st_time;
    GetSystemTime(&st_time);
    return (long int)(st_time.wMilliseconds + 1000*(st_time.wSecond + 60*(st_time.wMinute + 60*st_time.wHour)));
#else 
    return 0;
#endif
  }
  //! Sleep for a certain numbers of milliseconds.
  inline void sleep(const int milliseconds) {
#if cimg_OS==0 || cimg_OS==1 || cimg_OS==3
    struct timespec tv;
    tv.tv_sec = milliseconds/1000;
    tv.tv_nsec = (milliseconds%1000)*1000000;
    nanosleep(&tv,NULL);
#elif cimg_OS==2
    Sleep(milliseconds);
#endif
  }
  //! Wait a certain number of milliseconds since the last call of wait_sync().
  //! If the desired delay has expired, this function returns immediately.
  inline long int wait_sync(const int milliseconds=20,long int reference_time=-1) {
    static long int latest_time = cimg::time();
    if (reference_time>=0) latest_time = reference_time;
    const long int current_time = cimg::time(), time_diff = milliseconds + latest_time - current_time;
    if (time_diff>0) { cimg::sleep(time_diff); return (latest_time = current_time + time_diff); }
    else return (latest_time = current_time);
  }
  //! Return the absolute value of \p a
  template<typename T> inline const T abs(const T& a) { return a>=0?a:-a; }
  //! Return the minimum between \p a and \p b.
  template<typename T> inline const T& min(const T& a,const T& b) { return a<=b?a:b; }
  //! Return the minimum between \p a,\p b and \a c.
  template<typename T> inline const T& min(const T& a,const T& b,const T& c) { return min(min(a,b),c); }
  //! Return the minimum between \p a,\p b,\p c and \p d.
  template<typename T> inline const T& min(const T& a,const T& b,const T& c,const T& d) { return min(min(min(a,b),c),d); }
  //! Return the maximum between \p a and \p b.
  template<typename T> inline const T& max(const T& a,const T& b) { return a>=b?a:b; }
  //! Return the maximum between \p a,\p b and \p c.
  template<typename T> inline const T& max(const T& a,const T& b,const T& c) { return max(max(a,b),c); }
  //! Return the maximum between \p a,\p b,\p c and \p d.
  template<typename T> inline const T& max(const T& a,const T& b,const T& c,const T& d) { return max(max(max(a,b),c),d); }
  //! Return the sign of \p x.
  template<typename T> inline T sign(const T& x) { return x<0?-1:(x==0?0:1); }
  //! Return \p x modulo \p m (generic modulo).
  template<typename T> inline T mod(const T& x,const T& m) { return x-m*std::floor((double)(x/m)); }
  //! Return \p x modulo \p m (integer-only modulo).
  inline int modi(const int x,const int m) { return x>=0?x%m:(x%m?m+x%m:0); }
  //! Return minmod(\p a,\p b).
  template<typename T> inline T minmod(const T& a,const T& b) { return a*b<=0?0:(a>0?(a<b?a:b):(a<b?b:a)); }
  //! Return a random variable between [0,1] (uniform distribution).
  inline double rand() { return (double)std::rand()/RAND_MAX; }
  //! Return a random variable between [-1,1] (uniform distribution).
  inline double crand() { return 1-2*rand(); }
  //! Return a random variable (gaussian distribution, with a variance of 1).
  inline double grand() { return std::sqrt(-2*std::log((double)(1e-10 + (1-2e-10)*rand())))*std::cos((double)(2*PI*rand())); }
  inline int xln(const int x) { return x>0?(int)(1+std::log10((double)x)):1; }
  inline char uncase(const char x) { return (x<'A'||x>'Z')?x:x-'A'+'a'; }
  inline float atof(const char *str) {
    float x=0,y=1;
    if (!str) return 0; else { std::sscanf(str,"%g/%g",&x,&y); return x/y; }
  }
  inline int strlen(const char *s) { if (s) { int k; for (k=0; s[k]; k++) ; return k; } return -1; }
  inline int strncmp(const char *s1,const char *s2,const int l) {
    if (s1 && s2) { int n=0; for (int k=0; k<l; k++) n+=abs(s1[k] - s2[k]); return n; }
    return 0;
  }
  inline int strncasecmp(const char *s1,const char *s2,const int l) {
    if (s1 && s2) { int n=0; for (int k=0; k<l; k++) n+=abs(uncase(s1[k])-uncase(s2[k])); return n; }
    return 0;
  }
  inline int strcmp(const char *s1,const char *s2)     { return strncmp(s1,s2,1+min(strlen(s1),strlen(s2))); }
  inline int strcasecmp(const char *s1,const char *s2) { return strncasecmp(s1,s2,1+min(strlen(s1),strlen(s2))); }
  inline int strfind(const char *s,const char c) { if (s) { int l; for (l=strlen(s); l>=0 && s[l]!=c; l--) ; return l; }	return -1; }
  inline const char* basename(const char *s)  { return (cimg_OS!=2)?(s?s+1+strfind(s,'/'):NULL):(s?s+1+strfind(s,'\\'):NULL); }

  //! Return the path of the ImageMagick convert tool (if found).
  //! If you have installed the ImageMagick package (http://www.imagemagick.org)
  //! in a standart directory, this function should return the correct path of the 'convert' tool.
  //! Conversely, if the path is not auto-detected, you must define the macro \p cimg_convert_path with the correct path
  //! of the \p convert executable, before including \p CImg.h in your program.
  inline const char* convert_path() {
    static char *convert_path = NULL;
    if (!convert_path) {
      convert_path = new char[1024];
#ifdef cimg_convert_path
      std::strcpy(convert_path,cimg_convert_path);
#else
#if cimg_OS==2
      std::FILE *file;
      bool stopflag=false;
      for (int k=0; k<=9 && !stopflag; k++) {
        std::sprintf(convert_path,"C:\\PROGRA~1\\IMAGEM~1.%d-Q\\convert.exe",k);
        file = std::fopen(convert_path,"r");
        if (file) { std::fclose(file); stopflag = true; }
      }
      if (!stopflag) std::strcpy(convert_path,"convert.exe");
#else
      std::strcpy(convert_path,"convert");
#endif
#endif
    }
    return convert_path;
  }
   
  //! Return a path to store temporary files (if found).
  //! If you are running on a standart Unix or Windows system, this function should return a correct path.
  //! If the path is not auto-detected, you must define the macro \p cimg_temporary_path,
  //! before including \p CImg.h in your program.
  inline const char* temporary_path() {
    static char *temporary_path = NULL;
    if (!temporary_path) {
      temporary_path = new char[1024];
#ifdef cimg_temporary_path
      std::strcpy(temporary_path,cimg_temporary_path);
#else
      const char* testing_path[6] = { "/tmp","C:\\WINNT\\Temp", "C:\\WINDOWS\\Temp","","C:",NULL };
      char filetmp[1024];
      std::FILE *file=NULL;
      int i=-1;
      while (!file && testing_path[++i]) {
        std::sprintf(filetmp,"%s/CImg%.4d.ppm",testing_path[i],std::rand()%10000);
        if ((file=std::fopen(filetmp,"w"))!=NULL) { std::fclose(file); std::remove(filetmp); }
      }
      if (!file) throw CImgIOException("cimg::temporary_path() : Unable to find a temporary path accessible for writing\n\
you have to set the macro 'cimg_temporary_path' to a valid path where you have writing access :\n \
#define cimg_temporary_path \"path\" (before including 'CImg.h')");
      std::strcpy(temporary_path,testing_path[i]);
#endif
    }
    return temporary_path;
  }

  inline const char *file_split(const char *const filename, char *const body=NULL) {
    if (!filename) throw CImgArgumentException("cimg::file_split() : Can't split the NULL filename");
    int l=strfind(filename,'.');
    if (l>=0) { if (body) { std::strncpy(body,filename,l); body[l]='\0'; }}
    else { if (body) std::strcpy(body,filename); l=(int)std::strlen(filename)-1; }
    return filename+l+1;
  }

  inline char* file_number(const char *filename,const int number,const uint n,char *const string) {
    char format[1024],body[1024];
    const char *ext = cimg::file_split(filename,body);
    if (n>0) std::sprintf(format,"%s_%%.%dd.%s",body,n,ext);
    else std::sprintf(format,"%s_%%d.%s",body,ext);
    std::sprintf(string,format,number);
    return string;
  }
  inline std::FILE *fopen(const char *const path,const char *const mode) {
    if(!path || !mode) throw CImgArgumentException("cimg::fopen() : Can't open file '%s' with mode '%s'",path,mode);
    if (path[0]=='-') return (mode[0]=='r')?stdin:stdout; else {
      std::FILE *dest=std::fopen(path,mode);
      if(!dest) throw CImgIOException("cimg::fopen() : File '%s' cannot be opened %s",
                                      path,mode[0]=='r'?"for reading":(mode[0]=='w'?"for writing":""),path);
      return dest;
    }
  }
  inline int fclose(std::FILE *file) {
    warn(!file,"cimg::fclose() : Can't close NULL file");
    if (!file || file==stdin || file==stdout) return 0;
    const int errn=std::fclose(file);
    warn(errn!=0,"cimg::fclose() : Error %d during file closing",errn);
    return errn;
  }
  inline int fread(void *ptr,const uint size,const uint nmemb,std::FILE *stream) {
    if (!ptr || size<=0 || nmemb<=0 || !stream)
      throw CImgArgumentException("cimg::fread() : Can't read %dx%d bytes of file '%p' in %p",nmemb,size,stream,ptr);
    const uint errn = (uint)std::fread(ptr,size,nmemb,stream);
    warn(errn!=nmemb,"cimg::fread() : File reading problems, only %d/%d elements read",errn,nmemb);
    return errn;
  }
  inline int fwrite(const void *ptr,const uint size,const uint nmemb,std::FILE *stream) {
    if (!ptr || size<=0 || nmemb<=0 || !stream)
      throw CImgArgumentException("cimg::fwrite() : Can't write %dx%d bytes of file '%p' from '%p'",nmemb,size,stream,ptr);
    const uint errn = (uint)std::fwrite(ptr,size,nmemb,stream);
    if(errn!=nmemb)
      throw CImgIOException("cimg::fwrite() : File writing problems, only %d/%d elements written",errn,nmemb);
    return errn;
  }
  inline void bswap(void *const xbuffer,const uint size,const uint n) {
    char minibuf[32],*buffer=(char*)xbuffer;
    if (!buffer || n>32)
      throw CImgArgumentException("cimg::bswap() : Can't swap buffer %p with blocks of size %d",buffer,n);
    for (uint y=0; y<size; y++) {
      { for (uint x=0; x<n; x++) minibuf[x] = *(buffer++); }
      { for (uint x=0; x<n; x++) *(--buffer) = minibuf[x]; }
      buffer+=n;
    }
  }
  inline void inr_header_read(std::FILE *file,int out[8],float *voxsize=NULL) {
    char item[1024],tmp1[64],tmp2[64];
    out[0]=out[1]=out[2]=out[3]=out[5]=1; out[4]=out[6]=out[7]=-1;
    fscanf(file,"%63s",item);
    if(strncasecmp(item,"#INRIMAGE-4#{",13)!=0)
      throw CImgIOException("cimg::inr_header_read() : INRIMAGE-4 Header not found");
    while (fscanf(file," %63[^\n]%*c",item)!=EOF  && strncmp(item,"##}",3)) {
      std::sscanf(item," XDIM%*[^0-9]%d",out);
      std::sscanf(item," YDIM%*[^0-9]%d",out+1);
      std::sscanf(item," ZDIM%*[^0-9]%d",out+2);
      std::sscanf(item," VDIM%*[^0-9]%d",out+3);
      std::sscanf(item," PIXSIZE%*[^0-9]%d",out+6);
      if (voxsize) {
        std::sscanf(item," VX%*[^0-9.eE+-]%f",voxsize);
        std::sscanf(item," VY%*[^0-9.eE+-]%f",voxsize+1);
        std::sscanf(item," VZ%*[^0-9.eE+-]%f",voxsize+2);
      }
      if (std::sscanf(item," CPU%*[ =]%s",tmp1)) out[7]=strncasecmp(tmp1,"sun",3)?1:0;
      switch(std::sscanf(item," TYPE%*[ =]%s %s",tmp1,tmp2)) {
      case 0: break;
      case 2: out[5] = strncasecmp(tmp1,"unsigned",8)?1:0; std::strcpy(tmp1,tmp2);
      case 1:
        if (!strncasecmp(tmp1,"int",3)   || !strncasecmp(tmp1,"fixed",5))  out[4]=0;
        if (!strncasecmp(tmp1,"float",5) || !strncasecmp(tmp1,"double",6)) out[4]=1;
        if (!strncasecmp(tmp1,"packed",6))                                       out[4]=2;
        if (out[4]>=0) break;
      default: throw CImgIOException("cimg::inr_header_read() : Invalid TYPE '%s'",tmp2);
      }
    }
    if(out[0]<0 || out[1]<0 || out[2]<0 || out[3]<0)
      throw CImgIOException("cimg::inr_header_read() : Bad dimensions in .inr file = ( %d , %d , %d , %d )",out[0],out[1],out[2],out[3]);
    if(out[4]<0 || out[5]<0) throw CImgIOException("cimg::inr_header_read() : TYPE is not fully defined");
    if(out[6]<0) throw CImgIOException("cimg::inr_header_read() : PIXSIZE is not fully defined");
    if(out[7]<0) throw CImgIOException("cimg::inr_header_read() : CPU type is not fully defined");
  }
  
  inline const char* option(const char *const name,const uint argc,char **argv,const char *const defaut,
                            const char *const usage=NULL) {
    static bool first=true, visu=false;
    const char *res = NULL;
    if (first) { first=false; visu = option("-h",argc,argv,(const char*)NULL)!=NULL; }
    if (!name && visu) {
      std::fprintf(stderr,"\n %s%s%s",t_red,basename(argv[0]),t_normal);
      if (usage) std::fprintf(stderr," : %s",usage);
      std::fprintf(stderr," (%s, %s)\n\n",__DATE__,__TIME__);
    }
    if (name) {
      if (argc>0) {
        uint k=0,i;
        while (k<argc && strcmp(argv[k],name)) k++;
        i=k;
        res=(k++==argc?defaut:(k==argc?argv[--k]:argv[k]));
      } else res = defaut;
      if (visu && usage) std::fprintf(stderr,"    %s%-8s%s = %-12s : %s%s%s\n",t_bold,name,t_normal,res?res:"NULL",t_purple,usage,t_normal);
    }
    return res;
  }
  inline bool option(const char *const name,const uint argc,char **argv,const bool defaut,const char *const usage=NULL) {
    const char *s = option(name,argc,argv,(const char*)NULL);
    const bool res = s?(strcasecmp(s,"false") && strcasecmp(s,"off") && strcasecmp(s,"0")):defaut;
    option(name,0,NULL,res?"true":"false",usage);
    return res;
  }
  inline int option(const char *const name,const uint argc,char **argv,const int defaut,const char *const usage=NULL) {
    const char *s = option(name,argc,argv,(const char*)NULL);
    const int res = s?atoi(s):defaut;
    char tmp[256];
    std::sprintf(tmp,"%d",res);
    option(name,0,NULL,tmp,usage);
    return res;
  }
  inline double option(const char *const name,const uint argc,char **argv,const double defaut,const char *const usage=NULL) {
    const char *s = option(name,argc,argv,(const char*)NULL);
    const double res = s?atof(s):defaut;
    char tmp[256];
    std::sprintf(tmp,"%g",res);
    option(name,0,NULL,tmp,usage);
    return res;
  }
}


/*-------------------------------------------------------




  Definition of the CImgStats structure




  ------------------------------------------------------*/
//! This structure represents statistics of an image.
//! Use it to compute basic statistics on pixel values, such as the minimum, maximum, average values
//! and the variance of a \c CImg<T> images.
struct CImgStats
{
  double min;                 //!< Minimum of the pixel values after statistics computation.
  double max;                 //!< Maximum of the pixel values after statistics computation.
  double mean;                //!< Mean of the pixel values after statistics computation.
  double variance;            //!< Variance of the pixel values after statistics computation.

  //! Empty constructor.
  CImgStats():min(0),max(0),mean(0),variance(0) {}
  //! Copy constructor.
  CImgStats(const CImgStats& stats):min(stats.min),max(stats.max),mean(stats.mean),variance(stats.variance) {};
  //! Assignement operator.
  CImgStats& operator=(const CImgStats stats) {
    min = stats.min;
    max = stats.max;
    mean = stats.mean;
    variance = stats.variance;
    return *this;
  }
  //! Create a CImgStats object by computing the statistics of an image \p img.
  //! If \p compute_variance = true, the variance field of the CImgStats structure is computed, else it is set to zero.
    template<typename T> CImgStats(const CImg<T>& img,const bool compute_variance=true):mean(0),variance(0) {
      cimg_test(img);
      T pmin=img[0], pmax=pmin,a,*ptr;
      cimg_map(img,ptr) { a=*ptr; mean+=(double)a; if (a<pmin) pmin=a; if (a>pmax) pmax=a; }
      mean/=img.size();
      min=(double)pmin;
      max=(double)pmax;
      if (compute_variance) {
        cimg_map(img,ptr) { const double tmpf=(*ptr)-mean; variance+=tmpf*tmpf; }
        variance = std::sqrt(variance/img.size());
      }
    }
  //! Create a CImgStats object by computing the statistics of a list of images \p list.
  //! If \p compute_variance = true, the variance field of the CImgStats structure is computed, else it is set to zero.
  template<typename T> CImgStats(const CImgl<T>& list,const bool compute_variance=true):mean(0),variance(0) {
    cimgl_test(list);
    T pmin=list[0][0], pmax=pmin,a,*ptr;
    int psize=0;
    cimgl_map(list,l) {
      cimg_map(list[l],ptr) {
        a=*ptr; mean+=(double)a; if (a<pmin) pmin=a; if (a>pmax) pmax=a;
      }
      psize+=list[l].size();
    }
    mean/=psize;
    min=(double)pmin;
    max=(double)pmax;
    if (compute_variance) {
      cimgl_map(list,l) cimg_map(list[l],ptr) { const double tmpf=(*ptr)-mean; variance+=tmpf*tmpf; }
      variance = std::sqrt(variance/psize);
    }
  }
  //! Print the current statistics on the standart error output.
  const CImgStats& print(const char* title=NULL) const {
    fprintf(stderr,"%-8s = { %g, %g [%g], %g }\n",title?title:"CImgStats",min,mean,variance,max);
    return *this;
  }
};

/*-------------------------------------------------------
  



  Definition of the CImgDisplay structure




  ------------------------------------------------------*/
//! This structure is used to display a CImg<T> image in a window, and interact with mouse and keyboard.
struct CImgDisplay
{
  uint width;                 //!< Width of the display window.
  uint height;                //!< Height of the display window.
  uint normalize;             //!< Pixel value normalization (0=no normalization, 1=normalization between [0,255], 2=first-step normalization).
  volatile uint event_type;   //!< Type of events handled by the display window (0=no events, 1=resize+close, 2=mouse+key, 3=full events).
  volatile int mousex;        //!< When event_type>=2, this variable corresponds to the mouse pointer X-coordinate into the display window.
  volatile int mousey;        //!< When event_type>=2, this variable corresponds to the mouse pointer Y-coordinate into the display window.
  volatile uint button;       //!< When event_type>=2, these variables corresponds to the mouse button clicked into the display window.
  volatile uint key;          //!< When event_type>=2, these variables corresponds to the key pressed into the display window.
  volatile bool closed;       //!< When event_type>=1, this variable tells if the window is closed (true) or still visible (false).
  double min,max;
  
  CImgDisplay& operator=(const CImgDisplay& disp) {
    throw CImgDisplayException("CImgDisplay()::operator=() : Assignement operator has been invoked, which is not allowed. Use pointers instead !");
    return *this;
  }
  
  //! Same as \ref cimg::wait_sync().
  const CImgDisplay& wait_sync(const uint milliseconds) const {
    cimg::wait_sync(milliseconds);
    return *this;
  }

  //! Create a display window from an image list.
  //! Images are appended along the axe \p axe (can be 'x' or 'y')
  //! and centered with the position \p position (can be 'p','c','n')
  template<typename T> CImgDisplay& display(const CImgl<T>& list,const char axe='x',const char position='c') { 
    return display(list.get_append(axe,position)); 
  } 

  //! Resize a display window with the dimensions of the image \p img.
  template<typename T> CImgDisplay& resize(const CImg<T>& img) { return resize(img.width,img.height); }
  
  // When no display available
  //---------------------------
#if cimg_display_type==0
  void nodisplay_available() { cimg::warn(true,"CImgDisplay() : Display has been required but is not available (cimg_display_type=0)"); }
  
  //! Create a display window with a specified size (\p pwidth,\p height).
  CImgDisplay(const uint pwidth=320,const uint pheight=256,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=false) { 
    nodisplay_available(); 
  }
  //! Create a new display window by copying another one.
  CImgDisplay(const CImgDisplay& win, char *title=NULL) { nodisplay_available(); }
  //! Create a display window from an image.
  template<typename T> CImgDisplay(const CImg<T>& pimg,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=false) { 
    nodisplay_available(); 
  }
  //! Create a display window from an image list.
  template<typename T> CImgDisplay(const CImgl<T>& plist,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=false) { 
    nodisplay_available(); 
  }
  //! Resize a display window with new dimensions \p nwidth and \p nheight.
  CImgDisplay& resize(const int nwidth, const int nheight) { return *this; }
  
  //! Destructor. Close and destroy a display.
  ~CImgDisplay() {}

  //! Fill the pixel data of the window buffer according to the image \p pimg.
  template<typename T> void render(const CImg<T>& pimg,const uint pymin=0,const uint pymax=-1) {}

  //! Display an image in a window.
  template<typename T> CImgDisplay& display(const CImg<T>& img,const uint ymin=0,const uint ymax=-1) { return *this; }

  //! Wait for a window event
  CImgDisplay& wait()  { return *this; }

  //! Show a closed display
  CImgDisplay& show()  { return *this; }

  //! Close a visible display
  CImgDisplay& close() { return *this; }
  
  // X11-based display
  //------------------
#elif cimg_display_type==1
  unsigned long *data;
  XImage *image;
  Window window;
  
  CImgDisplay(const uint pwidth=320,const uint pheight=200,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    width(pwidth),height(pheight),normalize(pnormalize),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    new_lowlevel(title);
    std::memset(data,0,sizeof(unsigned long)*width*height);
    pthread_mutex_lock(cimg_mutex);
    XPutImage(cimg_displayX11,window,*cimg_gc,image,0,0,0,0,width,height);
    XFlush(cimg_displayX11);
    pthread_mutex_unlock(cimg_mutex);
  }
  CImgDisplay(const CImgDisplay& win, char *title="[Copy]"):width(win.width),height(win.height),normalize(win.normalize),event_type(win.event_type),mousex(-1),mousey(-1),button(0),key(0),closed(win.closed),min(win.min),max(win.max) {
    new_lowlevel(title);
    std::memcpy(data,win.data,sizeof(unsigned long)*width*height);
    pthread_mutex_lock(cimg_mutex);
    XPutImage(cimg_displayX11,window,*cimg_gc,image,0,0,0,0,width,height);
    XFlush(cimg_displayX11);
    pthread_mutex_unlock(cimg_mutex);
  }
  template<typename T> CImgDisplay(const CImg<T>& pimg,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    normalize(pnormalize),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    cimg_test(pimg);
    CImg<T> tmp;
    const CImg<T>& img = (pimg.depth==1)?pimg:(tmp=pimg.get_3dplanes(pimg.width/2,pimg.height/2,pimg.depth/2));
    width = img.width; height = img.height;
    if (normalize==2) { CImgStats st(img,false); min=st.min; max=st.max; }
    new_lowlevel(title);
    display(img);
  }
  template<typename T> CImgDisplay(const CImgl<T>& plist,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    normalize(pnormalize),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    CImg<T> tmp;
    const CImg<T> img0 = plist.get_append('x'), &img = (img0.depth==1)?img0:(tmp=img0.get_3dplanes(img0.width/2,img0.height/2,img0.depth/2));
    width = img.width; height = img.height;
    if (normalize==2) { CImgStats st(img,false); min=st.min; max=st.max; }
    new_lowlevel(title);
    display(img);
  }

  CImgDisplay& resize(const int nwidth, const int nheight) {
    const uint dimx=nwidth>0?nwidth:-width*nwidth/100, dimy=nheight>0?nheight:-height*nheight/100;
    if (!dimx || !dimy) throw CImgArgumentException("CImgDisplay::resize() : bad specified dimensions %dx%d",dimx,dimy);
    if (dimx==width && dimy==height) return *this;
    unsigned long *ndata = new unsigned long[dimx*dimy];
    for (uint y=0; y<dimy; y++) for (uint x=0; x<dimx; x++) ndata[x+y*dimx] = data[x*width/dimx + width*(y*height/dimy)];
    width  = dimx;
    height = dimy;
    data = ndata;
    pthread_mutex_lock(cimg_mutex);
    XDestroyImage(image);
    image = XCreateImage(cimg_displayX11,DefaultVisual(cimg_displayX11,DefaultScreen(cimg_displayX11)),
                         cimg_bits,ZPixmap,0,(char*)data,width,height,8,0);
    XResizeWindow(cimg_displayX11,window,width,height);
    pthread_mutex_unlock(cimg_mutex);
    return *this;
  }
  
  ~CImgDisplay() {
    uint i;
    pthread_mutex_lock(cimg_mutex);
    for (i=0; i<cimg_nb_wins && cimg_wins[i]!=this; i++) i++;
    for (; i<cimg_nb_wins-1; i++) cimg_wins[i]=cimg_wins[i+1];
    cimg_nb_wins--;
    XDestroyWindow(cimg_displayX11,window);
    XDestroyImage(image);
    if (!cimg_nb_wins) {
      pthread_cancel(*cimg_event_thread);
      pthread_join(*cimg_event_thread,NULL);
      XCloseDisplay(cimg_displayX11);
      cimg_displayX11=NULL;
      pthread_mutex_unlock(cimg_mutex);
      pthread_mutex_destroy(cimg_mutex);
      delete cimg_event_thread;
      delete cimg_mutex;
      delete cimg_gc;
    } else pthread_mutex_unlock(cimg_mutex);
  }
  
  void new_lowlevel(const char *title=NULL) {
    cimg::warn(event_type&4,"CImgDisplay::new_lowlevel() : Fullscreen mode requested, but not supported on X11 Displays");
    if (!cimg_displayX11) {
      cimg_nb_wins = 0;
      cimg_thread_finished = false;
      cimg_mutex = new pthread_mutex_t;
      pthread_mutex_init(cimg_mutex,NULL);
      pthread_mutex_lock(cimg_mutex);
      cimg_displayX11 = XOpenDisplay((getenv("DISPLAY") ? getenv("DISPLAY") : ":0.0"));
      if (!cimg_displayX11) throw CImgDisplayException("CImgDisplay::new_lowlevel() : Can't open display");
      cimg_bits = DefaultDepth(cimg_displayX11, DefaultScreen(cimg_displayX11));
      if (cimg_bits!=16 && cimg_bits!=24)
        throw CImgDisplayException("CImgDisplay::new_lowlevel() : %d bits mode is not supported (only 16 and 24 bits are supported)",cimg_bits);
      cimg_gc = new GC;
      *cimg_gc = DefaultGC(cimg_displayX11,DefaultScreen(cimg_displayX11));
      cimg_event_thread = new pthread_t;
      pthread_create(cimg_event_thread,NULL,thread_lowlevel,NULL);
    } else pthread_mutex_lock(cimg_mutex);
    window  = XCreateSimpleWindow(cimg_displayX11,RootWindow(cimg_displayX11,DefaultScreen(cimg_displayX11)),0,0,width,height,2,0,0x0L);
    data    = new unsigned long[width*height];
    image   = XCreateImage(cimg_displayX11,DefaultVisual(cimg_displayX11,DefaultScreen(cimg_displayX11)),cimg_bits,ZPixmap,0,(char*)data,width,height,8,0);
    XStoreName(cimg_displayX11,window,title?title:"");
    if (!closed) {
      XEvent event;
      XSelectInput(cimg_displayX11,window,StructureNotifyMask);
      XMapWindow(cimg_displayX11,window);
      do XWindowEvent(cimg_displayX11,window,StructureNotifyMask,&event); while (event.type!=MapNotify);
    }
    if (event_type&3) { Atom atom = XInternAtom(cimg_displayX11, "WM_DELETE_WINDOW", False); XSetWMProtocols(cimg_displayX11, window, &atom, 1); }
    cimg_wins[cimg_nb_wins++]=this;
    pthread_mutex_unlock(cimg_mutex);
  }
  
  void proc_lowlevel(XEvent *pevent) {
    const uint buttoncode[3] = { 1,4,2 };
    XEvent event=*pevent;
    switch (event.type) {
    case ClientMessage: XUnmapWindow(cimg_displayX11,window); closed=true; mousex=mousey=-1; key=button=0; break;
    case ResizeRequest: {
      while (XCheckWindowEvent(cimg_displayX11,window,ResizeRedirectMask,&event));
      uint nw = event.xresizerequest.width, nh = event.xresizerequest.height;
      if (nw>width || nh>height) XResizeWindow(cimg_displayX11,window,width,height);
    } break;
    case Expose: while (XCheckWindowEvent(cimg_displayX11,window,ExposureMask,&event));
      XPutImage(cimg_displayX11,window,*cimg_gc,image,0,0,0,0,width,height);
      break;
    case ButtonPress:   while (XCheckWindowEvent(cimg_displayX11,window,ButtonPressMask,&event));   button |= buttoncode[event.xbutton.button-1];  break;
    case ButtonRelease: while (XCheckWindowEvent(cimg_displayX11,window,ButtonReleaseMask,&event)); button &= ~buttoncode[event.xbutton.button-1]; break;
    case KeyPress:      while (XCheckWindowEvent(cimg_displayX11,window,KeyPressMask,&event));      key = event.xkey.keycode;                      break;
    case KeyRelease:    while (XCheckWindowEvent(cimg_displayX11,window,KeyReleaseMask,&event));    key = 0;                                       break;
    case LeaveNotify:   while (XCheckWindowEvent(cimg_displayX11,window,LeaveWindowMask,&event));   mousex=mousey=-1; key=button=0;                break;
    case MotionNotify:  while (XCheckWindowEvent(cimg_displayX11,window,PointerMotionMask,&event));
      mousex = event.xmotion.x; mousey = event.xmotion.y;
      if (mousex<0) mousex = 0;
      if (mousey<0) mousey = 0;
      if (mousex>=(int)width)  mousex = width-1;
      if (mousey>=(int)height) mousey = height-1;
      break;
    }
  }
  
  static void* thread_lowlevel(void *arg) {
    XEvent event;
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
    for (;;) {
      pthread_mutex_lock(cimg_mutex);
      for (uint i=0; i<cimg_nb_wins; i++) {
        const uint xevent_type = (cimg_wins[i]->event_type)&3;
        const uint emask =
          ((xevent_type>=1)?ExposureMask|StructureNotifyMask|ResizeRedirectMask:0)
          |((xevent_type>=2)?ButtonPressMask|KeyPressMask|PointerMotionMask|LeaveWindowMask:0)
          |((xevent_type>=3)?ButtonReleaseMask|KeyReleaseMask:0);
        XSelectInput(cimg_displayX11,cimg_wins[i]->window,emask);
      }
      bool event_flag = XCheckTypedEvent(cimg_displayX11, ClientMessage, &event);
      if (!event_flag) event_flag = XCheckMaskEvent(cimg_displayX11, ExposureMask|StructureNotifyMask|ResizeRedirectMask|ButtonPressMask
                                                    |KeyPressMask|PointerMotionMask|LeaveWindowMask|ButtonReleaseMask|KeyReleaseMask,&event);
      if (event_flag) {
        for (uint i=0; i<cimg_nb_wins; i++)
          if (!cimg_wins[i]->closed && event.xany.window==cimg_wins[i]->window) cimg_wins[i]->proc_lowlevel(&event);
        cimg_thread_finished = true;
      }
      pthread_mutex_unlock(cimg_mutex);
      cimg::wait_sync(20);
    }
    return NULL;
  }

  template<typename T> XImage* render(const CImg<T>& pimg,const uint pymin=0,const uint pymax=-1) {
    cimg_test(pimg);
    CImg<T> tmp;
    const CImg<T>& img0 = (pimg.depth==1)?pimg:(tmp=pimg.get_3dplanes(pimg.width/2,pimg.height/2,pimg.depth/2));
    const CImg<T>& img  = (img0.width==width && img0.height==height)?img0:(tmp=img0.get_resize(width,height,1,-100,1));
    uint ymin = pymin<pymax?(pymin<0?0:pymin):(pymax<0?0:pymax),
         ymax = pymin<pymax?(pymax>=height?height-1:pymax):(pymin>=height?height-1:pymin);
    T *data1 = img.ptr(0,ymin,0,0),
      *data2 = img.dim>=2?img.ptr(0,ymin,0,1):data1,
      *data3 = img.dim>=3?img.ptr(0,ymin,0,2):data1,
      pmin, mm;

    pthread_mutex_lock(cimg_mutex);
    XImage *ximg = image;
    switch(normalize) {
    case 0:
      switch (cimg_bits) {
      case 16: 
        for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) { 
          XPutPixel(ximg,x,y,(((uchar)*(data1++)>>3)<<11) | (((uchar)*(data2++)>>2)<<5) | ((uchar)*(data3++)>>3)); 
        }
        break;
      case 24: 
        for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) { 
          XPutPixel(ximg,x,y,((uchar)*(data1++)<<16)      | ((uchar)*(data2++)<<8)      | (uchar)*(data3++)     ); 
        }
        break;
      };
      break;
    case 1: { CImgStats st(img,false); min=(T)st.min; max=(T)st.max; }
    default:
      pmin = (T)min; mm = (T)(max-min); if (mm==0) mm=1;
      switch (cimg_bits) {
      case 16: for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) {
          const uchar
            val1 = (uchar)(255*(*(data1++)-pmin)/mm),
            val2 = (uchar)(255*(*(data2++)-pmin)/mm),
            val3 = (uchar)(255*(*(data3++)-pmin)/mm);
          XPutPixel(ximg,x,y,((val1>>3)<<11) | ((val2>>2)<<5) | (val3>>3));
        }
        break;
      case 24: for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) {
          const uchar
            val1 = (uchar)(255*(*(data1++)-pmin)/mm),
            val2 = (uchar)(255*(*(data2++)-pmin)/mm),
            val3 = (uchar)(255*(*(data3++)-pmin)/mm);
          XPutPixel(ximg,x,y,(val1<<16) | (val2<<8) | val3);
        }
        break;
      } 
      break;
    }
    pthread_mutex_unlock(cimg_mutex);
    return image;
  }

  template<typename T> CImgDisplay& display(const CImg<T>& pimg,const uint pymin=0,const uint pymax=-1) {
    const uint
      ymin = pymin<pymax?pymin:pymax,
      ymax = pymin<pymax?(pymax>=height?height-1:pymax):(pymin>=height?height-1:pymin);
    render(pimg,ymin,ymax);
    if (!closed) {      
      pthread_mutex_lock(cimg_mutex);
      XPutImage(cimg_displayX11,window,*cimg_gc,image,0,ymin,0,ymin,width,ymax-ymin+1);
      XFlush(cimg_displayX11);
      pthread_mutex_unlock(cimg_mutex);
    }
    return *this;
  }
  
  CImgDisplay& wait() {
    if (!closed && (event_type&2)) {
      XEvent event;
      do {
        pthread_mutex_lock(cimg_mutex);
        const uint xevent_type = event_type&3;
        const uint emask = ExposureMask|StructureNotifyMask|ResizeRedirectMask
          |((xevent_type>=2)?ButtonPressMask|KeyPressMask|PointerMotionMask|LeaveWindowMask:0)|((xevent_type>=3)?ButtonReleaseMask|KeyReleaseMask:0);
        XSelectInput(cimg_displayX11,window,emask);
        XPeekEvent(cimg_displayX11,&event);
        cimg_thread_finished = false;
        pthread_mutex_unlock(cimg_mutex);
      } while (event.xany.window!=window);
      while (!cimg_thread_finished) cimg::wait_sync(20);
    }
    return *this;
  }
  CImgDisplay& show() {
    if (closed) {
      pthread_mutex_lock(cimg_mutex);
      XEvent event;
      XSelectInput(cimg_displayX11,window,StructureNotifyMask);
      XMapWindow(cimg_displayX11,window);
      do XWindowEvent(cimg_displayX11,window,StructureNotifyMask,&event); while (event.type!=MapNotify);
      XPutImage(cimg_displayX11,window,*cimg_gc,image,0,0,0,0,width,height);
      XFlush(cimg_displayX11);
      closed = false;
      pthread_mutex_unlock(cimg_mutex);
    }
    return *this;
  }
  CImgDisplay& close() {
    if (!closed) {
      pthread_mutex_lock(cimg_mutex);
      XUnmapWindow(cimg_displayX11,window);
      XFlush(cimg_displayX11);
      closed = true;
      pthread_mutex_unlock(cimg_mutex);
    }
    return *this;
  }
  
  // Windows-based display
  //-----------------------
#elif cimg_display_type==2
  CLIENTCREATESTRUCT ccs;
  BITMAPINFO bmi;
  uint *data;
  DEVMODE curr_mode;
  HWND window;
  HDC hdc;
  HANDLE thread;
  HANDLE wait_disp;
  HANDLE created;
  HANDLE mutex;
  

  CImgDisplay(const uint pwidth=320,const uint pheight=200,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    width(pwidth),height(pheight),normalize(pnormalize<2?pnormalize:1),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    new_lowlevel(title);
    std::memset(data,0,sizeof(uint)*width*height);
    SetDIBitsToDevice(hdc,0,0,width,height,0,0,0,height,data,&bmi,DIB_RGB_COLORS);
  }

  CImgDisplay(const CImgDisplay& win, char *title="[Copy]"):width(win.width),height(win.height),normalize(win.normalize),event_type(win.event_type),mousex(-1),mousey(-1),button(0),key(0),closed(win.closed),min(win.min),max(win.max) {
    new_lowlevel(title);
    std::memcpy(data,win.data,sizeof(unsigned long)*width*height);
    SetDIBitsToDevice(hdc,0,0,width,height,0,0,0,height,data,&bmi,DIB_RGB_COLORS);
  }

  template<typename T> CImgDisplay(const CImg<T>& pimg,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    normalize(pnormalize),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    cimg_test(pimg);
    CImg<T> tmp;
    const CImg<T>& img = (pimg.depth==1)?pimg:(tmp=pimg.get_3dplanes(pimg.width/2,pimg.height/2,pimg.depth/2));
    width = img.width; height = img.height;
    if (normalize==2) { CImgStats st(img,false); min=st.min; max=st.max; }
    new_lowlevel(title);
    display(img);
  }

  template<typename T> CImgDisplay(const CImgl<T>& plist,const char *title=NULL,const uint pnormalize=1,const uint pevent_type=3,const bool visible=true):
    normalize(pnormalize),event_type(pevent_type),mousex(-1),mousey(-1),button(0),key(0),closed(!visible),min(0),max(0) {
    CImg<T> tmp;
    const CImg<T> img0 = plist.get_append('x'), &img = (img0.depth==1)?img0:(tmp=img0.get_3dplanes(img0.width/2,img0.height/2,img0.depth/2));
    width = img.width; height = img.height;
    if (normalize==2) { CImgStats st(img,false); min=st.min; max=st.max; }
    new_lowlevel(title);
    display(img);
  }

  CImgDisplay& resize(const int nwidth, const int nheight) {
    const uint dimx=nwidth>0?nwidth:(-nwidth)*width/100, dimy=nheight>0?nheight:(-nheight)*height/100;
    if (!dimx || !dimy) throw CImgArgumentException("CImgDisplay::resize() : bad specified dimensions %dx%d",dimx,dimy);
    if (dimx==width && dimy==height) return *this;
    uint *ndata = new uint[dimx*dimy];
    for (uint y=0; y<dimy; y++) for (uint x=0; x<dimx; x++) ndata[x+y*dimx] = data[x*width/dimx + width*(y*height/dimy)];
    delete[] data;
    width  = dimx;
    height = dimy;
    data = ndata;
    bmi.bmiHeader.biWidth=width;
    bmi.bmiHeader.biHeight=-(int)height;
    SetWindowPos(window,HWND_TOP,0,0,width,height+24,SWP_NOMOVE);
    return *this;
  }

  ~CImgDisplay() {
    uint imode=0;
    bool stopflag=0;
    DestroyWindow(window);
    if (event_type&3) TerminateThread(thread,0);
    delete[] data;
    if (curr_mode.dmSize) ChangeDisplaySettings(&curr_mode,0);
  }
  
  void new_lowlevel(const char *title=NULL) {
    unsigned long ThreadID;
    DEVMODE mode;
    uint imode=0,ibest=0,bestbpp=0;
    bool stopflag = false;
    void *arg = (void*)(new void*[2]);
    ((void**)arg)[0]=(void*)this;
    ((void**)arg)[1]=(void*)title;
    if (event_type&4) {
       for (mode.dmSize = sizeof(DEVMODE), mode.dmDriverExtra = 0; EnumDisplaySettings(NULL,imode,&mode); imode++)
          if (mode.dmPelsWidth==width && mode.dmPelsHeight==height && mode.dmBitsPerPel>bestbpp) { bestbpp = mode.dmBitsPerPel; ibest=imode; }
      cimg::warn(!bestbpp,"CImgDisplay::new_lowlevel() : Could not initialize fullscreen mode %dx%d\n",width,height);
      if (bestbpp) {
        curr_mode.dmSize = sizeof(DEVMODE); curr_mode.dmDriverExtra = 0;
        EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&curr_mode);
        EnumDisplaySettings(NULL,ibest,&mode);
        ChangeDisplaySettings(&mode,0);
      }
      else curr_mode.dmSize = 0;
    }
    else curr_mode.dmSize = 0;
    if (event_type&3) {
      mutex = CreateMutex(NULL,FALSE,NULL);
      created = CreateEvent(NULL,FALSE,FALSE,NULL);
      wait_disp = CreateEvent(NULL,FALSE,FALSE,NULL);
      thread = CreateThread(NULL,0,thread_lowlevel,arg,0,&ThreadID);
      WaitForSingleObject(created,INFINITE);
    } else thread_lowlevel(arg);
  }
  
  static LRESULT APIENTRY proc_lowlevel(HWND window,UINT msg,WPARAM wParam,LPARAM lParam) {
    CImgDisplay* disp = (CImgDisplay*)GetWindowLong(window,GWL_USERDATA);
    const uint event_type = disp->event_type&3;
    MSG st_msg;
    switch(msg) {
    case WM_CLOSE: disp->closed=true; ReleaseMutex(disp->mutex); ShowWindow(disp->window,SW_HIDE); return 0;
    case WM_SIZE: {
      while (PeekMessage(&st_msg,window,WM_SIZE,WM_SIZE,PM_REMOVE));
      WaitForSingleObject(disp->mutex,INFINITE);
      const uint nw = LOWORD(lParam), nh = HIWORD(lParam);     
      if (nw>disp->width || nh>disp->height) SetWindowPos(window,HWND_TOP,0,0,disp->width,disp->height+24,SWP_NOMOVE);
      ReleaseMutex(disp->mutex);
    }
      break;
    case WM_PAINT:
      WaitForSingleObject(disp->mutex,INFINITE);
      SetDIBitsToDevice(disp->hdc,0,0,disp->width,disp->height,0,0,0,disp->height,disp->data,&(disp->bmi),DIB_RGB_COLORS);
      ReleaseMutex(disp->mutex);
      break;
    }
    if (event_type>=2) switch(msg) {
    case WM_KEYDOWN:     while (PeekMessage(&st_msg,window,WM_KEYDOWN,WM_KEYDOWN,PM_REMOVE)); disp->key=(int)wParam; break;
    case WM_MOUSELEAVE:  disp->mousex=disp->mousey=-1; disp->button=disp->key=0; break;
    case WM_MOUSEMOVE:   {
      while (PeekMessage(&st_msg,window,WM_MOUSEMOVE,WM_MOUSEMOVE,PM_REMOVE));
      disp->mousex = LOWORD(lParam);
      disp->mousey = HIWORD(lParam);
      if (disp->mousex<0) disp->mousex = 0;
      if (disp->mousey<0) disp->mousey = 0;
      if (disp->mousex>=(int)disp->width)  disp->mousex = disp->width-1;
      if (disp->mousey>=(int)disp->height) disp->mousey = disp->height-1;

      TRACKMOUSEEVENT tme;
      tme.cbSize = sizeof(TRACKMOUSEEVENT);
      tme.dwFlags = TME_LEAVE;
      tme.hwndTrack = disp->window;
      TrackMouseEvent(&tme);     
    }
      break;
    case WM_LBUTTONDOWN: while (PeekMessage(&st_msg,window,WM_LBUTTONDOWN,WM_LBUTTONDOWN,PM_REMOVE)); disp->button |= 1;  break;
    case WM_RBUTTONDOWN: while (PeekMessage(&st_msg,window,WM_RBUTTONDOWN,WM_RBUTTONDOWN,PM_REMOVE)); disp->button |= 2;  break;
    case WM_MBUTTONDOWN: while (PeekMessage(&st_msg,window,WM_MBUTTONDOWN,WM_MBUTTONDOWN,PM_REMOVE)); disp->button |= 4;  break;
    }
    if (event_type>=3) switch(msg) {
    case WM_KEYUP:       while (PeekMessage(&st_msg,window,WM_KEYUP,WM_KEYUP,PM_REMOVE));         disp->key=0; break;
    case WM_LBUTTONUP:   while (PeekMessage(&st_msg,window,WM_LBUTTONUP,WM_LBUTTONUP,PM_REMOVE)); disp->button &= ~1; break;
    case WM_RBUTTONUP:   while (PeekMessage(&st_msg,window,WM_RBUTTONUP,WM_RBUTTONUP,PM_REMOVE)); disp->button &= ~2; break;
    case WM_MBUTTONUP:   while (PeekMessage(&st_msg,window,WM_MBUTTONUP,WM_MBUTTONUP,PM_REMOVE)); disp->button &= ~4; break;
    }
    return DefWindowProc(window,msg,wParam,lParam);
  }
  
  static DWORD WINAPI thread_lowlevel(void* arg) {
    CImgDisplay *disp  = (CImgDisplay*)(((void**)arg)[0]);
    const char *title = (const char*)(((void**)arg)[1]);
    MSG msg;
    delete[] (void**)arg;
    disp->bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
    disp->bmi.bmiHeader.biWidth=disp->width;
    disp->bmi.bmiHeader.biHeight=-(int)disp->height;
    disp->bmi.bmiHeader.biPlanes=1;
    disp->bmi.bmiHeader.biBitCount=32;
    disp->bmi.bmiHeader.biCompression=BI_RGB;
    disp->bmi.bmiHeader.biSizeImage=0;
    disp->bmi.bmiHeader.biXPelsPerMeter=1;
    disp->bmi.bmiHeader.biYPelsPerMeter=1;
    disp->bmi.bmiHeader.biClrUsed=0;
    disp->bmi.bmiHeader.biClrImportant=0;
    disp->data = new uint[disp->width*disp->height];
    if (!disp->curr_mode.dmSize) disp->window = CreateWindow("MDICLIENT",title?title:"",
                                                 WS_OVERLAPPEDWINDOW | WS_VISIBLE, CW_USEDEFAULT,CW_USEDEFAULT,
                                                 disp->width+9,disp->height+28,NULL,NULL,NULL,&(disp->ccs));
    else disp->window = CreateWindow("MDICLIENT",title?title:"",
                                     WS_POPUP | WS_VISIBLE, CW_USEDEFAULT,CW_USEDEFAULT,
                                     disp->width,disp->height,NULL,NULL,NULL,&(disp->ccs));
    SetForegroundWindow(disp->window);
    disp->hdc = GetDC(disp->window);
    if (disp->event_type&3) {
      SetWindowLong(disp->window,GWL_USERDATA,(LONG)disp);
      SetWindowLong(disp->window,GWL_WNDPROC,(LONG)proc_lowlevel);
      SetEvent(disp->created);
      while( GetMessage( &msg, NULL, 0, 0 ) ) { DispatchMessage( &msg ); SetEvent(disp->wait_disp); }
    }
    return 0;
  }
  
  template<typename T> BITMAPINFO* render(const CImg<T>& pimg,const uint pymin=0,const uint pymax=-1) {
    cimg_test(pimg);
    CImg<T> tmp;
    WaitForSingleObject(mutex,INFINITE);
    const CImg<T>& img0 = (pimg.depth==1)?pimg:(tmp=pimg.get_3dplanes(pimg.width/2,pimg.height/2,pimg.depth/2));
    const CImg<T>& img  = (img0.width==width && img0.height==height)?img0:(tmp=img0.get_resize(width,height,1,-100,1));
    uint ymin = pymin<pymax?pymin:pymax,
         ymax = pymin<pymax?(pymax>=height?height-1:pymax):(pymin>=height?height-1:pymin);
    T *data1 = img.ptr(0,ymin,0,0),
      *data2 = img.dim>=2?img.ptr(0,ymin,0,1):data1,
      *data3 = img.dim>=3?img.ptr(0,ymin,0,2):data1,
      pmin, mm;
    uint *ximg = data + ymin*width;
    if (!normalize)
      for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) *(ximg++) = ((uchar)*(data1++)<<16) | ((uchar)*(data2++)<<8) | (uchar)*(data3++);
    else {
      if (normalize==1) { CImgStats st(img,false); min=(T)st.min; max=(T)st.max; }
      pmin = (T)min; mm = (T)max - pmin; if (mm==0) mm=1;
      for (uint y=ymin; y<=ymax; y++) cimg_mapX(*this,x) {
        const uchar
          val1 = (uchar)(255*(*(data1++)-pmin)/mm),
          val2 = (uchar)(255*(*(data2++)-pmin)/mm),
          val3 = (uchar)(255*(*(data3++)-pmin)/mm);
        *(ximg++) = (val1<<16) | (val2<<8) | (val3);
      }
    }    
    ReleaseMutex(mutex);
    return &bmi;
  }

  template<typename T> CImgDisplay& display(const CImg<T>& img,const uint pymin=0,const uint pymax=(uint)-1) {
    cimg_test(img);
    const uint 
      ymin = pymin<pymax?pymin:pymax,
      ymax = pymin<pymax?(pymax>=height?height-1:pymax):(pymin>=height?height-1:pymin);
    render(img,ymin,ymax);
    if (!closed) {
      WaitForSingleObject(mutex,INFINITE);
      SetDIBitsToDevice(hdc,0,ymin,width,ymax-ymin+1,0,0,0,ymax-ymin+1,data+ymin*width,&bmi,DIB_RGB_COLORS);
      ReleaseMutex(mutex);
    }
    return *this;
  }
  
  CImgDisplay& wait() {
    if (!closed && (event_type&2)) WaitForSingleObject(wait_disp,INFINITE);
    return *this;
  }

  CImgDisplay& show() {
    if (closed) {
      ShowWindow(window,SW_SHOW);
      SetDIBitsToDevice(hdc,0,0,width,height,0,0,0,height,data,&bmi,DIB_RGB_COLORS);
      closed = false;
    }
    return *this;
  }

  CImgDisplay& close() {
    if (!closed) {
      ShowWindow(window,SW_HIDE);
      closed = true;
    }
    return *this;
  }
#endif
  
};


/*-------------------------------------------------------



  
	Definition of the CImg<> structure
	
	
	
	
  ------------------------------------------------------*/

//! The basic CImg class, representing images, with pixels of type \p T.
template<typename T> struct CImg {
  uint width;                 //!< Width of the image (numbers of columns).
  uint height;                //!< Height of the image (number of rows).
  uint depth;                 //!< Depth of the image (number of z-slices).
  uint dim;                   //!< Dimension of the image (number of vector channels).
  T *data;                    //!< Pointer to pixel values (tab of size width*height*depth*data).
  
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Constructors - Destructor - Copy
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Create an image of size (\p dx,\p dy,\p dz,\p dv) with pixels of type \p T.
  explicit CImg(const uint dx=0,const uint dy=1,const uint dz=1,const uint dv=1):width(dx),height(dy),depth(dz),dim(dv) {
#if cimg_debug>1
    cimg::warn(dy==0 || dz==0 || dv==0,"CImg<%s>::CImg() : at least one null dimension = (%d,%d,%d,%d) have been passed to image constructor",
               pixel_type(),dx,dy,dz,dv);
#endif
    if (size()>0) data = new T[size()];
    else { data=NULL; width=height=depth=dim=0; }
  }
  //! Create an image of size (\p dx,\p dy,\p dz,\p dv) with pixels of type \p T, and fill the image with the value \p val.
  explicit CImg(const uint dx,const uint dy,const uint dz,const uint dv,const T& val):width(dx),height(dy),depth(dz),dim(dv) {
#if cimg_debug>1
    cimg::warn(dy==0 || dz==0 || dv==0,"CImg<%s>::CImg() : at least one null dimension = (%d,%d,%d,%d) have been passed to image constructor",
               pixel_type(),dx,dy,dz,dv);
#endif
    if (size()>0) data = new T[size()];
    else { data=NULL; width=height=depth=dim=0; }
    fill(val);
  }
  //! Copy constructor. If \p pixel_copy=false, the pixel data are not copied.
  template<typename t> CImg(const CImg<t>& img,const bool pixel_copy):width(img.width),height(img.height),depth(img.depth),dim(img.dim) {
    if (size()>0) {
      T *ptrd = data = new T[size()];
      if (pixel_copy) {
        t *ptrs = img.data+img.size();
        cimg_map(*this,ptrd) (*ptrd)=(T)*(--ptrs);
      }
    } else data = NULL;
  }
  //! Copy constructor.
  template<typename t> CImg(const CImg<t>& img):width(img.width),height(img.height),depth(img.depth),dim(img.dim) {
    if (size()>0) {
      T *ptrd = data = new T[size()];
      t *ptrs = img.data+img.size();
      cimg_map(*this,ptrd) (*ptrd)=(T)*(--ptrs);
    } else data = NULL;
  }
  //! Copy constructor (optimized for same pixel types). If \p pixel_copy=false, the pixel data are not copied.
  CImg(const CImg& img,const bool pixel_copy):width(img.width),height(img.height),depth(img.depth),dim(img.dim) {
    data = new T[width*height*depth*dim];
    if (pixel_copy) std::memcpy(data,img.data,size()*sizeof(T));
  }
  //! Copy constructor (optimized for same pixel types).
  CImg(const CImg<T>& img):width(img.width),height(img.height),depth(img.depth),dim(img.dim) {
    data = new T[width*height*depth*dim];
    std::memcpy(data,img.data,size()*sizeof(T));
  }
  //! Create an image by reading a filename.
  CImg(const char *filename):width(0),height(0),depth(0),dim(0),data(NULL) { load(filename).swap(*this); }

  //! Create an image from a data buffer.
  CImg(const T *const data_buffer,uint dx=1,uint dy=1,uint dz=1,uint dv=1):width(dx),height(dy),depth(dz),dim(dv) {
    if (data && dx*dy*dz*dv) {
      data = new T[dx*dy*dz*dv];
      std::memcpy(data,data_buffer,dx*dy*dz*dv*sizeof(T));
    } else { width=height=depth=dim=0; data = NULL; }
  }
  //! Destructor.
  ~CImg() { if (data) delete[] data; }

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Arithmetics and Boolean Operators
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Operator=
  template<typename t> CImg<T>& operator=(const CImg<t>& img) {
    const uint s = img.size();
    if (s) {
      if (s!=size()) { if (data) delete[] data; data = new T[s]; }
      width = img.width; height = img.height;	depth = img.depth; dim = img.dim;
      T *ptrd;
      t *ptrs = img.data + s;
      cimg_map(*this,ptrd) (*ptrd) = (T)(*(--ptrs));
    } else {
      if (data) delete[] data;
      width = height = depth = dim = 0;
      data = NULL;
    }
    return *this;
  }
  //! Operator=
  CImg& operator=(const CImg& img) {
    if (&img==this) return *this;
    const uint s = img.size();
    if (s) {
      if (s!=size()) { if (data) delete[] data; data = new T[s]; }
      width = img.width; height = img.height;	depth = img.depth; dim = img.dim;
      std::memcpy(data,img.data,s*sizeof(T));			
    } else {
      if (data) delete[] data;
      width = height = depth = dim = 0;
      data = NULL;
    }
    return *this;
  }
  //! Operator=
  CImg& operator=(const T& val) { return fill(val); }

  //! Operator=. Assign the value of each pixel of the current instance with the values of the given buffer \p buf.
  //! Note that \p buf must have at least the size of the current instance.
  CImg& operator=(const T *buf) {
    std::memcpy(data,buf,size()*sizeof(T));
    return *this;
  }
   
  //! Operator+=
  template<typename t> CImg& operator+=(const CImg<t>& img) {
    const uint sizemax = cimg::min(size(),img.size());
    t *ptrs = img.data+sizemax;
    T *ptrd;
    cimg_map(*this,ptrd) (*ptrd) = (T)((*ptrd)+(*(--ptrs)));
    return *this;
  }
  
  //! Operator-=
  template<typename t> CImg& operator-=(const CImg<t>& img) {
    const uint sizemax = cimg::min(size(),img.size());
    t *ptrs = img.data+sizemax;
    T *ptrd;
    cimg_map(*this,ptrd) (*ptrd) = (T)((*ptrd)-(*(--ptrs)));
    return *this;
  }
  


  //! Operator%=
  CImg& operator%=(const T& val)     { T *ptr; cimg_map(*this,ptr) (*ptr)%=val; return *this; }
  //! Operator+=
  CImg& operator+=(const T& val)     { T *ptr; cimg_map(*this,ptr) (*ptr)+=val; return *this; }
  //! Operator-=
  CImg& operator-=(const T& val)     { T *ptr; cimg_map(*this,ptr) (*ptr)-=val; return *this; }
  //! Operator*=
  CImg& operator*=(const double val) { T *ptr; cimg_map(*this,ptr) (*ptr)=(T)((*ptr)*val); return *this; }
  //! Operator/=
  CImg& operator/=(const double val) { T *ptr; cimg_map(*this,ptr) (*ptr)=(T)((*ptr)/val); return *this; }

  //! Operator+
  friend CImg operator+(const T& val, const CImg& img)    { return CImg<T>(img)+=val; }
  //! Operator*
  friend CImg operator*(const double val,const CImg &img) { return CImg<T>(img)*=val; }
  //! Operator-
  friend CImg operator-(const T& val, const CImg& img)    { return CImg<T>(img.width,img.height,img.depth,img.dim,val)-=img; }

  //! Operator%
  CImg operator%(const T& val)     const { return CImg<T>(*this)%=val; }  
  //! Operator+
  CImg operator+(const T& val)     const { return CImg<T>(*this)+=val; }
  //! Operator-
  CImg operator-(const T& val)     const { return CImg<T>(*this)-=val; }
  //! Operator*
  CImg operator*(const double val) const { return CImg<T>(*this)*=val; }
  //! Operator/
  CImg operator/(const double val) const { return CImg<T>(*this)/=val; }
  //! Operator+
  CImg operator+(const CImg& img)  const { return CImg<T>(*this)+=img; }
  //! Operator-
  CImg operator-(const CImg& img)  const { return CImg<T>(*this)-=img; }
    
  //!  Operator==
  template<typename t> const bool operator==(const CImg<t>& img) const {
    bool vequal = true;
    if (size()!=img.size()) return false;
    for (uint off=0; vequal && off<size(); off++) vequal = vequal && ((*this)[off]==img[off]);
    return vequal;
  }
  //! Operator!=
  template<typename t> const bool operator!=(const CImg<t>& img) const { return !((*this)==img); }
  
  //! Operator&= (bitwise AND)
  template<typename t> CImg& operator&=(const CImg<t>& img) {
    const uint sizemax = cimg::min(size(),img.size());
    t *ptrs = img.data+sizemax;
    T *ptrd;
    cimg_map(*this,ptrd) (*ptrd) = (T)((*ptrd)&(*(--ptrs)));
    return *this;
  }
  //! Operator& (bitwise AND)
  template<typename t> CImg operator&(const CImg<t>& img) const { return CImg<T>(*this)&=img; }
  
  //! Operator|= (bitwise OR)
  template<typename t> CImg& operator|=(const CImg<t>& img) {
    const uint sizemax = cimg::min(size(),img.size());
    t *ptrs = img.data+sizemax;
    T *ptrd;
    cimg_map(*this,ptrd) (*ptrd) = (T)((*ptrd)|(*(--ptrs)));
    return *this;
  }
  //! Operator| (bitwise OR)
  template<typename t> CImg operator|(const CImg<t>& img) const { return CImg<T>(*this)|=img; }
  
  //! Operator^= (bitwise XOR)
  template<typename t> CImg& operator^=(const CImg<t>& img) {
    const uint sizemax = cimg::min(size(),img.size());
    t *ptrs = img.data+sizemax;
    T *ptrd;
    cimg_map(*this,ptrd) (*ptrd) = (T)((*ptrd)^(*(--ptrs)));
    return *this;
  }
  //! Operator^ (bitwise XOR)
  template<typename t> CImg operator^(const CImg<t>& img) const { return CImg<T>(*this)^=img; }  

  //! Replace the image by the pointwise multiplication between \p *this and \p img.
  template<typename t> CImg& pmul(const CImg<t>& img) {
    t *ptrs = img.data;
    T *ptrf = data + cimg::min(size(),img.size());
    for (T* ptrd = data; ptrd<ptrf; ptrd++) (*ptrd)=(T)(*ptrd*(*(ptrs++)));
    return *this;
  }
  //! Return an image from a pointwise multiplication between \p *this and \p img.
  template<typename t> CImg get_pmul(const CImg<t>& img) const { return CImg<T>(*this).pmul(img); }
  
  //! Replace the image by the pointwise division between \p *this and \p img.
  template<typename t> CImg& pdiv(const CImg<t>& img) {
    t *ptrs = img.data;
    T *ptrf = data + cimg::min(size(),img.size());
    for (T* ptrd = data; ptrd<ptrf; ptrd++) (*ptrd)=(T)(*ptrd/(*(ptrs++)));
    return *this;
  }
  //! Return an image from a pointwise division between \p *this and \p img.
  template<typename t> CImg get_pdiv(const CImg<t>& img) const { return CImg<T>(*this).pdiv(img); }
  
  //! Replace the image by the pointwise max operator between \p *this and \p img
  template<typename t> CImg& max(const CImg<t>& img) {
    t *ptrs = img.data;
    T *ptrf = data + cimg::min(size(),img.size());
    for (T* ptrd = data; ptrd<ptrf; ptrd++) (*ptrd)=cimg::max((T)(*(ptrs++)),*ptrd);
    return *this;
  }
  //! Return the image corresponding to the max value for each pixel.
  template<typename t> CImg get_max(const CImg<t>& img) const { return CImg<T>(*this).max(img); }
  
  //! Replace the image by the pointwise min operator between \p *this and \p img
  template<typename t> CImg& min(const CImg<t>& img) {
    t *ptrs = img.data;
    T *ptrf = data + cimg::min(size(),img.size());
    for (T* ptrd = data; ptrd<ptrf; ptrd++) (*ptrd)=cimg::min((T)(*(ptrs++)),*ptrd);
    return *this;
  }
  //! Return the image corresponding to the min value for each pixel.
  template<typename t> CImg get_min(const CImg<t>& img) const { return CImg<T>(*this).min(img); }

  //! Replace each image pixel by its square root.
  CImg& sqrt() {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=(T)std::sqrt((double)(*ptr));
    return *this;
  }
  //! Return the image of the square root of the pixel values.
  CImg get_sqrt() const { return CImg<T>(*this).sqrt(); }
  
  //! Replace each image pixel by its power by \p p.
  CImg& pow(const double p) {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=(T)std::pow((double)(*ptr),p);
    return *this;
  }
  //! Return the image of the square root of the pixel values.
  CImg get_pow(const double p) const { return CImg<T>(*this).pow(p); }
  
  //! Replace each pixel value by its absolute value.
  CImg& abs() {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=cimg::abs(*ptr);
    return *this;
  }
  //! Return the image of the absolute value of the pixel values.
  CImg get_abs() const { return CImg<T>(*this).abs(); }
  
  //! Replace each image pixel by its cosinus.
  CImg& cos() {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=(T)std::cos((double)(*ptr));
    return *this;
  }
  //! Return the image of the cosinus of the pixel values.
  CImg get_cos() const { return CImg<T>(*this).cos(); }
 
  //! Replace each image pixel by its sinus.
  CImg& sin() {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=(T)std::sin((double)(*ptr));
    return *this;
  }
  //! Return the image of the sinus of the pixel values.
  CImg get_sin() const { return CImg<T>(*this).sin(); }
  
  //! Replace each image pixel by its tangent.
  CImg& tan() {
    T *ptr;
    cimg_map(*this,ptr) (*ptr)=(T)std::tan((double)(*ptr));
    return *this;
  }
  //! Return the image of the tangent of the pixel values.
  CImg get_tan() const { return CImg<T>(*this).tan(); }
  

 //@}
  //-----------------------------------------------------
  //-----------------------------------------------------
  //
  //! \name Access to image dimensions and pixel values
  //@{
  //-----------------------------------------------------
  //-----------------------------------------------------
  
  //! Return a string describing the type of the image pixels (template parameter \p T).
  static const char* pixel_type() { T val; return cimg::template_type(val); }

  //! Return the total number of pixels of an image (i.e. \ref width*\ref height*\ref depth*\ref dim)
  const uint size() const { return width*height*depth*dim; }  

  //! Return the width of the image (i.e. the field \ref width)
  const int dimx() const { return (int)width; }  

  //! Return the height of the image (i.e. the field \ref height)
  const int dimy() const { return (int)height; }
  
  //! Return the depth of the image (i.e. the field \ref depth)
  const int dimz() const { return (int)depth; }
  
  //! Return the vector dimension of the image (i.e. the field \ref dim)
  const int dimv() const { return (int)dim; }
  
  //! Return an offset corresponding to the location of the pixel value located at (\p x,\p y,\p z,\p v) with respect to the pixel data pointer \ref data
  const int offset(const int x=0, const int y=0, const int z=0, const int v=0) const { return x+width*(y+height*(z+depth*v)); }
  
  //! Return a pointer to the pixel value located at (\p x,\p y,\p z,\p v).
  T* ptr(const uint x=0, const uint y=0, const uint z=0, const uint v=0) const {
#if cimg_debug>1
    const int off = offset(x,y,z,v);
    if (off<0 || off>=(int)size()) {
      cimg::warn(true,"CImg<%s>::ptr() : Trying to get a pointer at (%d,%d,%d,%d) (offset=%d) which is outside the data of the image (%d,%d,%d,%d) (size=%d)",
		 pixel_type(),x,y,z,v,off,width,height,depth,dim,size());
      return data;
    }
#endif
    return data+offset(x,y,z,v);
  }
  //! Pixel access (without boundary checking, unless \p cimg_debug=2).
  T& operator()(const uint x,const uint y=0,const uint z=0,const uint v=0) const {
    const int off = offset(x,y,z,v);
#if cimg_debug>1
    if (!data || off>=(int)size()) {
      cimg::warn(true,
                 "CImg<%s>::operator() : Trying to get a pixel at (%d,%d,%d,%d) (offset=%d) outside the range of the image (%d,%d,%d,%d,%d) (size=%d)",
                 pixel_type(),x,y,z,v,offset(x,y,z,v),width,height,depth,dim,data,size());			
      return *data;
    }
#endif
    return data[off];
  }
  //! Buffer access (without boundary checking, unless \p cimg_debug=2).
  T& operator[](const uint off) const {
#if cimg_debug>1
    if (!data || off>=(int)size()) {
      cimg::warn(true,
                 "CImg<%s>::operator[] : Trying to get a pixel at offset=%d, outside the range of the image (%d,%d,%d,%d,%d) (size=%d)",
                 pixel_type(),off,width,height,depth,dim,data,size());			
      return *data;
    }
#endif
    return data[off];
  }

  //! Pixel access with Dirichlet boundary conditions.
  T dirichlet_pix4d(const int x,const int y=0,const int z=0,const int v=0,const T out_val=(T)0) const {
    return (x<0 || y<0 || z<0 || v<0 || x>=(int)width || y>=(int)height || z>=(int)depth || v>=(int)dim)?out_val:(*this)(x,y,z,v);
  }
  //! Pixel access with Dirichlet boundary conditions.
  T dirichlet_pix3d(const int x,const int y=0,const int z=0,const int v=0,const T out_val=(T)0) const {
    return (x<0 || y<0 || z<0 || x>=(int)width || y>=(int)height || z>=(int)depth)?out_val:(*this)(x,y,z,v);
  }
  //! Pixel access with Dirichlet boundary conditions.
  T dirichlet_pix2d(const int x,const int y=0,const int z=0,const int v=0,const T out_val=(T)0) const {
    return (x<0 || y<0 || x>=(int)width || y>=(int)height)?out_val:(*this)(x,y,z,v);
  }
  //! Pixel access with Dirichlet boundary conditions.
  T dirichlet_pix1d(const int x,const int y=0,const int z=0,const int v=0,const T out_val=(T)0) const {
    return (x<0 || x>=(int)width)?out_val:(*this)(x,y,z,v);
  }
  //! Pixel access with Neumann boundary conditions.
  const T& neumann_pix4d(const int x,const int y=0,const int z=0,const int v=0) const {
    return (*this)(x<0?0:(x>=(int)width?(int)width-1:x),
                   y<0?0:(y>=(int)height?(int)height-1:y),
                   z<0?0:(z>=(int)depth?(int)depth-1:z),
                   v<0?0:(v>=(int)dim?(int)dim-1:v));
  }
  //! Pixel access with Neumann boundary conditions.
  const T& neumann_pix3d(const int x,const int y=0,const int z=0,const int v=0) const {
    return (*this)(x<0?0:(x>=(int)width?(int)width-1:x),
                   y<0?0:(y>=(int)height?(int)height-1:y),
                   z<0?0:(z>=(int)depth?(int)depth-1:z),v);
  }
  //! Pixel access with Neumann boundary conditions.
  const T& neumann_pix2d(const int x,const int y=0,const int z=0,const int v=0) const {
    return (*this)(x<0?0:(x>=(int)width?(int)width-1:x),
                   y<0?0:(y>=(int)height?(int)height-1:y),z,v);
  }
  //! Pixel access with Neumann boundary conditions.
  const T& neumann_pix1d(const int x,const int y=0,const int z=0,const int v=0) const {
    return (*this)(x<0?0:(x>=(int)width?(int)width-1:x),y,z,v);
  }

  //! Pixel access with Neumann boundary conditions and linear interpolation.
  double linear_pix4d(const float ffx,const float ffy=0,const float ffz=0,const float ffv=0) const {
    double valx0,valx1,valy0,valy1,valz0,valz1;
    const float fx = ffx<0?0:(ffx>width-1?width-1:ffx), fy = ffy<0?0:(ffy>height-1?height-1:ffy),
      fz = ffz<0?0:(ffz>depth-1?depth-1:ffz), fv = ffv<0?0:(ffv>dim-1?dim-1:ffv);
    const uint x = (uint)fx, y = (uint)fy,  z = (uint)fz, v = (uint)fv;
    const float dx = fx-x, dy = fy-y, dz = fz-z, dv = fv-v;
    const uint nx = dx>0?x+1:x, ny = dy>0?y+1:y,  nz = dz>0?z+1:z, nv = dv>0?v+1:v;
    valx0 = (1-dx)*(*this)(x,y,z,v)  + (dx)*(*this)(nx,y,z,v);
    valx1 = (1-dx)*(*this)(x,ny,z,v) + (dx)*(*this)(nx,ny,z,v);
    valy0 = (1-dy)*valx0 + (dy)*valx1;
    valx0 = (1-dx)*(*this)(x,y,nz,v)  + (dx)*(*this)(nx,y,nz,v);
    valx1 = (1-dx)*(*this)(x,ny,nz,v) + (dx)*(*this)(nx,ny,nz,v);
    valy1 = (1-dy)*valx0 + (dy)*valx1;
    valz0 = (1-dz)*valy0 + (dz)*valy1;
    valx0 = (1-dx)*(*this)(x,y,z,nv)  + (dx)*(*this)(nx,y,z,nv);
    valx1 = (1-dx)*(*this)(x,ny,z,nv) + (dx)*(*this)(nx,ny,z,nv);
    valy0 = (1-dy)*valx0 + (dy)*valx1;
    valx0 = (1-dx)*(*this)(x,y,nz,nv)  + (dx)*(*this)(nx,y,nz,nv);
    valx1 = (1-dx)*(*this)(x,ny,nz,nv) + (dx)*(*this)(nx,ny,nz,nv);
    valy1 = (1-dy)*valx0 + (dy)*valx1;
    valz1 = (1-dz)*valy0 + (dz)*valy1;
    return (1-dv)*valz0 + (dv)*valz1;
  }
  //! Pixel access with Neumann boundary conditions and linear interpolation.
  double linear_pix3d(const float ffx,const float ffy=0,const float ffz=0,const int v=0) const {
    double valx0,valx1,valy0,valy1;
    const float fx = ffx<0?0:(ffx>width-1?width-1:ffx), fy = ffy<0?0:(ffy>height-1?height-1:ffy), fz = ffz<0?0:(ffz>depth-1?depth-1:ffz);
    const uint x = (uint)fx, y = (uint)fy, z = (uint)fz;
    const float dx = fx-x, dy = fy-y, dz = fz-z;
    const uint nx = dx>0?x+1:x, ny = dy>0?y+1:y, nz = dz>0?z+1:z;
    valx0 = (1-dx)*(*this)(x,y,z,v)  + (dx)*(*this)(nx,y,z,v);
    valx1 = (1-dx)*(*this)(x,ny,z,v) + (dx)*(*this)(nx,ny,z,v);
    valy0 = (1-dy)*valx0 + (dy)*valx1;
    valx0 = (1-dx)*(*this)(x,y,nz,v)  + (dx)*(*this)(nx,y,nz,v);
    valx1 = (1-dx)*(*this)(x,ny,nz,v) + (dx)*(*this)(nx,ny,nz,v);
    valy1 = (1-dy)*valx0 + (dy)*valx1;
    return (1-dz)*valy0 + (dz)*valy1;
  }
  //! Pixel access with Neumann boundary conditions and linear interpolation.
  double linear_pix2d(const float ffx,const float ffy=0,const int z=0,int v=0) const {
    double valx0,valx1;
    const float fx = ffx<0?0:(ffx>width-1?width-1:ffx), fy = ffy<0?0:(ffy>height-1?height-1:ffy);
    const uint x = (uint)fx, y = (uint)fy;
    const float dx = fx-x, dy = fy-y;
    const uint nx = dx>0?x+1:x, ny = dy>0?y+1:y;
    valx0 = (1-dx)*(*this)(x,y,z,v)  + (dx)*(*this)(nx,y,z,v);
    valx1 = (1-dx)*(*this)(x,ny,z,v) + (dx)*(*this)(nx,ny,z,v);
    return (1-dy)*valx0 + (dy)*valx1;
  }
  //! Pixel access with Neumann boundary conditions and linear interpolation.
  double linear_pix1d(const float ffx,const int y=0,const int z=0,int v=0) const {
    const float fx = ffx<0?0:(ffx>width-1?width-1:ffx);
    const uint x = (uint)fx;
    const float dx = fx-x;
    const uint nx = dx>0?x+1:x;
    return (1-dx)*(*this)(x,y,z,v)  + (dx)*(*this)(nx,y,z,v);
  }

  //! Pixel access with Neumann boundary conditions and cubic interpolation.
  double cubic_pix2d(const float pfx,const float pfy=0,const int z=0,int v=0) const {
    const float fx = pfx<0?0:(pfx>width-1?width-1:pfx), fy = pfy<0?0:(pfy>height-1?height-1:pfy);
    const uint 
      x = (uint)fx,  px = (int)x-1>=0?x-1:0, nx = x+1<width?x+1:width-1, ax = nx+1<width?nx+1:width-1,
      y = (uint)fy,  py = (int)y-1>=0?y-1:0, ny = y+1<height?y+1:height-1, ay = ny+1<height?ny+1:height-1;
    const float dx = fx-x, dy = fy-y;
    const T& 
      a = (*this)(px,py,z,v), b = (*this)(x,py,z,v), c = (*this)(nx,py,z,v), d = (*this)(ax,py,z,v),
      e = (*this)(px, y,z,v), f = (*this)(x, y,z,v), g = (*this)(nx, y,z,v), h = (*this)(ax, y,z,v),
      i = (*this)(px,ny,z,v), j = (*this)(x,ny,z,v), k = (*this)(nx,ny,z,v), l = (*this)(ax,ny,z,v),
      m = (*this)(px,ay,z,v), n = (*this)(x,ay,z,v), o = (*this)(nx,ay,z,v), p = (*this)(ax,ay,z,v);
    const double 
      A = dx*dx*dx*(2*(b-c)+0.5*(c-a+d-b)) + dx*dx*(2*c-2.5*b+a-0.5*d) + dx*0.5*(c-a) + b,
      B = dx*dx*dx*(2*(f-g)+0.5*(g-e+h-f)) + dx*dx*(2*g-2.5*f+e-0.5*h) + dx*0.5*(g-e) + f,
      C = dx*dx*dx*(2*(j-k)+0.5*(k-i+l-j)) + dx*dx*(2*k-2.5*j+i-0.5*l) + dx*0.5*(k-i) + j,
      D = dx*dx*dx*(2*(n-o)+0.5*(o-m+p-n)) + dx*dx*(2*o-2.5*n+m-0.5*p) + dx*0.5*(o-m) + n;
    return dy*dy*dy*(2*(B-C)+0.5*(C-A+D-B)) + dy*dy*(2*C-2.5*B+A-0.5*D) + dy*0.5*(C-A) + B;
  }
  
  //! Pixel access with Neumann boundary conditions and cubic interpolation.
  double cubic_pix1d(const float pfx,const int y=0,const int z=0,int v=0) const {
    const float fx = pfx<0?0:(pfx>width-1?width-1:pfx);
    const uint x = (uint)fx, px = (int)x-1>=0?x-1:0, nx = x+1<width?x+1:width-1, ax = nx+1<width?nx+1:width-1;
    const float dx = fx-x;
    const T& a = (*this)(px,y,z,v), b = (*this)(x,y,z,v), c = (*this)(nx,y,z,v), d = (*this)(ax,y,z,v);
    return dx*dx*dx*(2*(b-c)+0.5*(c-a+d-b)) + dx*dx*(2*c-2.5*b+a-0.5*d) + dx*0.5*(c-a) + b;
  }
  
  //! Compute and return statistics on the image.
  CImgStats stats(const bool compute_variance=true) const { return CImgStats(*this,compute_variance); }
  
  //! Print image information on the standart error output
  const CImg& print(const char *title=NULL,const uint print_flag=1) const {
    std::fprintf(stderr,"%-8s(%p) : (%d,%d,%d,%d,%p) : ",title?title:"CImg",this,width,height,depth,dim,data);
    if (size()==0 || !data) { std::fprintf(stderr,"Undefined pixel data\n"); return *this; }
    if (print_flag>=1) { 
      CImgStats st(*this);
      std::fprintf(stderr,"stats = { %g, %g [%g], %g } : ",st.min,st.mean,st.variance,st.max); 
    }
    if (print_flag>=2 || size()<=16) {
      std::fprintf(stderr,"%s = [ ",title?title:"data");
      cimg_mapXYZV(*this,x,y,z,k) std::fprintf(stderr,"%g%s",(double)(*this)(x,y,z,k),((x+1)*(y+1)*(z+1)*(k+1)==(int)size()?" ]":(((x+1)%width==0)?" ; ":" ")));
    }
    std::fputc('\n',stderr);
    return *this;
  }
  //! Same as above, without the first parameter.
  const CImg& print(const uint print_flag) const { return print(NULL,print_flag); }
  
  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Basic Image Transformation
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Fill all pixel values with value \a val.
  CImg& fill(const T& val0=(T)0) {
    cimg_test(*this);
    T *ptr;
    if (val0!=0) cimg_map(*this,ptr) *ptr=val0; else std::memset(data,0,size()*sizeof(T));
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1.
  CImg& fill(const T& val0,const T& val1) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=2) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=2) *ptr=val1;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2.
  CImg& fill(const T& val0,const T& val1,const T& val2) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=3) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=3) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=3) *ptr=val2;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3.
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=4) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=4) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=4) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=4) *ptr=val3;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3 and \a val4.
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3,const T& val4) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=5) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=5) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=5) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=5) *ptr=val3;
    for (ptr=data+4; ptr<ptr_end; ptr+=5) *ptr=val4;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3 and \a val4 and \a val5
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3,const T& val4,const T& val5) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=6) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=6) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=6) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=6) *ptr=val3;
    for (ptr=data+4; ptr<ptr_end; ptr+=6) *ptr=val4;
    for (ptr=data+5; ptr<ptr_end; ptr+=6) *ptr=val5;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3 and \a ... and \a val8.
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3,const T& val4,const T& val5,const T& val6,const T& val7,const T& val8) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=9) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=9) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=9) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=9) *ptr=val3;
    for (ptr=data+4; ptr<ptr_end; ptr+=9) *ptr=val4;
    for (ptr=data+5; ptr<ptr_end; ptr+=9) *ptr=val5;
    for (ptr=data+6; ptr<ptr_end; ptr+=9) *ptr=val6;
    for (ptr=data+7; ptr<ptr_end; ptr+=9) *ptr=val7;
    for (ptr=data+8; ptr<ptr_end; ptr+=9) *ptr=val8;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3 and \a ... and \a val11.
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3,const T& val4,const T& val5,const T& val6,
             const T& val7,const T& val8,const T& val9,const T& val10,const T& val11) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=12) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=12) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=12) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=12) *ptr=val3;
    for (ptr=data+4; ptr<ptr_end; ptr+=12) *ptr=val4;
    for (ptr=data+5; ptr<ptr_end; ptr+=12) *ptr=val5;
    for (ptr=data+6; ptr<ptr_end; ptr+=12) *ptr=val6;
    for (ptr=data+7; ptr<ptr_end; ptr+=12) *ptr=val7;
    for (ptr=data+8; ptr<ptr_end; ptr+=12) *ptr=val8;
    for (ptr=data+9; ptr<ptr_end; ptr+=12) *ptr=val9;
    for (ptr=data+10; ptr<ptr_end; ptr+=12) *ptr=val10;
    for (ptr=data+11; ptr<ptr_end; ptr+=12) *ptr=val11;
    return *this;
  }
  //! Fill sequentially all pixel values with values \a val0 and \a val1 and \a val2 and \a val3 and \a ... and \a val15.
  CImg& fill(const T& val0,const T& val1,const T& val2,const T& val3,const T& val4,const T& val5,const T& val6,const T& val7,
             const T& val8,const T& val9,const T& val10,const T& val11,const T& val12,const T& val13,const T& val14,const T& val15) {
    cimg_test(*this);
    T *ptr, *ptr_end = data+size();
    for (ptr=data;   ptr<ptr_end; ptr+=16) *ptr=val0;
    for (ptr=data+1; ptr<ptr_end; ptr+=16) *ptr=val1;
    for (ptr=data+2; ptr<ptr_end; ptr+=16) *ptr=val2;
    for (ptr=data+3; ptr<ptr_end; ptr+=16) *ptr=val3;
    for (ptr=data+4; ptr<ptr_end; ptr+=16) *ptr=val4;
    for (ptr=data+5; ptr<ptr_end; ptr+=16) *ptr=val5;
    for (ptr=data+6; ptr<ptr_end; ptr+=16) *ptr=val6;
    for (ptr=data+7; ptr<ptr_end; ptr+=16) *ptr=val7;
    for (ptr=data+8; ptr<ptr_end; ptr+=16) *ptr=val8;
    for (ptr=data+9; ptr<ptr_end; ptr+=16) *ptr=val9;
    for (ptr=data+10; ptr<ptr_end; ptr+=16) *ptr=val10;
    for (ptr=data+11; ptr<ptr_end; ptr+=16) *ptr=val11;
    for (ptr=data+12; ptr<ptr_end; ptr+=16) *ptr=val12;
    for (ptr=data+13; ptr<ptr_end; ptr+=16) *ptr=val13;
    for (ptr=data+14; ptr<ptr_end; ptr+=16) *ptr=val14;
    for (ptr=data+15; ptr<ptr_end; ptr+=16) *ptr=val15;
    return *this;
  }
  
  //! Linear normalization of the pixel values between \a a and \a b.
  CImg& normalize(const T& a,const T& b) {
    cimg_test(*this);
    const CImgStats st(*this,false);
    if (st.min==st.max) fill(0);
    else {
      T *ptr;
      cimg_map(*this,ptr) *ptr=(T)((*ptr-st.min)/(st.max-st.min)*(b-a)+a);
    }
    return *this;
  }
  //! Return the image of normalized values.
  CImg get_normalize(const T& a,const T& b) const { return CImg<T>(*this).normalize(a,b); }
  
  //! Cut pixel values between \a a and \a b.
  CImg& cut(const T& a, const T& b) {
    cimg_test(*this);
    T *ptr;
    cimg_map(*this,ptr) *ptr = (*ptr<a)?a:((*ptr>b)?b:*ptr);
    return *this;
  }
  //! Return the image of cutted values.
  CImg get_cut(const T& a, const T& b) const { return CImg<T>(*this).cut(a,b); }
  //! Quantify pixel values into \n levels.
  CImg& quantify(const uint n=256) {
    cimg_test(*this);
    const CImgStats st(*this,false);
    const double range = st.max-st.min;
    T *ptr;
    cimg_map(*this,ptr) *ptr = (T)(st.min + range*(int)((*ptr-st.min)*(int)n/range)/n);
    return *this;
  }
  //! Return a quantified image, with \n levels.
  CImg get_quantify(const uint n=256) const { return CImg<T>(*this).quantify(n); }
  //! Threshold the image.
  CImg& threshold(const T& thres) {
    cimg_test(*this);
    T *ptr;
    cimg_map(*this,ptr) *ptr = *ptr<=thres?(T)0:(T)1;
    return *this;
  }
  //! Return a thresholded image.
  CImg get_threshold(const T& thres) const { return CImg<T>(*this).threshold(thres); }
  
  //! Return a rotated image.
  CImg get_rotate(const float angle,const float pcx,const float pcy,const float zoom=1,const uint pcond=5) const {
    cimg_test(*this);
    const float rad = (float)(angle*cimg::PI/180.0), ca=(float)std::cos(rad)/zoom,sa=(float)std::sin(rad)/zoom;
    float w2,h2,cx=pcx,cy=pcy;
    uint cond=pcond;
    CImg dest;
    if (cond>=3) {
      w2 = 0.5f*zoom*width;
      h2 = 0.5f*zoom*height;
      dest = CImg<T>((int)(2*cimg::max(std::fabs((double)(ca*w2-sa*h2)),std::fabs((double)(ca*w2+sa*h2)))),
                     (int)(2*cimg::max(std::fabs((double)(sa*w2-ca*h2)),std::fabs((double)(sa*w2+ca*h2)))),
                     depth,dim);
      w2 = 0.5f*dest.width;
      h2 = 0.5f*dest.height;
      cx = 0.5f*width;
      cy = 0.5f*height;
      cond-=3;
    }
    else {
      dest = CImg<T>(*this,false);
      w2 = cx;
      h2 = cy;
    }
    switch (cond) {
    case 0:                 // zero filling with bloc interpolation
      { cimg_mapXY(dest,x,y) {
          cimg_mapZV(*this,z,v) dest(x,y,z,v) = dirichlet_pix2d((int)(cx + (x-w2)*ca + (y-h2)*sa),(int)(cy - (x-w2)*sa + (y-h2)*ca),z,v);
        }}
      break;
    case 1:               // mosaic filling
      { cimg_mapXY(dest,x,y) {
          cimg_mapZV(*this,z,v) dest(x,y,z,v) = (*this)(cimg::modi((int)(cx + (x-w2)*ca + (y-h2)*sa),width),
                                                        cimg::modi((int)(cy - (x-w2)*sa + (y-h2)*ca),height),z,v);
        }}
      break;
    default:               // zero filling with linear interpolation
      { cimg_mapXY(dest,x,y) {
          const float X = cx + (x-w2)*ca + (y-h2)*sa, Y = cy - (x-w2)*sa + (y-h2)*ca;
          const int ix = (int)X, iy = (int)Y;
          if (ix<0 || ix>=(int)width || iy<0 || iy>=(int)height) cimg_mapZV(*this,z,v) dest(x,y,z,v) = 0;
          else cimg_mapZV(*this,z,v) dest(x,y,z,v) = (T)linear_pix2d(X,Y,z,v);
        }}
      break;
    }
    return dest;
  }
  //! Return a rotated image.
  CImg get_rotate(const float angle,const uint cond=5) const { return get_rotate(angle,0.5f*width,0.5f*height,1,cond); }
  
  //! Rotate the image.
  CImg& rotate(const float angle,const float cx,const float cy,const float zoom=1,const uint cond=5) {
    return get_rotate(angle,cx,cy,zoom,cond).swap(*this);
  }
  //! Rotate the image.
  CImg& rotate(const float angle,const uint cond=5) { return rotate(angle,0.5f*width,0.5f*height,1,cond); }

  //! Return a rotated image, with an angle of 90 degrees
  CImg get_rotate90(const bool resize_flag=true) {
    cimg_test(*this);
    CImg res = resize_flag?CImg<T>(height,width,depth,dim):CImg<T>(width,height,depth,dim).fill(0);
    if (resize_flag) cimg_mapXYZV(res,x,y,z,v) res(x,y,z,v) = (*this)(y,height-1-x,z,v);
    else {
      const uint
        xmin = cimg::max(0,((int)width-(int)height)/2), xmax = cimg::min(width,xmin+height),
        ymin = cimg::max(0,((int)height-(int)width)/2), ymax = cimg::min(height,ymin+width),
        xoff = xmin + cimg::min(0,((int)width-(int)height)/2),
        yoff = ymin + cimg::min(0,((int)height-(int)width)/2);
      cimg_mapZV(res,z,v) for (uint y=ymin; y<ymax; y++) for (uint x=xmin; x<xmax; x++)
        res(x,y,z,v) = (*this)(y-yoff,height-1-x+xoff,z,v);
    }
    return res;
  }
  //! Return a rotated image, with an angle of 270 degrees
  CImg get_rotate270(const bool resize_flag=true) {
    cimg_test(*this);
    CImg res = resize_flag?CImg<T>(height,width,depth,dim):CImg<T>(width,height,depth,dim).fill(0);
    if (resize_flag) cimg_mapXYZV(res,x,y,z,v) res(x,y,z,v) = (*this)(width-1-y,x,z,v);
    else {
      const uint
        xmin = cimg::max(0,((int)width-(int)height)/2), xmax = cimg::min(width,xmin+height),
        ymin = cimg::max(0,((int)height-(int)width)/2), ymax = cimg::min(height,ymin+width),
        xoff = xmin + cimg::min(0,((int)width-(int)height)/2),
        yoff = ymin + cimg::min(0,((int)height-(int)width)/2);
      cimg_mapZV(res,z,v) for (uint y=ymin; y<ymax; y++) for (uint x=xmin; x<xmax; x++)
        res(x,y,z,v) = (*this)(width-1-y+yoff,x-xoff,z,v);
    }
    return res;
  }
  //! Rotate the image, with an angle of 90 degrees
  CImg& rotate90(const bool resize_flag=true) { return get_rotate90(resize_flag).swap(*this); }
  //! Rotate the image, with an angle of 270 degrees
  CImg& rotate270(const bool resize_flag=true) { return get_rotate270(resize_flag).swap(*this); }

  //! Rotate the image, with an angle of 180 degrees
  CImg& rotate180() {
    cimg_test(*this);
    cimg_mapZV(*this,z,v) {
      T *ptr1 = ptr(0,0,z,v), *ptr2 = ptr1+width*height;
      while (ptr1<ptr2) { const T val = *ptr1; *(ptr1++)=*(--ptr2); *ptr2=val; }
    }
    return *this;
  }

  //! Return a rotated image, with an angle of 180 degrees
  CImg get_rotate180() { return CImg<T>(*this).rotate180(); }

  //! Return a resized image.\n
  //! Parameters \p pdx, \p pdy, \p pdz, \p pdv sets the new size along each of the axes x,y,z and v.
  //! If negative, it corresponds to a percentage of the original size (the default value is -100).\n
  //! Parameter \p interp sets the interpolation type (0=no interpolation, 1=bloc, 2=mosaic, 3=linear, 4=grid, 5=cubic).  
  CImg get_resize(const int pdx=-100,const int pdy=-100,const int pdz=-100,const int pdv=-100,const uint interp=1) const {
    cimg_test(*this);
    const uint dx = pdx<0?-pdx*width/100:pdx, dy = pdy<0?-pdy*height/100:pdy,dz = pdz<0?-pdz*depth/100:pdz,dv = pdv<0?-pdv*dim/100:pdv;
    CImg res(dx?dx:1,dy?dy:1,dz?dz:1,dv?dv:1);
    if (width==res.width && height==res.height && depth==res.depth && dim==res.dim) return *this;
    const float sx = (float)width/res.width, sy = (float)height/res.height, sz = (float)depth/res.depth, sk = (float)dim/res.dim;

    switch (interp) {
    case 0:                 // 0 filling
      {
        uint w = cimg::min(dx,width), h = cimg::min(dy,height), d = cimg::min(dz,depth), v = cimg::min(dv,dim);
        T *ptr = data;
        w*=sizeof(T);
        res.fill(0);
        for (uint k=0; k<v; k++) for (uint z=0; z<d; z++) for (uint y=0; y<h; y++) { std::memcpy(res.ptr(0,y,z,k),ptr,w); ptr+=width; }
      }
      break;
    case 1:               // bloc interpolation
      {
        float cx,cy,cz,ck=0;
        cimg_mapV(res,k) { cz = 0; 
          cimg_mapZ(res,z) { cy = 0; 
            cimg_mapY(res,y) { cx = 0; 
              cimg_mapX(res,x) { res(x,y,z,k) = (*this)((uint)cx,(uint)cy,(uint)cz,(uint)ck); cx+=sx;
              } cy+=sy;
            } cz+=sz;
          } ck+=sk;
        }
      }
      break;
    case 2:               // mosaic filling
      {
        cimg_mapXYZV(res,x,y,z,k) res(x,y,z,k) = (*this)(x%width,y%height,z%depth,k%dim);
      }
      break;
    case 3:               // linear interpolation
      {
        float cx,cy,cz,ck = 0;
        cimg_mapV(res,k) { cz = 0; 
          cimg_mapZ(res,z) { cy = 0;
            cimg_mapY(res,y) { cx = 0; 
              cimg_mapX(res,x) { res(x,y,z,k) = (T)linear_pix4d(cx,cy,cz,ck); cx+=sx;
              } cy+=sy;
            } cz+=sz;
          } ck+=sk;
        }
      }
      break;
    case 4:              // grid filling
      {
        res.fill(0);
        cimg_mapXYZV(*this,x,y,z,k) res((int)(x/sx),(int)(y/sy),(int)(z/sz),(int)(k/sk)) = (*this)(x,y,z,k);
      }
      break;
    case 5:             // cubic interpolation
      {
        float cx,cy,cz,ck = 0;
        cimg_mapV(res,k) { cz = 0;
          cimg_mapZ(res,z) { cy = 0;
            cimg_mapY(res,y) { cx = 0;
              cimg_mapX(res,x) { res(x,y,z,k) = (T)cubic_pix2d(cx,cy,(int)cz,(int)ck); cx+=sx;
              } cy+=sy;
            } cz+=sz;
          } ck+=sk;
        }
      }
      break;      
    }
    return res;
  }
  //! Return a resized image.
  CImg get_resize(const CImg& src,const uint interp=1) const { return get_resize(src.width,src.height,src.depth,src.dim,interp); }
  
  //! Resize the image.
  CImg& resize(const int pdx=-100,const int pdy=-100,const int pdz=-100,const int pdv=-100,const uint interp=1) {
    const uint
      dx = pdx<0?-pdx*width/100 :(pdx==0?1:pdx),
      dy = pdy<0?-pdy*height/100:(pdy==0?1:pdy),
      dz = pdz<0?-pdz*depth/100 :(pdz==0?1:pdz),
      dv = pdv<0?-pdv*dim/100   :(pdv==0?1:pdv);
    if (width==dx && height==dy && depth==dz && dim==dv) return *this;
    else return get_resize(dx,dy,dz,dv,interp).swap(*this);
  }
  //! Resize the image.
  CImg& resize(const CImg& src,const uint interp=1) { return resize(src.width,src.height,src.depth,src.dim,interp); }

  //! Return an half-resized image, using a special filter.
  CImg get_resize_halfXY() const {
    cimg_test(*this);
    const float mask[3][3] = { {
        0.07842776544f, 0.1231940459f, 0.07842776544f
      },
                               { 0.1231940459f,  0.1935127547f, 0.1231940459f },
                               { 0.07842776544f, 0.1231940459f, 0.07842776544f }
    };
    CImg_3x3(I,float);
    CImg dest(width/2,height/2,depth,dim);
    cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) dest(x/2,y/2,z,k) = (T)cimg_conv3x3(I,mask);
    return dest;
  }
  //! Half-resize the image, using a special filter
  CImg& resize_halfXY() {	return get_resize_halfXY().swap(*this); }

  //! Return a square region of the image, as a new image
  CImg get_sprite(const uint x0,const uint y0,const uint z0,const uint v0,const uint x1,const uint y1,const uint z1,const uint v1) const {
    cimg_test(*this);
    if (x0>=width || x1>=width || y0>=height || y1>=height || z0>=depth || z1>=depth || v0>=dim || v1>=dim || x1<x0 || y1<y0 || z1<z0 || v1<v0)
      throw CImgArgumentException("CImg<%s>::get_sprite() : Bad sprite coordinates (%d,%d,%d,%d)-(%d,%d,%d,%d) in image (%d,%d,%d,%d)",
                                  pixel_type(),x0,y0,z0,v0,x1,y1,z1,v1,width,height,depth,dim);
    const uint dx=x1-x0+1, dy=y1-y0+1, dz=z1-z0+1, dv=v1-v0+1;
    CImg dest(dx,dy,dz,dv);
    T *psrc = ptr(x0,y0,z0,v0), *pdest = dest.ptr(0,0,0,0);
    if (dx!=width)
      for (uint k=0; k<dv; k++) {
        for (uint z=0; z<dz; z++) {
          for (uint y=0; y<dy; y++) {
            std::memcpy(pdest,psrc,dx*sizeof(T));
            pdest+=dx;
            psrc+=width;
          }
          psrc+=width*(height-dy);
        }
        psrc+=width*height*(depth-dz);
      }
    else {
      if (dy!=height)         
        for (uint k=0; k<dv; k++) {
          for (uint z=0; z<dz; z++) {
            std::memcpy(pdest,psrc,dx*dy*sizeof(T));
            pdest+=dx*dy;
            psrc+=width*height;
          }
          psrc+=width*height*(depth-dz);
        }
      else {
        if (dz!=depth)
          for (uint k=0; k<dv; k++) {
            std::memcpy(pdest,psrc,dx*dy*dz*sizeof(T));
            pdest+=dx*dy*dz;
            psrc+=width*height*depth;
          }
        else std::memcpy(pdest,psrc,dx*dy*dz*dv*sizeof(T));
      }
    }
    return dest;
  }
  //! Return a square region of the image, as a new image
  CImg get_sprite(const uint x0,const uint y0,const uint z0,const uint x1,const uint y1,const uint z1) const {
    return get_sprite(x0,y0,z0,0,x1,y1,z1,dim-1);
  }
  //! Return a square region of the image, as a new image
  CImg get_sprite(const uint x0,const uint y0,const uint x1,const uint y1) const {
    return get_sprite(x0,y0,0,0,x1,y1,depth-1,dim-1);
  }
  //! Return a square region of the image, as a new image
  CImg get_sprite(const uint x0,const uint x1) const { return get_sprite(x0,0,0,0,x1,height-1,depth-1,dim-1); }

  //! Replace the image by a square region of the image
  CImg& sprite(const uint x0,const uint y0,const uint z0,const uint v0,const uint x1,const uint y1,const uint z1,const uint v1) {
    return get_sprite(x0,y0,z0,v0,x1,y1,z1,v1).swap(*this);
  }
  //! Replace the image by a square region of the image
  CImg& sprite(const uint x0,const uint y0,const uint z0,const uint x1,const uint y1,const uint z1) {
    return sprite(x0,y0,z0,0,x1,y1,z1,dim-1);
  }
  //! Replace the image by a square region of the image
  CImg& sprite(const uint x0,const uint y0,const uint x1,const uint y1) { return sprite(x0,y0,0,0,x1,y1,depth-1,dim-1); }

  //! Replace the image by a square region of the image
  CImg& sprite(const uint x0,const uint x1) { return sprite(x0,0,0,0,x1,height-1,depth-1,dim-1); }

  //! Get the channel \a v of the current image, as a new image.
  CImg get_channel(const uint v0=0) const { return get_sprite(0,0,0,v0,width-1,height-1,depth-1,v0); }

  //! Get the z-slice \a z of *this, as a new image.
  CImg get_slice(const uint z0=0) const { return get_sprite(0,0,z0,0,width-1,height-1,z0,dim-1); }

  //! Get the z-slice \a z of the channel \a v of the current image, as a new image.
  CImg get_plane(const uint z0=0,const uint v0=0) const { return get_sprite(0,0,z0,v0,width-1,height-1,z0,v0); }

  //! Return a reference to a set of points (x0->x1,y0,z0,v0) of the image. Use it carefully !
  CImgROI<T> ref_pointset(const uint xmin,const uint xmax,const uint y0=0,const uint z0=0,const uint v0=0) const {
    cimg_test(*this);
    if (xmax<xmin || xmax>=width || y0>=height || z0>=depth || v0>=dim)
      throw CImgArgumentException("CImg<%s>::ref_pointset() : Cannot return a reference (%d->%d,%d,%d,%d) from a (%d,%d,%d,%d) image",
                                  pixel_type(),xmin,xmax,y0,z0,v0,width,height,depth,dim);
    return CImgROI<T>(1+xmax-xmin,1,1,1,ptr(xmin,y0,z0,v0));
  }

  //! Return a reference to a set of lines (y0->y1,z0,v0) of the image. Use it carefully !
  CImgROI<T> ref_lineset(const uint ymin,const uint ymax,const uint z0=0,const uint v0=0) const {
    cimg_test(*this);
    if (ymax<ymin || ymax>=height || z0>=depth || v0>=dim)
      throw CImgArgumentException("CImg<%s>::ref_lineset() : Cannot return a reference (0->%d,%d->%d,%d,%d) from a (%d,%d,%d,%d) image",
                                  pixel_type(),width-1,ymin,ymax,z0,v0,width,height,depth,dim);
    return CImgROI<T>(width,1+ymax-ymin,1,1,ptr(0,ymin,z0,v0));
  }
  
  //! Return a reference to a set of planes (z0->z1,v0) of the image. Use it carefully !
  CImgROI<T> ref_planeset(const uint zmin,const uint zmax,const uint v0=0) const {
    cimg_test(*this);
    if(zmax<zmin || zmax>=depth || v0>=dim)
      throw CImgArgumentException("CImg<%s>::ref_planeset() : Cannot return a reference (0->%d,0->%d,%d->%d,%d) from a (%d,%d,%d,%d) image",
                                  pixel_type(),width-1,height-1,zmin,zmax,v0,width,height,depth,dim);
    return CImgROI<T>(width,height,1+zmax-zmin,1,ptr(0,0,zmin,v0));
  }

  //! Return a reference to a set of channels (v0->v1) of the image. Use it carefully !
  CImgROI<T> ref_channelset(const uint vmin,const uint vmax) const {
    cimg_test(*this);
    if (vmax<vmin || vmax>=dim)
      throw CImgArgumentException("CImg<%s>::ref_channelset() : Cannot return a reference (0->%d,0->%d,0->%d,%d->%d) from a (%d,%d,%d,%d) image",
                                  pixel_type(),width-1,height-1,depth-1,vmin,vmax,width,height,depth,dim);
    return CImgROI<T>(width,height,depth,1+vmax-vmin,ptr(0,0,0,vmin));
  }
  
  //! Return a reference to a line (y0,z0,v0) of the image. Use it carefully !
  CImgROI<T> ref_line(const uint y0,const uint z0=0,const uint v0=0) const { return ref_pointset(0,width-1,y0,z0,v0); }

  //! Return a reference to a plane (z0,v0) of the image. Use it carefully !
  CImgROI<T> ref_plane(const uint z0,const uint v0=0) const { return ref_lineset(0,height-1,z0,v0); }

  //! Return a reference to a channel (v0) of the image. Use it carefully !
  CImgROI<T> ref_channel(const uint v0) const { return ref_planeset(0,depth-1,v0); }

  //! Replace the image by one of its channel
  CImg& channel(const uint v0=0) { return get_channel(v0).swap(*this); }

  //! Replace the image by one of its z-slice
  CImg& slice(const uint z0=0) { return get_slice(z0).swap(*this); }

  //! Replace the image by one of its plane
  CImg& plane(const uint z0=0, const uint v0=0) { return get_plane(z0,v0).swap(*this); }
  
  //! Flip an image along the specified axis
  CImg& flip(const char axe='x') {
    cimg_test(*this);
    T *pf,*pb,*buf=NULL;
    switch (axe) {
    case 'x':
      {
        pf = ptr(); pb = ptr(width-1);
        for (uint yzv=0; yzv<height*depth*dim; yzv++) { 
          for (uint x=0; x<width/2; x++) { const T val = *pf; *(pf++)=*pb; *(pb--)=val; }
          pf+=width-width/2;
          pb+=width+width/2;
        }
      }
      break;
    case 'y':
      {
        buf = new T[width];
        pf = ptr(); pb = ptr(0,height-1);
        for (uint zv=0; zv<depth*dim; zv++) {
          for (uint y=0; y<height/2; y++) {
            std::memcpy(buf,pf,width*sizeof(T));
            std::memcpy(pf,pb,width*sizeof(T));
            std::memcpy(pb,buf,width*sizeof(T));
            pf+=width;
            pb-=width;
          }
          pf+=width*(height-height/2);
          pb+=width*(height+height/2);
        }
      }
      break;
    case 'z':
      {
        buf = new T[width*height];
        pf = ptr(); pb = ptr(0,0,depth-1);
        cimg_mapV(*this,v) {
          for (uint z=0; z<depth/2; z++) {
            std::memcpy(buf,pf,width*height*sizeof(T));
            std::memcpy(pf,pb,width*height*sizeof(T));
            std::memcpy(pb,buf,width*height*sizeof(T));
            pf+=width*height;
            pb-=width*height;
          }
          pf+=width*height*(depth-depth/2);
          pb+=width*height*(depth+depth/2);
        }
      }
      break;
    case 'v':
      {
        buf = new T[width*height*depth];
        pf = ptr(); pb = ptr(0,0,0,dim-1);
        for (uint v=0; v<dim/2; v++) {
          std::memcpy(buf,pf,width*height*depth*sizeof(T));
          std::memcpy(pf,pb,width*height*depth*sizeof(T));
          std::memcpy(pb,buf,width*height*depth*sizeof(T));
          pf+=width*height*depth;
          pb-=width*height*depth;
        }
      }
      break;
    default: cimg::warn(true,"CImg<%s>::flip() : unknow axe '%c', should be 'x','y','z' or 'v'",pixel_type(),axe);
    }
    if (buf) delete[] buf;
    return *this;
  }
  //! Get a flipped version of the image, along the specified axis
  CImg get_flip(const char axe='x') { return CImg<T>(*this).flip(axe); }
    
  //! Return a 2D representation of a 3D image, with three slices.
  CImg get_3dplanes(const uint px0,const uint py0,const uint pz0) const {
    cimg_test(*this);
    const uint
      x0=(px0>=width)?width-1:px0,
      y0=(py0>=height)?height-1:py0,
      z0=(pz0>=depth)?depth-1:pz0;
    CImg res(width+depth,height+depth,1,dim);
    res.fill((*this)[0]);
    { cimg_mapXYV(*this,x,y,k) res(x,y,0,k)        = (*this)(x,y,z0,k); }
    { cimg_mapYZV(*this,y,z,k) res(width+z,y,0,k)  = (*this)(x0,y,z,k); }
    { cimg_mapXZV(*this,x,z,k) res(x,height+z,0,k) = (*this)(x,y0,z,k); }
    return res;
  }

  //! Return the histogramm of the image
  CImg<float> get_histogram(const uint nblevels=256,const float pvmin=0,const float pvmax=0) const {
    cimg_test(*this);
    float vmin=pvmin,vmax=pvmax;
    T *ptr;
    if (nblevels<1) throw CImgArgumentException("CImg<%s>::histogram() : Can't compute an histogram with %d levels",pixel_type(),nblevels);
    CImg<float> res(nblevels);
    if (vmin==vmax && vmin==0) { CImgStats st(*this,false); vmin = (float)st.min; vmax = (float)st.max; }
    res.fill(0);
    cimg_map(*this,ptr) { const uint pos = (uint)((*ptr-vmin)*(nblevels-1)/(vmax-vmin)); if (pos<nblevels) res[pos]++; }
    return res;
  }

  //! Equalize the image histogram
  CImg& equalize_histogram(const uint nblevels=256) {
    cimg_test(*this);
    CImgStats st(*this,false);
    CImg<float> hist = get_histogram(nblevels,(float)st.min,(float)st.max);
    float cumul=0;
    T *ptr;
    cimg_mapX(hist,pos) { cumul+=hist[pos]; hist[pos]=cumul; }
    cimg_map(*this,ptr) {
      uint pos = (uint)((*ptr-st.min)*nblevels/(1+st.max-st.min));
      *ptr = (T)(st.min + (st.max-st.min)*hist[pos]/size());
    }
    return *this;
  }
  //! Return the histogram-equalized version of the current image.
  CImg get_equalize_histogram(const uint nblevels=256) const { return CImg<T>(*this).equalize_histogram(nblevels); }

  //! Return the image of the vector norms of the current image.
  CImg<float> get_norm_pointwise(int ntype=2) const {
    cimg_test(*this);
    CImg<float> res(width,height,depth);
    switch(ntype) {
    case -1:                // Linf norm
      {
        cimg_mapXYZ(*this,x,y,z) {
          float n=0; cimg_mapV(*this,v) if (std::fabs((double)((*this)(x,y,z,v)))>n) n=(float)(*this)(x,y,z,v); res(x,y,z) = n;
        }
      } break;
    case 1:               // L1 norm
      {
        cimg_mapXYZ(*this,x,y,z) {
          float n=0; cimg_mapV(*this,v) n+=(float)std::fabs((double)((*this)(x,y,z,v))); res(x,y,z) = n;
        }
      } break;
    default:              // L2 norm
      {
        cimg_mapXYZ(*this,x,y,z) {
          float n=0; cimg_mapV(*this,v) n+=(float)((*this)(x,y,z,v)*(*this)(x,y,z,v)); res(x,y,z) = (float)std::sqrt((double)n);
        }
      } break;
    }
    return res;
  }

  //! Replace each pixel value with its vector norm
  CImg& norm_pointwise() { return CImg<T>(get_norm_pointwise()).swap(*this); }

  //! Return an image of the normalized vectors
  CImg get_orient_pointwise() const {
    cimg_test(*this);
    CImg dest(width,height,depth,dim);
    cimg_mapXYZ(dest,x,y,z) {
      float n = 0;
      cimg_mapV(*this,v) n+=(float)((*this)(x,y,z,v)*(*this)(x,y,z,v));
      n = (float)std::sqrt((double)n);
      if (n>0) cimg_mapV(dest,v) dest(x,y,z,v)=(T)((*this)(x,y,z,v)/n); else cimg_mapV(dest,v) dest(x,y,z,v)=0;
    }
    return dest;
  }

  //! Replace each pixel value with its normalized vector component
  CImg& orient_pointwise() { return get_orient_pointwise().swap(*this); }

  //! Return a list of images, corresponding to a split of the current image into a specific axis.
  CImgl<T> get_split(const char Axe='v') const {
    cimg_test(*this);
    const char axe = cimg::uncase(Axe);
    const uint n = (axe=='x'?width:(axe=='y'?height:(axe=='z'?depth:(axe=='v'?dim:0))));
    if (n==0) throw CImgArgumentException("CImg<%s>::get_split() : Unknow axe '%c', use rather 'x','y','z' or 'v'",pixel_type());
    CImgl<T> res(n);
    cimgl_map(res,l) switch(axe) {
    case 'x': res[l] = get_sprite(l,0,0,0,l,height-1,depth-1,dim-1);   break;
    case 'y': res[l] = get_sprite(0,l,0,0,width-1,l,depth-1,dim-1);    break;
    case 'z': res[l] = get_sprite(0,0,l,0,width-1,height-1,l,dim-1);   break;
    case 'v': res[l] = get_sprite(0,0,0,l,width-1,height-1,depth-1,l); break;
    }
    return res;
  }

  //! Return a list of images, corresponding to the XY-gradients of an image.
  CImgl<T> get_gradientXY(const int scheme=0) const {
    cimg_test(*this);
    double a,b;
    CImgl<T> res(2,width,height,depth,dim);
    CImg_3x3(I,T);
    switch(scheme) {
    case -1:                  // backward finite differences
      { cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) { res[0](x,y,z,k) = Icc-Ipc; res[1](x,y,z,k) = Icc-Icp; } }
      break;
    case 1:                 // forward finite differences
      { cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) { res[0](x,y,0,k) = Inc-Icc; res[1](x,y,z,k) = Icn-Icc; } }
      break;
    case 2:                 // using Sobel mask
      {
        a = 1; b = 2;
        cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) {
          res[0](x,y,z,k) = (T)(-a*Ipp-b*Ipc-a*Ipn+a*Inp+b*Inc+a*Inn);
          res[1](x,y,z,k) = (T)(-a*Ipp-b*Icp-a*Inp+a*Ipn+b*Icn+a*Inn);
        }
      }
    case 3:                   // using rotation invariant mask
      {
        a = 0.25*(2-std::sqrt(2.0));
        b = 0.5*(std::sqrt(2.0)-1);
        cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) {
          res[0](x,y,z,k) = (T)(-a*Ipp-b*Ipc-a*Ipn+a*Inp+b*Inc+a*Inn);
          res[1](x,y,z,k) = (T)(-a*Ipp-b*Icp-a*Inp+a*Ipn+b*Icn+a*Inn);
        }
      }
      break;
    case 0:                 // central finite differences
    default:
      { cimg_mapZV(*this,z,k) cimg_map3x3(*this,x,y,z,k,I) { res[0](x,y,z,k) = (T)(0.5*(Inc-Ipc)); res[1](x,y,z,k) = (T)(0.5*(Icn-Icp)); } }
      break;
    }
    return res;
  }

  //! Return a list of images, corresponding to the XYZ-gradients of an image.
  CImgl<T> get_gradientXYZ(const int scheme=0) const {
    cimg_test(*this);
    CImgl<T> res(3,width,height,depth,dim);
    CImg_3x3x3(I,T);
    switch(scheme) {
    case -1:                  // backward finite differences
      { cimg_mapV(*this,k) cimg_map3x3x3(*this,x,y,z,k,I) { res[0](x,y,z,k) = Iccc-Ipcc; res[1](x,y,z,k) = Iccc-Icpc; res[2](x,y,z,k) = Iccc-Iccp; } }
      break;
    case 1:                 // forward finite differences
      { cimg_mapV(*this,k) cimg_map3x3x3(*this,x,y,z,k,I) { res[0](x,y,z,k) = Incc-Iccc; res[1](x,y,z,k) = Icnc-Iccc; res[2](x,y,z,k) = Iccn-Iccc; } }
      break;
    case 0:                 // central finite differences
    default:
      { cimg_mapV(*this,k) cimg_map3x3x3(*this,x,y,z,k,I) { res[0](x,y,z,k) = (T)(0.5*(Incc-Ipcc)); res[1](x,y,z,k) = (T)(0.5*(Icnc-Icpc)); 
          res[2](x,y,z,k) = (T)(0.5*(Iccn-Iccp)); } }
      break;
    }
    return res;
  }


  //! Return a dilated image (\p times dilatation).
  CImg get_dilate(const uint n=1) {
    CImg_3x3x3(I,T);
    if (n==1) {
      CImg dest(*this);
      cimg_mapV(*this,k) cimg_map3x3x3(*this,x,y,z,k,I) if (Iccc && (!Incc || !Ipcc || !Icnc || !Icpc || !Iccn || !Iccp)) dest(x,y,z,k) = 0;
      return dest;
    }
    CImg img1(*this),img2(*this,false);
    CImg *src = &img1, *dest = &img2, *tmp = NULL;
    for (uint iter=0; iter<n; iter++) {
      *dest = *src;
      cimg_mapV(*src,k) cimg_map3x3x3(*src,x,y,z,k,I) if (Iccc && (!Incc || !Ipcc || !Icnc || !Icpc || !Iccn || !Iccp)) (*dest)(x,y,z,k) = 0;
      tmp = src;
      src = dest;
      dest = tmp;
    }
    return *src;      
  }
  //! Dilate the image \p n times.
  CImg& dilate(const uint n=1) { return get_dilate(n).swap(*this); }

  //! Return an eroded image (\p times erosion).
  CImg get_erode(const uint n=1) {
    CImgStats stats(*this);
    const T tmax = stats.max!=0?(T)stats.max:(T)1;
    CImg_3x3x3(I,T);
    if (n==1) {
      CImg dest(*this);
      cimg_mapV(*this,k) cimg_map3x3x3(*this,x,y,z,k,I) if (!Iccc && (Incc || Ipcc || Icnc || Icpc || Iccn || Iccp)) dest(x,y,z,k) = tmax;
      return dest;
    }
    CImg img1(*this),img2(*this,false);
    CImg *src = &img1, *dest = &img2, *tmp = NULL;
    for (uint iter=0; iter<n; iter++) {
      *dest = *src;
      cimg_mapV(*src,k) cimg_map3x3x3(*src,x,y,z,k,I) if (!Iccc && (Incc || Ipcc || Icnc || Icpc || Iccn || Iccp)) (*dest)(x,y,z,k) = tmax;
      tmp = src;
      src = dest;
      dest = tmp;
    }
    return *src;      
  }
  //! Erode the image \p n times.
  CImg& erode(const uint n=1) { return get_erode(n).swap(*this); }

  //@}
  //--------------------------------------
  //--------------------------------------
  //
  //! \name Color conversion functions
  //@{
  //--------------------------------------
  //--------------------------------------
  
  //! Convert (R,G,B) pixels of a color image into the (X,Y,Z)_709 color space.
  CImg& RGBtoXYZ() {
    cimg_test(*this);
    if (dim!=3) throw CImgInstanceException("CImg<%s>::RGBtoXYZ() : Input image dimension is dim=%d, should be a (R,G,B) image (dim=3)",
                                            pixel_type(),dim);
    cimg_mapXYZ(*this,x,y,z) {
      const T R = (*this)(x,y,z,0), G = (*this)(x,y,z,1), B = (*this)(x,y,z,2);
      (*this)(x,y,z,0) = (T)(0.412453*R + 0.357580*G + 0.180423*B);
      (*this)(x,y,z,1) = (T)(0.212671*R + 0.715160*G + 0.072169*B);
      (*this)(x,y,z,2) = (T)(0.019334*R + 0.119193*G + 0.950227*B);
    }
    return *this;
  }
  //! Get a new image with (X,Y,Z) color-based pixels from a (R,G,B) image.
  CImg get_RGBtoXYZ() const { return CImg<T>(*this).RGBtoXYZ(); }

  //! Convert (X,Y,Z)_709 pixels of a color image into the (R,G,B) color space.
  CImg& XYZtoRGB() {
    cimg_test(*this);
    if (dim!=3) throw CImgInstanceException("CImg<%s>::XYZtoRGB() : Input image dimension is dim=%d, should be a (X,Y,Z) image (dim=3)",
                                            pixel_type(),dim);
    cimg_mapXYZ(*this,x,y,z) {
      const T X = (*this)(x,y,z,0), Y = (*this)(x,y,z,1), Z = (*this)(x,y,z,2);
      (*this)(x,y,z,0) = (T)(3.240479*X  - 1.537150*Y - 0.498535*Z);
      (*this)(x,y,z,1) = (T)(-0.969256*X + 1.875992*Y + 0.041556*Z);
      (*this)(x,y,z,2) = (T)(0.055648*X  - 0.204043*Y + 1.057311*Z);
    }
    return *this;
  }
  //! Get a new image with (R,G,B) color-based pixels from a (X,Y,Z) image.
  CImg get_XYZtoRGB() const { return CImg<T>(*this).XYZtoRGB(); }

  //! Convert (X,Y,Z)_709 pixels of a color image into the (L*,a*,b*) color space.
  CImg& XYZtoLab() {
    cimg_test(*this);
    if (dim!=3) throw CImgInstanceException("CImg<%s>::XYZtoLab() : Input image dimension is dim=%d, should be a (X,Y,Z) image (dim=3)",
                                            pixel_type(),dim);
    CImg<double> white = CImg<double>(1,1,1,3).fill(1,1,1).RGBtoXYZ();
    const double Xn = white(0), Yn = white(1), Zn = white(2);
    
    cimg_mapXYZ(*this,x,y,z) {
      const T X = (*this)(x,y,z,0), Y = (*this)(x,y,z,1), Z = (*this)(x,y,z,2);
      const double L = (T)(116*std::pow(Y/Yn,1/3.0)-16);
      (*this)(x,y,z,0) = (T)(L>=0?L:0);
      (*this)(x,y,z,1) = (T)(500*(std::pow(X/Xn,1/3.0)-std::pow(Y/Yn,1/3.0)));
      (*this)(x,y,z,2) = (T)(200*(std::pow(Y/Yn,1/3.0)-std::pow(Z/Zn,1/3.0)));
    }
    return *this;
  }
  //! Get a new image with (L,a,b) color-based pixels from a (X,Y,Z) image.
  CImg get_XYZtoLab() const { return CImg<T>(*this).XYZtoLab(); }

  //@}
  //--------------------------------------
  //--------------------------------------
  //
  //! \name Drawing functions
  //@{
  //--------------------------------------
  //--------------------------------------
  
  //! Draw a CImg<t> 'sprite' in the current image, at specified position (x0,y0,z0,v0)
  CImg& draw_image(const CImg& sprite,const int px0=0,const int py0=0,const int pz0=0,const int pv0=0,const float opacity=1) {
    if (this==&sprite) return draw_image(CImg<T>(sprite),px0,py0,pz0,pv0,opacity);
    cimg_test(*this); cimg_test(sprite);
    const uint wh = width*height, swh = sprite.width*sprite.height;
    const T *ptrs = sprite.data;
    int lX=sprite.width, lY=sprite.height, lZ=sprite.depth, lV=sprite.dim;
    if (px0+lX>(int)width)  lX-=px0+lX-width;
    if (py0+lY>(int)height) lY-=py0+lY-height;
    if (pz0+lZ>(int)depth)  lZ-=pz0+lZ-depth;
    if (pv0+lV>(int)dim)    lV-=pv0+lV-dim;
    if (px0<0) { ptrs-=px0;                  lX+=px0; }
    if (py0<0) { ptrs-=py0*sprite.width;     lY+=py0; }
    if (pz0<0) { ptrs-=pz0*swh;              lZ+=pz0; }
    if (pv0<0) { ptrs-=pv0*swh*sprite.depth; lV+=pv0; }
    T *ptrd = ptr(px0<0?0:px0,py0<0?0:py0,pz0<0?0:pz0,pv0<0?0:pv0);
    if (opacity>=1) {
      if (lX>0) {
        lX*=sizeof(T);
        for (int v=0; v<lV; v++) {
          for (int z=0; z<lZ; z++) {
            for (int y=0; y<lY; y++) {
              std::memcpy(ptrd,ptrs,lX);
              ptrd+=width;  ptrs+=sprite.width;
            }
            ptrd-=width*(lY-height); ptrs-=sprite.width*(lY-sprite.height);
          }
          ptrd-=wh*(lZ-depth); ptrs-=swh*(lZ-sprite.depth);
        }
      }
    }
    else
      for (int v=0; v<lV; v++) {
        for (int z=0; z<lZ; z++) {
          for (int y=0; y<lY; y++) {
            for (int x=0; x<lX; x++)
              *(ptrd++) = (T)((1-opacity)*(*ptrd) + opacity*(*(ptrs++)));
            ptrd+=width-lX; ptrs+=sprite.width-lX;
          }
          ptrd-=width*(lY-height); ptrs-=sprite.width*(lY-sprite.height);
        }
        ptrd-=wh*(lZ-depth); ptrs-=swh*(lZ-sprite.depth);
      }
    return *this;
  }

  //! same as draw_image, with a transparency mask (CImg with values between [0,1])
  template<class t> CImg& draw_image(const CImg& sprite,const CImg<t>& mask,
                                     const int px0=0,const int py0=0,const int pz0=0,const int pv0=0) {
    cimg_test(*this); cimg_test(sprite); cimg_test(mask);
    if (this==&sprite)                return draw_image(CImg<T>(sprite),mask,px0,py0,pz0,pv0);
    if(mask.width!=sprite.width || mask.height!=sprite.height || mask.depth!=sprite.depth || mask.dim!=1)
      throw CImgArgumentException("CImg<%s>::draw_image() : mask dimension is not valid : (%d,%d,%d,%d), while sprite is (%d,%d,%d,%d)",
                                  pixel_type(),mask.width,mask.height,mask.depth,mask.dim,sprite.width,sprite.height,sprite.depth,sprite.dim);
    const uint wh=width*height, swh=sprite.width*sprite.height, mwh=mask.width*mask.height;
    int lX=sprite.width, lY=sprite.height, lZ=sprite.depth, lV=sprite.dim;
    const T *ptrs = sprite.ptr();
    const t *ptrm = mask.ptr();
    if (px0+lX>(int)width)  lX-=px0+lX-width;
    if (py0+lY>(int)height) lY-=py0+lY-height;
    if (pz0+lZ>(int)depth)  lZ-=pz0+lZ-depth;
    if (pv0+lV>(int)dim)    lV-=pv0+lV-dim;
    if (px0<0) { ptrs-=px0;                  ptrm-=px0;                lX+=px0; }
    if (py0<0) { ptrs-=py0*sprite.width;     ptrm-=py0*mask.width;     lY+=py0; }
    if (pz0<0) { ptrs-=pz0*swh;              ptrm-=pz0*mwh;            lZ+=pz0; }
    if (pv0<0) { ptrs-=pv0*swh*sprite.depth; ptrm-=pv0*mwh*mask.depth; lV+=pv0; }
    T *ptrd = ptr(px0<0?0:px0,py0<0?0:py0,pz0<0?0:pz0,pv0<0?0:pv0);
    for (int v=0; v<lV; v++) {
      for (int z=0; z<lZ; z++) {
        for (int y=0; y<lY; y++) {
          for (int x=0; x<lX; x++) {
            const t& opacity = *(ptrm++);
            *(ptrd++) = (T)((1-opacity)*(*ptrd) + opacity*(*(ptrs++)));
          }
          ptrd+=width-lX; ptrs+=sprite.width-lX; ptrm+=mask.width-lX;
        }
        ptrd-=width*(lY-height); ptrs-=sprite.width*(lY-sprite.height); ptrm-=mask.width*(lY-mask.height);
      }
      ptrd-=wh*(lZ-depth); ptrs-=swh*(lZ-sprite.depth); ptrm-=mwh*lZ;
    }
    return *this;
  }

  //! Draw a line (\p px0,\p py0)-(\p px1,\p py1) in a 2D image, with a color \p color.
  CImg& draw_line(const int px0,const int py0,const int px1,const int py1,const T *color) {
    cimg_test(*this); 
    if (!color) throw CImgArgumentException("CImg<%s>::draw_line() : specified color pointer is NULL",pixel_type());
    int x0=px0, y0=py0, x1=px1, y1=py1;
    if (x0>x1) { const int tmpx=x0, tmpy=y0; x0=x1; x1=tmpx; y0=y1; y1=tmpy;}
    if (x1<0 || x0>=(int)width) return *this;
    if (x0<0) { y0-=x0*(y1-y0)/(x1-x0); x0=0; }
    if (x1>=(int)width) { y1+=(x1-(int)width)*(y0-y1)/(x1-x0); x1=(int)width-1;}
    if (y0>y1) { const int tmpx=x0, tmpy=y0; x0=x1; x1=tmpx; y0=y1; y1=tmpy;}
    if (y1<0 || y0>=(int)height) return *this;
    if (y0<0) { x0-=y0*(x1-x0)/(y1-y0); y0=0; }
    if (y1>=(int)height) { x1+=(y1-(int)height)*(x0-x1)/(y1-y0); y1=(int)height-1;}
    const uint dmax = (uint)cimg::max(std::abs(x1-x0),y1-y0);
    const float px = dmax?(x1-x0)/(float)dmax:0, py = dmax?(y1-y0)/(float)dmax:0;
    float x = (float)x0, y = (float)y0;
    for (uint t=0; t<=dmax; t++) { 
      const uint ix=(uint)x, iy=(uint)y;
      cimg_mapV(*this,k) (*this)(ix,iy,0,k) = *(color++);
      color-=dim; x+=px; y+=py;
    }
    return *this;
  }

  //! Draw a textured line (\p px0,\p py0)-(\p px1,\p py1) in a 2D image, with a texture \p texture.
  CImg& draw_line(const int px0,const int py0,const int px1,const int py1,const CImg& texture,
                  const int ptx0,const int pty0,const int ptx1,const int pty1) {
    cimg_test(*this); cimg_test(texture);
    int x0=px0, y0=py0, x1=px1, y1=py1, tx0=ptx0, ty0=pty0, tx1=ptx1, ty1=pty1;
    if (x0>x1) { const int tmpx=x0, tmpy=y0, ttmpx=tx0, ttmpy=ty0; x0=x1; x1=tmpx; y0=y1; y1=tmpy; tx0=tx1; ty0=ty1; tx1=ttmpx; ty1=ttmpy; }
    if (x1<0 || x0>=(int)width) return *this;
    if (x0<0) { y0-=x0*(y1-y0)/(x1-x0); tx0-=x0*(tx1-tx0)/(x1-x0); ty0-=x0*(ty1-ty0)/(x1-x0); x0=0; }
    if (x1>=(int)width) { y1+=(x1-(int)width)*(y0-y1)/(x1-x0); tx1+=(x1-(int)width)*(tx0-tx1)/(x1-x0); ty1+=(x1-(int)width)*(ty0-ty1)/(x1-x0); x1=(int)width-1; }
    if (y0>y1) { const int tmpx=x0, tmpy=y0, ttmpx=tx0, ttmpy=ty0; x0=x1; x1=tmpx; y0=y1; y1=tmpy; tx0=tx1; ty0=ty1; tx1=ttmpx; ty1=ttmpy; }
    if (y1<0 || y0>=(int)height) return *this;
    if (y0<0) { x0-=y0*(x1-x0)/(y1-y0); tx0-=y0*(tx1-tx0)/(y1-y0); ty0-=y0*(ty1-ty0)/(y1-y0); y0=0; }
    if (y1>=(int)height) { x1+=(y1-(int)height)*(x0-x1)/(y1-y0); tx1+=(y1-(int)height)*(tx0-tx1)/(y1-y0); ty1+=(y1-(int)height)*(ty0-ty1)/(y1-y0); y1=(int)height-1; }
    const uint dmax = (uint)cimg::max(std::abs(x1-x0),y1-y0);
    const float
      px = dmax?(x1-x0)/(float)dmax:0,
      py = dmax?(y1-y0)/(float)dmax:0,
      tpx = dmax?(tx1-tx0)/(float)dmax:0,
      tpy = dmax?(ty1-ty0)/(float)dmax:0;
    float x = (float)x0, y = (float)y0, tx = (float)tx0, ty = (float)ty0;
    for (uint t=0; t<=dmax; t++) { 
      const uint ix=(uint)x, iy=(uint)y, itx=(uint)tx, ity=(uint)ty; 
      cimg_mapV(*this,k) (*this)(ix,iy,0,k) = texture(itx,ity,0,k);
      x+=px; y+=py; tx+=tpx; ty+=tpy;
    }
    return *this;
  }

  //! Draw an hatched line (\p px0,\p py0)-(\p px1,\p py1) in a 2D image, with a color \p color.
  CImg& draw_line(const int px0,const int py0,const int px1,const int py1,const T *color,const uint phatch) {
    cimg_test(*this);
    if (!color) throw CImgArgumentException("CImg<%s>::draw_line() : specified color pointer is NULL",pixel_type());
    int x0=px0, y0=py0, x1=px1, y1=py1, hatch=phatch&0xff, st_hatch=phatch>>8;
    if (x0>x1) { const int tmpx=x0, tmpy=y0; x0=x1; x1=tmpx; y0=y1; y1=tmpy;}
    if (x1<0 || x0>=(int)width) return *this;
    if (x0<0) { y0-=x0*(y1-y0)/(x1-x0); x0=0; }
    if (x1>=(int)width) { y1+=(x1-width)*(y0-y1)/(x1-x0); x1=width-1;}
    if (y0>y1) { const int tmpx=x0, tmpy=y0; x0=x1; x1=tmpx; y0=y1; y1=tmpy;}
    if (y1<0 || y0>=(int)height) return *this;
    if (y0<0) { x0-=y0*(x1-x0)/(y1-y0); y0=0; }
    if (y1>=(int)height) { x1+=(y1-height)*(x0-x1)/(y1-y0); y1=height-1;}
    const uint dmax = (uint)cimg::max(std::abs(x1-x0),y1-y0);
    const float px = dmax?(x1-x0)/(float)dmax:0, py = dmax?(y1-y0)/(float)dmax:0;
    float x = (float)x0, y = (float)y0;
    for (uint t=0; t<=dmax; t++) {
      const uint ix=(uint)x, iy=(uint)y, n=(st_hatch>>8)&1;
      if (!n) { cimg_mapV(*this,k) (*this)(ix,iy,0,k) = *(color++); color-=dim; }
      x+=px; y+=py; st_hatch+=hatch;
    }
    return *this;
  }

  //! Draw a line (\p px0,\p py0,\p pz0)-(\p px1,\p py1,\p pz1) in a 3D volume, with a color \p color.
  CImg& draw_line(const int px0,const int py0,const int pz0,const int px1,const int py1,const int pz1,const T *color) {
    cimg_test(*this); 
    if (!color) throw CImgArgumentException("CImg<%s>::draw_line() : specified color pointer is NULL",pixel_type());
    int x0=px0, y0=py0, z0=pz0, x1=px1, y1=py1, z1=pz1;
    if (x0>x1) { const int tmpx=x0, tmpy=y0, tmpz=z0; x0=x1; x1=tmpx; y0=y1; y1=tmpy; z0=z1; z1=tmpz;}
    if (x1<0 || x0>=(int)width) return *this;
    if (x0<0) { y0-=x0*(y1-y0)/(x1-x0); z0-=x0*(z1-z0)/(x1-x0); x0=0; }
    if (x1>=(int)width) { y1+=(x1-(int)width)*(y0-y1)/(x1-x0); z1+=(x1-(int)width)*(z0-z1)/(x1-x0); x1=(int)width-1;}
    if (y0>y1) { const int tmpx=x0, tmpy=y0, tmpz=z0; x0=x1; x1=tmpx; y0=y1; y1=tmpy; z0=z1; z1=tmpz;}
    if (y1<0 || y0>=(int)height) return *this;
    if (y0<0) { x0-=y0*(x1-x0)/(y1-y0); z0-=y0*(z1-z0)/(y1-y0); y0=0; }
    if (y1>=(int)height) { x1+=(y1-(int)height)*(x0-x1)/(y1-y0); z1+=(y1-(int)height)*(z0-z1)/(y1-y0); y1=(int)height-1;}
    if (z0>z1) { const int tmpx=x0, tmpy=y0, tmpz=z0; x0=x1; x1=tmpx; y0=y1; y1=tmpy; z0=z1; z1=tmpz;}
    if (z1<0 || z0>=(int)depth) return *this;
    if (z0<0) { x0-=z0*(x1-x0)/(z1-z0); y0-=z0*(y1-y0)/(z1-z0); z0=0; }
    if (z1>=(int)depth) { x1+=(z1-(int)depth)*(x0-x1)/(z1-z0); y1+=(z1-(int)depth)*(y0-y1)/(z1-z0); z1=(int)depth-1;}
    const uint dmax = (uint)cimg::max(std::abs(x1-x0),std::abs(y1-y0),z1-z0);
    const float px = dmax?(x1-x0)/(float)dmax:0;  py = dmax?(y1-y0)/(float)dmax:0; pz = dmax?(z1-z0)/(float)dmax:0;
    float x = (float)x0; y = (float)y0; z = (float)z0;
    for (uint t=0; t<=dmax; t++) { 
      const uint ix=(uint)x, iy=(uint)y, iz=(uint)z; 
      cimg_mapV(*this,k) (*this)(ix,iy,iz,k) = *(color++);
      color-=dim; x+=px; y+=py; z+=pz; 
    }
    return *this;
  }

  //! Draw a filled triangle (\p px0,\p py0)-(\p px1,\p py1)-(\p px2,\p py2) in a 2D image, with a color \p color.
  CImg& draw_triangle(const int px0,const int py0,const int px1,const int py1,const int px2,const int py2,const T *color) {
    cimg_test(*this); 
    if (!color) throw CImgArgumentException("CImg<%s>::draw_triangle() : specified color pointer is NULL",pixel_type());
    int x0=px0,y0=py0,x1=px1,y1=py1,x2=px2,y2=py2;
    if (y0>y1) { const int xt=x0, yt=y0; x0=x1; x1=xt; y0=y1; y1=yt; }
    if (y0>y2) { const int xt=x0, yt=y0; x0=x2; x2=xt; y0=y2; y2=yt; }
    if (y1>y2) { const int xt=x1, yt=y1; x1=x2; x2=xt; y1=y2; y2=yt; }	
    if (y0>=(int)height || y2<0) return *this;
    const float 
      p1 = (y1-y0)?(x1-x0)/(float)(y1-y0):(x1-x0),
      p2 = (y2-y0)?(x2-x0)/(float)(y2-y0):(x2-x0),
      p3 = (y2-y1)?(x2-x1)/(float)(y2-y1):(x2-x1);
    float xleft = (float)x0, xright = xleft, pleft = (p1<p2)?p1:p2, pright = (p1<p2)?p2:p1;
    if (y0<0) { xleft-=y0*pleft; xright-=y0*pright; }
    const int ya = y1>dimy()?height:y1;
    for (int y=y0<0?0:y0; y<ya; y++) {
      const int xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) cimg_mapV(*this,k) (*this)(x,y,0,k) = color[k];			
      xleft+=pleft; xright+=pright;
    }
    if (p1<p2) { xleft=(float)x1;  pleft=p3;  if (y1<0) xleft-=y1*pleft; } 
    else       { xright=(float)x1; pright=p3; if (y1<0) xright-=y1*pright; }
    const int yb = y2>=dimy()?height-1:y2;
    for (int yy=y1<0?0:y1; yy<=yb; yy++) {
      const int xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) cimg_mapV(*this,k) (*this)(x,yy,0,k) = color[k];
      xleft+=pleft; xright+=pright;
    }
    return *this;
  }

  //! Draw a textured triangle in a 2D image.
  CImg& draw_triangle(const int px0,const int py0,const int px1,const int py1,const int px2,const int py2,
                      const CImg& texture,const int ptx0,const int pty0,const int ptx1,const int pty1,const int ptx2,const int pty2) {

    cimg_test(*this); cimg_test(texture);
    int x0=px0,y0=py0,x1=px1,y1=py1,x2=px2,y2=py2,tx0=ptx0,ty0=pty0,tx1=ptx1,ty1=pty1,tx2=ptx2,ty2=pty2;
    if (y0>y1) { const int xt=x0, yt=y0, txt=tx0, tyt=ty0; x0=x1; x1=xt; y0=y1; y1=yt; tx0=tx1; tx1=txt; ty0=ty1; ty1=tyt; }
    if (y0>y2) { const int xt=x0, yt=y0, txt=tx0, tyt=ty0; x0=x2; x2=xt; y0=y2; y2=yt; tx0=tx2; tx2=txt; ty0=ty2; ty2=tyt; }
    if (y1>y2) { const int xt=x1, yt=y1, txt=tx1, tyt=ty1; x1=x2; x2=xt; y1=y2; y2=yt; tx1=tx2; tx2=txt; ty1=ty2; ty2=tyt; }
    if (y0>=(int)height || y2<0) return *this;
    const float 
      p1 = (y1-y0)?(x1-x0)/(float)(y1-y0):(x1-x0),
      p2 = (y2-y0)?(x2-x0)/(float)(y2-y0):(x2-x0),
      p3 = (y2-y1)?(x2-x1)/(float)(y2-y1):(x2-x1),
      tpx1 = (y1-y0)?(tx1-tx0)/(float)(y1-y0):0,
      tpy1 = (y1-y0)?(ty1-ty0)/(float)(y1-y0):0,
      tpx2 = (y2-y0)?(tx2-tx0)/(float)(y2-y0):0,
      tpy2 = (y2-y0)?(ty2-ty0)/(float)(y2-y0):0,
      tpx3 = (y2-y1)?(tx2-tx1)/(float)(y2-y1):0,
      tpy3 = (y2-y1)?(ty2-ty1)/(float)(y2-y1):0;
    float pleft,pright,tpxleft,tpyleft,tpxright,tpyright,
      txleft=(float)tx0,tyleft=(float)ty0,xleft=(float)x0,txright=txleft,tyright=tyleft,xright=xleft;
    if (p1<p2) { pleft=p1; pright=p2; tpxleft=tpx1; tpyleft=tpy1; tpxright=tpx2; tpyright=tpy2; } 
    else       { pleft=p2; pright=p1; tpxleft=tpx2; tpyleft=tpy2; tpxright=tpx1; tpyright=tpy1; }
    if (y0<0) { xleft-=y0*pleft; xright-=y0*pright; txleft-=y0*tpxleft; tyleft-=y0*tpyleft; txright-=y0*tpxright; tyright-=y0*tpyright; }		
    const int ya = y1<dimy()?y1:height;
    for (int y=(y0<0?0:y0); y<ya; y++) {
      const int dx = (int)xright-(int)xleft;
      const float tpx = dx?((int)txright-(int)txleft)/(float)dx:0, tpy = dx?((int)tyright-(int)tyleft)/(float)dx:0;
      float
        tx = (float)((xleft>=0)?(int)txleft:(int)(txleft-(int)xleft*tpx)),
        ty = (float)((xleft>=0)?(int)tyleft:(int)(tyleft-(int)xleft*tpy));
      const int xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<(int)width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) { cimg_mapV(*this,k) (*this)(x,y,0,k) = texture((uint)tx,(uint)ty,0,k); tx+=tpx; ty+=tpy; }
      xleft+=pleft; xright+=pright; txleft+=tpxleft; tyleft+=tpyleft; txright+=tpxright; tyright+=tpyright;
    }
    
    if (p1<p2) {
      xleft=(float)x1; pleft=p3; txleft=(float)tx1; tyleft=(float)ty1; tpxleft=tpx3; tpyleft=tpy3;
      if (y1<0) { xleft-=y1*pleft; txleft-=y1*tpxleft; tyleft-=y1*tpyleft; }
    } else { 
      xright=(float)x1; pright=p3; txright=(float)tx1; tyright=(float)ty1; tpxright=tpx3; tpyright=tpy3;
      if (y1<0) { xright-=y1*pright; txright-=y1*tpxright; tyright-=y1*tpyright; }
    }

    const int yb = y2>=dimy()?(height-1):y2;
    for (int yy=(y1<0?0:y1); yy<=yb; yy++) {
      const int dx = (int)(xright)-(int)xleft;
      const float tpx = dx?((int)txright-(int)txleft)/(float)dx:0, tpy = dx?((int)tyright-(int)tyleft)/(float)dx:0;
      float
        tx = (float)((xleft>=0)?(int)txleft:(int)(txleft-(int)xleft*tpx)),
        ty = (float)((xleft>=0)?(int)tyleft:(int)(tyleft-(int)xleft*tpy));
      const int	xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<(int)width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) { cimg_mapV(*this,k) (*this)(x,yy,0,k) = texture((uint)tx,(uint)ty,0,k); tx+=tpx; ty+=tpy; }
      xleft+=pleft; xright+=pright; txleft+=tpxleft; tyleft+=tpyleft;	txright+=tpxright; tyright+=tpyright;
    }
    return *this;
  }

  //! Draw a bi-textured triangle in a 2D image.
  CImg& draw_triangle(const int px0,const int py0,const int px1,const int py1,const int px2,const int py2,
                      const CImg& texture,const int ptx0,const int pty0,const int ptx1,const int pty1,const int ptx2,const int pty2,
                      const CImg& motif,  const int pmx0,const int pmy0,const int pmx1,const int pmy1,const int pmx2,const int pmy2,
                      const float tweight,const float mweight) {
    cimg_test(*this); cimg_test(texture); cimg_test(motif);
    int x0=px0, y0=py0, x1=px1, y1=py1, x2=px2, y2=py2,
      tx0=ptx0, ty0=pty0, tx1=ptx1, ty1=pty1, tx2=ptx2, ty2=pty2,
      mx0=pmx0, my0=pmy0, mx1=pmx1, my1=pmy1, mx2=pmx2, my2=pmy2;
    if (y0>y1) { const int xt=x0, yt=y0, txt=tx0, tyt=ty0, mxt=mx0, myt=my0; x0=x1; x1=xt; y0=y1; y1=yt; tx0=tx1; tx1=txt; ty0=ty1; ty1=tyt; mx0=mx1; mx1=mxt; my0=my1; my1=myt; }
    if (y0>y2) { const int xt=x0, yt=y0, txt=tx0, tyt=ty0, mxt=mx0, myt=my0; x0=x2; x2=xt; y0=y2; y2=yt; tx0=tx2; tx2=txt; ty0=ty2; ty2=tyt; mx0=mx2; mx2=mxt; my0=my2; my2=myt; }
    if (y1>y2) { const int xt=x1, yt=y1, txt=tx1, tyt=ty1, mxt=mx1, myt=my1; x1=x2; x2=xt; y1=y2; y2=yt; tx1=tx2; tx2=txt; ty1=ty2; ty2=tyt; mx1=mx2; mx2=mxt; my1=my2; my2=myt; }
    if (y0>=(int)height || y2<0) return *this;
    const float
      p1 = (y1-y0)?(x1-x0)/(float)(y1-y0):(x1-x0),
      p2 = (y2-y0)?(x2-x0)/(float)(y2-y0):(x2-x0),
      p3 = (y2-y1)?(x2-x1)/(float)(y2-y1):(x2-x1),
      tpx1 = (y1-y0)?(tx1-tx0)/(float)(y1-y0):0,
      tpy1 = (y1-y0)?(ty1-ty0)/(float)(y1-y0):0,
      tpx2 = (y2-y0)?(tx2-tx0)/(float)(y2-y0):0,
      tpy2 = (y2-y0)?(ty2-ty0)/(float)(y2-y0):0,
      tpx3 = (y2-y1)?(tx2-tx1)/(float)(y2-y1):0,
      tpy3 = (y2-y1)?(ty2-ty1)/(float)(y2-y1):0,
      mpx1 = (y1-y0)?(mx1-mx0)/(float)(y1-y0):0,
      mpy1 = (y1-y0)?(my1-my0)/(float)(y1-y0):0,
      mpx2 = (y2-y0)?(mx2-mx0)/(float)(y2-y0):0,
      mpy2 = (y2-y0)?(my2-my0)/(float)(y2-y0):0,
      mpx3 = (y2-y1)?(mx2-mx1)/(float)(y2-y1):0,
      mpy3 = (y2-y1)?(my2-my1)/(float)(y2-y1):0;
    float pleft,pright,tpxleft,tpyleft,tpxright,tpyright,mpxleft,mpyleft,mpxright,mpyright,
      txleft  = (float)tx0,
      tyleft  = (float)ty0,
      mxleft  = (float)mx0,
      myleft  = (float)my0,
      xleft   = (float)x0,
      txright = txleft,			
      tyright = tyleft,
      mxright = mxleft,
      myright = myleft,
      xright  = xleft;
    if (p1<p2) { pleft=p1; pright=p2; tpxleft=tpx1; tpyleft=tpy1; tpxright=tpx2; tpyright=tpy2; mpxleft=mpx1; mpyleft=mpy1; mpxright=mpx2; mpyright=mpy2; } 
    else       { pleft=p2; pright=p1; tpxleft=tpx2; tpyleft=tpy2; tpxright=tpx1; tpyright=tpy1; mpxleft=mpx2; mpyleft=mpy2; mpxright=mpx1; mpyright=mpy1; }
    if (y0<0) {
      xleft-=y0*pleft; xright-=y0*pright; 
      txleft-=y0*tpxleft; tyleft-=y0*tpyleft; txright-=y0*tpxright; tyright-=y0*tpyright;
      mxleft-=y0*mpxleft; myleft-=y0*mpyleft; mxright-=y0*mpxright; myright-=y0*mpyright;
    }		
    const int ya = (y1<(int)height)?y1:height;
    for (int y=(y0<0?0:y0); y<ya; y++) {
      const int dx = (int)xright-(int)xleft;
      const float
        tpx = dx?((int)txright-(int)txleft)/(float)dx:0,
        tpy = dx?((int)tyright-(int)tyleft)/(float)dx:0,
        mpx = dx?((int)mxright-(int)mxleft)/(float)dx:0,
        mpy = dx?((int)myright-(int)myleft)/(float)dx:0;
      float
        tx = (float)((xleft>=0)?(int)txleft:(int)(txleft-(int)xleft*tpx)),
        ty = (float)((xleft>=0)?(int)tyleft:(int)(tyleft-(int)xleft*tpy)),
        mx = (float)((xleft>=0)?(int)mxleft:(int)(mxleft-(int)xleft*mpx)),
        my = (float)((xleft>=0)?(int)myleft:(int)(myleft-(int)xleft*mpy));
      const int xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) {
        cimg_mapV(*this,k) (*this)(x,y,0,k) = (T)(tweight*texture((uint)tx,(uint)ty,0,k)+mweight*motif((uint)mx,(uint)my,0,k));
        tx+=tpx; ty+=tpy; mx+=mpx; my+=mpy;
      }
      xleft+=pleft; xright+=pright;
      txleft+=tpxleft; tyleft+=tpyleft;	txright+=tpxright; tyright+=tpyright;
      mxleft+=mpxleft; myleft+=mpyleft;	mxright+=mpxright; myright+=mpyright;
    }
    
    if (p1<p2) { 
      xleft=(float)x1; pleft=p3; txleft=(float)tx1; tyleft=(float)ty1; tpxleft=tpx3; tpyleft=tpy3;
      mxleft=(float)mx1; myleft=(float)my1; mpxleft=mpx3; mpyleft=mpy3;
      if (y1<0) { xleft-=y1*pleft; txleft-=y1*tpxleft; tyleft-=y1*tpyleft; mxleft-=y1*mpxleft; myleft-=y1*mpyleft; }
    }
    else {
      xright=(float)x1; pright=p3; txright=(float)tx1; tyright=(float)ty1; tpxright=tpx3; tpyright=tpy3;
      mxright=(float)mx1; myright=(float)my1; mpxright=mpx3; mpyright=mpy3; 
      if (y1<0) { xright-=y1*pright; txright-=y1*tpxright; tyright-=y1*tpyright; mxright-=y1*mpxright; myright-=y1*mpyright; }
    }

    const int yb = (y2<(int)height)?y2:(height-1);
    for (int yy=(y1<0?0:y1); yy<=yb; yy++) {
      const int dx = (int)xright-(int)xleft;
      const float
        tpx = dx?((int)txright-(int)txleft)/(float)dx:0,
        tpy = dx?((int)tyright-(int)tyleft)/(float)dx:0,
        mpx = dx?((int)mxright-(int)mxleft)/(float)dx:0,
        mpy = dx?((int)myright-(int)myleft)/(float)dx:0;
      float
        tx = (float)((xleft>=0)?(int)txleft:(int)(txleft-(int)xleft*tpx)),
        ty = (float)((xleft>=0)?(int)tyleft:(int)(tyleft-(int)xleft*tpy)),
        mx = (float)((xleft>=0)?(int)mxleft:(int)(mxleft-(int)xleft*mpx)),
        my = (float)((xleft>=0)?(int)myleft:(int)(myleft-(int)xleft*mpy));
      const int xmin=(xleft>=0)?(int)xleft:0, xmax=(xright<width)?(int)xright:(width-1);
      for (int x=xmin; x<=xmax; x++) {
        cimg_mapV(*this,k) (*this)(x,yy,0,k) = (T)(tweight*texture((uint)tx,(uint)ty,0,k)+mweight*motif((uint)mx,(uint)my,0,k));
        tx+=tpx; ty+=tpy; mx+=mpx; my+=mpy;
      }
      xleft+=pleft; xright+=pright;
      txleft+=tpxleft; tyleft+=tpyleft;	txright+=tpxright; tyright+=tpyright;
      mxleft+=mpxleft; myleft+=mpyleft;	mxright+=mpxright; myright+=mpyright;
    }
    return *this;
  }

  //! Draw a filled rectangle (\p px0,\p py0,\p pz0,\p pv0)-(\p px1,\p py1,\p pz1,\p pv1) into an image, using a single value
  CImg& draw_rectangle(const int px0,const int py0,const int pz0,const int pv0,const int px1,const int py1,const int pz1,const int pv1,const T& val) {
    cimg_test(*this);
    const bool bx=(px0<px1), by=(py0<py1), bz=(pz0<pz1), bv=(pv0<pv1);
    const int x0=bx?px0:px1, x1=bx?px1:px0, y0=by?py0:py1, y1=by?py1:py0, z0=bz?pz0:pz1, z1=bz?pz1:pz0, v0=bv?pv0:pv1, v1=bv?pv1:pv0;
    int lX=1+x1-x0, lY=1+y1-y0, lZ=1+z1-z0, lV=1+v1-v0;
    T *ptrd = ptr(x0,y0,z0,v0);
    if (x0+lX>(int)width)  lX-=x0+lX-width;
    if (y0+lY>(int)height) lY-=y0+lY-height;
    if (z0+lZ>(int)depth)  lZ-=z0+lZ-depth;
    if (v0+lV>(int)dim)    lV-=v0+lV-dim;
    if (x0<0) { ptrd-=x0; lX+=x0; }
    if (y0<0) { ptrd-=y0*width; lY+=y0; }
    if (z0<0) { ptrd-=z0*width*height; lZ+=z0; }
    if (v0<0) { ptrd-=v0*width*height*depth; lV+=v0; }
    for (int v=0; v<lV; v++) {
      for (int z=0; z<lZ; z++) {
        for (int y=0; y<lY; y++) {
          for (int x=0; x<lX; x++) *(ptrd++) = val;
          ptrd+=width-lX;
        }
        ptrd-=width*(lY-height);
      }
      ptrd-=width*height*(lZ-depth);
    }
    return *this;
  }
  //! Draw a filled rectangle (\p px0,\p py0,\p pz0)-(\p px1,\p py1,\p pz1) into an image with a specified color \p color
  CImg& draw_rectangle(const int px0,const int py0,const int pz0,const int px1,const int py1,const int pz1,const T* color) {
    if (!color) throw CImgArgumentException("CImg<%s>::draw_rectangle : specified color pointer is NULL",pixel_type());
    cimg_mapV(*this,k) draw_rectangle(px0,py0,pz0,k,px1,py1,pz1,k,color[k]);
    return *this;
  }
  //! Same function as above, for 2D images
  CImg& draw_rectangle(const int px0,const int py0,const int px1,const int py1,const T *color) {
    return draw_rectangle(px0,py0,0,px1,py1,depth-1,color);
  }
  
  //! Draw a filled ellipse on a 2D image
  CImg& draw_ellipse(const int x0,const int y0,const float r1,const float r2,const float ru,const float rv,const T* color) {
    const float
      norm = (float)std::sqrt(ru*ru+rv*rv),
      u = norm>0?ru/norm:1,
      v = norm>0?rv/norm:0,
      rmax = cimg::max(r1,r2),
      l1 = (float)std::pow(rmax/(r1>0?r1:1e-6),2),
      l2 = (float)std::pow(rmax/(r2>0?r2:1e-6),2),
      a = l1*u*u + l2*v*v,
      b = u*v*(l1-l2),
      c = l1*v*v + l2*u*u;
    const int
      yb = (int)std::sqrt(a*rmax*rmax/(a*c-b*b)),
      ymin = (y0-yb<0)?0:(y0-yb),
      ymax = (1+y0+yb>=(int)height)?height-1:(1+y0+yb);
    for (int y=ymin; y<=ymax; y++) {
      const float
	Y = (float)(y-y0),
	delta = b*b*Y*Y-a*(c*Y*Y-rmax*rmax);
      if (delta>=0) {
	int xmin = (int)(x0-(b*Y+std::sqrt(delta))/a), xmax = (int)(x0-(b*Y-std::sqrt(delta))/a);
	if (xmin<0) xmin=0;
	if (xmax>=(int)width) xmax=width-1;
	for (int x=xmin; x<=xmax; x++) cimg_mapV(*this,k) (*this)(x,y,0,k) = color[k];
      }
    }
    return *this;
  }
  //! Draw a filled circle on a 2D image
  CImg& draw_circle(const int x0,const int y0,float r,const T* color) {
    return draw_ellipse(x0,y0,r,r,1,0,color);
  }

  
  //! Create an auto-cropped font (along the X axis) from a input font \p font.
  static CImgl<T> get_cropfont(const CImgl<T>& font,const uint padding=2) {
    CImgl<T> res;
    cimgl_map(font,l) {
      int xmin=font[l].width, xmax = 0;
      cimg_mapXY(font[l],x,y) if (font[l](x,y)) { if (x<xmin) xmin=x; if (x>xmax) xmax=x; }
      if (xmin>xmax) res.insert(font[l]);
      else {
        res.insert(CImg<T>(xmax-xmin+1+padding,font[l].height,1,font[l].dim,0));
        cimg_mapYV(res[l],y,k) for (int x=xmin; x<=xmax; x++) res[l](x-xmin,y,0,k) = font[l](x,y,0,k);
      }
    }
    return res;
  }
  
  //! Return the default cimg font : a 3-channel masked 7x11 font.
  //! Parameter \p fixed_size allows to get the fixed-size font (true), or the non fixed-size font (false);
  static CImgl<T> get_font7x11(const bool fixed_size = false) {
    CImgl<T> font(32,1,1,1,3);
    font.insert(CImgl<T>(224,7,11,1,3)).insert(CImgl<T>(32,1,1,1,1)).insert(CImgl<T>(224,7,11,1,1));
    for (uint i=0, off=0, boff=(uint)(1<<31); i<256; i++) for (uint j=0; j<font[i].width*font[i].height; j++) {
      font[256+i](j) = font[i](j,0,0) = font[i](j,0,1) = font[i](j,0,2) = (cimg::font7x11[off]&boff)?(T)1:(T)0;
      if (!(boff>>=1)) { boff=(uint)(1<<31); off++; }
    }
    if (!fixed_size) return get_cropfont(font,2);
    return font;
  }

  //! Draw a text into an image, with the specified colors \p fgcolor and \p bgcolor, and the specified font \p pfont.
  CImg& draw_text(const int x0,const int y0,const char* text,const T *fgcolor,const T *bgcolor,const CImgl<T>& font) {
    if (font.size<256) throw CImgArgumentException("CImg<%s>::draw_text() : Specified font contains %d elements, should be 256 or 512",
                                                   pixel_type(),font.size);
    cimg::warn(font.size!=256 && font.size!=512,"CImg<%s>::draw_text() : Specified font contains %d elements, should be 256 or 512",
               pixel_type(),font.size);
    if ((fgcolor || bgcolor) && (font[' '].dim<dim))
      throw CImgArgumentException("CImg<%s>::draw_text() : Font characters have dim=%d < Image dim=%d",pixel_type(),font[' '].dim,dim);
    int x=x0, y=y0;
    CImg letter;
    for (int i=0; i<cimg::strlen(text); i++) {
      const uchar c = text[i];
      switch (c) {
      case '\n': y+=font[' '].height; x=x0; break;
      case '\t': x+=4*font[' '].width; break;
      default: {
        letter = font[c];
        const CImg& mask = font.size>=512?font[c+256]:font[c];
        if (fgcolor) for (uint p=0; p<letter.width*letter.height; p++) if (mask(p)) cimg_mapV(*this,k) letter(p,0,0,k)*=fgcolor[k];
        if (bgcolor) for (uint p=0; p<letter.width*letter.height; p++) if (!mask(p)) cimg_mapV(*this,k) letter(p,0,0,k)=bgcolor[k];
        if (!bgcolor && font.size>=512) draw_image(letter,mask,x,y); else draw_image(letter,x,y);
        x+=letter.width;
      }
        break;
      }
    }
    return *this;
  }
  //! Draw a text into an image, with the specified colors \p fgcolor and \p bgcolor, and the specified font \p pfont.
  CImg& draw_text(const int x0,const int y0,const char* text,const T *fgcolor,const T *bgcolor,const CImgl<T>* pfont) {
    return draw_text(x0,y0,text,fgcolor,bgcolor,*pfont);
  }
  //! Draw a text into an image, with the specified colors \p fgcolor and \p bgcolor and a default 7x11 font.
  CImg& draw_text(const int x0,const int y0,const char* text,const T *fgcolor=NULL,const T *bgcolor=NULL) {
    static bool first = true;
    static CImgl<T> default_font;
    if (first) { default_font = get_font7x11(); first = false; } // this trick is used to get DigitalMars Compiler working
    return draw_text(x0,y0,text,fgcolor,bgcolor,default_font);
  }

  //! Draw a text into an image, with format and arguments (2D version).
  CImg& draw_text(const int x0,const int y0,const T *fgcolor,const T *bgcolor,const CImgl<T>* pfont,const char *format,...) {
    char tmp[2048];
    va_list ap;
    va_start(ap,format);
    std::vsprintf(tmp,format,ap);
    va_end(ap);
    return pfont?draw_text(x0,y0,tmp,fgcolor,bgcolor,pfont):draw_text(x0,y0,tmp,fgcolor,bgcolor);
  }
  //! Draw a vector field in the image
  template<typename t> CImg& draw_quiver(const CImg<t>& flow,const T* color,const uint sampling=25,const float factor=-20,const int quiver_type=0) {
    cimg_test(*this); cimg_test(flow);
    if (!color) throw CImgArgumentException("CImg<%s>::draw_quiver() : specified color pointer is NULL",pixel_type());
    if (sampling<=0) throw CImgArgumentException("CImg<%s>::draw_quiver() : incorrect sampling value = %g",pixel_type(),sampling);
    if (flow.dim!=2 || flow.depth!=1) throw CImgArgumentException("CImg<%s>::draw_quiver() : specified flow has not good dimensions = (%d,%d,%d,%d)",
                                                                  pixel_type(),flow.width,flow.height,flow.depth,flow.dim);
    float vmax,fact;
    if (factor<=0) {
      CImgStats st(flow.get_norm_pointwise(2),false);
      vmax = (float)cimg::max(std::fabs(st.min),std::fabs(st.max));
      fact = -factor;
    }
    else {
      fact = factor; vmax = 1;
    }
    for (uint y=sampling/2; y<height; y+=sampling) for (uint x=sampling/2; x<width; x+=sampling) {
      uint X = x*flow.width/width, Y = y*flow.height/height;
      float u = (float)flow(X,Y,0,0)*fact/vmax, v = (float)flow(X,Y,0,1)*fact/vmax;
      if (!quiver_type) {
        int xx = x+(int)u, yy = y+(int)v;
        float norm = (float)std::sqrt((double)(u*u+v*v));
        draw_line(x,y,xx,yy,color);
        if (norm>1e-14) { u*=4/norm; v*=4/norm; }
        float theta = (float)(40.0*cimg::PI/180), ct = (float)std::cos(theta), st = (float)std::sin(theta);
        draw_line(xx,yy,(int)(xx-u*ct+v*st),(int)(yy-u*st-v*ct),color);
        theta = (float)(40.0*cimg::PI/180); ct = (float)std::cos(theta); st = (float)std::sin(theta);
        draw_line(xx,yy,(int)(xx-u*ct-v*st),(int)(yy+u*st-v*ct),color);
      } else draw_line((int)(x-0.5*u),(int)(y-0.5*v),(int)(x+0.5*u),(int)(y+0.5*v),color);
    }
    return *this;
  }

  //! Draw a 1D graphical representation of a signal 'plot' in the image
  template<typename t> CImg& draw_plot(const CImg<t>& plot,const T* color,const double ymin,const double ymax,const uint gtype=1) {
    cimg_test(*this); 
    if (!color) throw CImgArgumentException("CImg<%s>::plot() : specified color pointer is NULL",pixel_type());
    float dcol,wratio,hratio;
    int valmid,val=0,val2,y;
    uint X;
    wratio = (float)width/plot.width;
    hratio = (float)((height-1)/(ymax-ymin));
    valmid = (int)(height-1+ymin*hratio);

    if (gtype&2) {            // bar graphics
      cimg_mapX(*this,x) {
        X = (uint)(x/wratio);
        val = (int)(height-1-(plot[X]-ymin)*hratio);
        dcol = 0.6f;
        if ((x-1)*plot.width/width!=X) dcol=0.3f;
        if ((x+1)*plot.width/width!=X) dcol=1;
        if (val<=valmid && valmid>=0) {
          for (y=(valmid<(int)(height)?valmid:(int)(height)-1); y>(val<0?0:val); y--) cimg_mapV(*this,k) (*this)(x,y,0,k) = (uchar)(color[k]*dcol);
          if (val<(int)height && val>=0) cimg_mapV(*this,k) (*this)(x,val,0,k) = color[k];
        }
        if (val>valmid && valmid<(int)height) {
          for (y=(valmid>=0?valmid:0); y<(val<(int)height?val:(int)height); y++) cimg_mapV(*this,k) (*this)(x,y,0,k) = (uchar)(color[k]*dcol);
          if (val<(int)height && val>=0) cimg_mapV(*this,k) (*this)(x,val,0,k) = (uchar)(color[k]*0.3);
        }
      }
      color+=dim;
    }
    if (gtype&1)              // line graphics
      cimg_mapX(plot,x) {
        val2 = (int)(height-1-(plot[x]-ymin)*hratio);
        if (x>0) draw_line((int)((x-0.5)*wratio),val,(int)((x+0.5)*wratio),val2,color);
        val = val2;
      }
    if (valmid>=0 && valmid<(int)height) cimg_mapXV(*this,x,k) (*this)(x,valmid,0,k) = ((*this)(x,valmid,0,k)+255)/2;
    return *this;
  }

  //! Draw a 1D graphical representation of a signal 'plot' in the image
  template<typename t> CImg& draw_plot(const CImg<t>& plot,const T* color,const uint gtype=1) { 
    CImgStats st(plot,false);
    return draw_plot(plot,color,st.min,st.max,gtype);
  }
  
  bool _draw_fill_comp(const CImg& A,const CImg& B,const float sigma) {
    bool vequal = true;
    cimg_mapoff(A,off) vequal = vequal && (cimg::abs(A[off]-B[off])<=sigma);
    return vequal;
  }

  //! Draw a filled region with a specified color, starting from the given point (x,y,z) in a 3D image.
  template<typename t> CImg& draw_fill(const int x,const int y,const int z,
				       const T *color,CImg<t>& region,const float sigma=0,const bool first=true) {
    static CImg value;
    if (x<0 || x>=(int)width || y<0 || y>=(int)height || z<0 || z>=(int)depth) return *this;
    if (first) { cimg_test(*this); value = get_vector(x,y,z); region = CImg<t>(width,height,depth).fill((t)false); }
    if (!region(x,y,z) && _draw_fill_comp(value,get_vector(x,y,z),sigma)) {
      int xmin,xmax;
      set_vector(color,x,y,z);
      region(x,y,z) = (t)true;
      for (xmin=x-1; xmin>=0 && _draw_fill_comp(value,get_vector(xmin,y,z),sigma); xmin--) {
	set_vector(color,xmin,y,z); region(xmin,y,z)=(t)true; }
      for (xmax=x+1; xmax<(int)width && _draw_fill_comp(value,get_vector(xmax,y,z),sigma); xmax++) {
	set_vector(color,xmax,y,z); region(xmax,y,z)=(t)true; }
      xmin++; xmax--;
      for (; xmin<=xmax; xmin++) { 
	draw_fill(xmin,y-1,z,color,region,sigma,false); 
	draw_fill(xmin,y+1,z,color,region,sigma,false);
	draw_fill(xmin,y,z-1,color,region,sigma,false); 
	draw_fill(xmin,y,z+1,color,region,sigma,false);
      }
    }
    return *this;
  }
  //! Draw a filled region with a specified color, starting from the given point (x,y,z) in a 3D image.
  CImg& draw_fill(const int x,const int y,const int z,const T *color,const float sigma=0) {
    CImg<bool> tmp;
    return draw_fill(x,y,z,color,tmp,sigma,true);
  }
  //! Draw a filled region with a specified color, starting from the given point (x,y,z) in a 3D image.
  CImg& draw_fill(const int x,const int y,const T *color,const float sigma=0) {
    CImg<bool> tmp;
    return draw_fill(x,y,0,color,tmp,sigma,true);
  }

  //! Draw a fractal-like plasma effect
  CImg& draw_plasma(const int px0=0,const int py0=0,const int px1=1<<15,const int py1=1<<15,
                    const double alpha=1.0,const double beta=1.0) {
    cimg_test(*this);
    int x0,x1,y0,y1;
    if (px1>px0) { x0=px0; x1=px1; } else { x0=px1; x1=px0; }
    if (py1>py0) { y0=py0; y1=py1; } else { y0=py1; y1=py0; }
    if (x0<0) x0=0;
    if (x1>=dimx()) x1=width-1;
    if (y0<0) y0=0;
    if (y1>=dimy()) y1=height-1;
    const int xc = (x0+x1)/2, yc = (y0+y1)/2, dx=(xc-x0), dy=(yc-y0);
    const double dc = std::sqrt((double)(dx*dx+dy*dy))*alpha + beta;
    cimg_mapV(*this,k) {
      (*this)(xc,y0,k) = (T)(0.5*((*this)(x0,y0,k)+(*this)(x1,y0,k)));
      (*this)(xc,y1,k) = (T)(0.5*((*this)(x0,y1,k)+(*this)(x1,y1,k)));
      (*this)(x0,yc,k) = (T)(0.5*((*this)(x0,y0,k)+(*this)(x0,y1,k)));
      (*this)(x1,yc,k) = (T)(0.5*((*this)(x1,y0,k)+(*this)(x1,y1,k)));
      (*this)(xc,yc,k) = (T)(0.25*((*this)(x0,y0,k)+(*this)(x1,y0,k)+(*this)(x1,y1,k)+(*this)(x0,y1,k)) + dc*cimg::grand());
    }
    if (xc!=x0 || yc!=y0) { 
      draw_plasma(x0,y0,xc,yc,alpha,beta);
      draw_plasma(xc,y0,x1,yc,alpha,beta);
      draw_plasma(x0,yc,xc,y1,alpha,beta);
      draw_plasma(xc,yc,x1,y1,alpha,beta); 
    }
    return *this;
  }
  
  //@}
  //---------------------------------------
  //---------------------------------------
  //
  //! \name Filtering functions
  //@{
  //---------------------------------------
  //---------------------------------------
  
  //! Return the correlation of the image by a mask.
  CImg get_correl(const CImg& mask,const uint cond=1,const bool weighted_correl=false) const {
    cimg_test_scalar(mask);
    CImg dest(*this,false);
    const int cxm=mask.width/2, cym=mask.height/2, czm=mask.depth/2, fxm=cxm-1+(mask.width%2), fym=cym-1+(mask.height%2), fzm=czm-1+(mask.depth%2);
    cimg_mapV(*this,v) 
      if (!weighted_correl) {	// Classical correlation
	for (int z=czm; z<(int)depth-czm; z++) for (int y=cym; y<(int)height-cym; y++) for (int x=cxm; x<(int)width-cxm; x++) {
	  double val = 0;
	  for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++)
	    val+= (*this)(x+xm,y+ym,z+zm,v)*mask(cxm+xm,cym+ym,czm+zm,0);
	  dest(x,y,z,v)=(T)val;
	}
	if (cond) cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++)
              val+= neumann_pix3d(x+xm,y+ym,z+zm,v)*mask(cxm+xm,cym+ym,czm+zm,0);
            dest(x,y,z,v)=(T)val;
          }
	else cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++)  for (int xm=-cxm; xm<=fxm; xm++)
              val+= dirichltet_pix3d(x+xm,y+ym,z+zm,v,0)*mask(cxm+xm,cym+ym,czm+zm,0);
            dest(x,y,z,v)=(T)val;
          }
      } else {	// Weighted correlation
	for (int z=czm; z<(int)depth-czm; z++) for (int y=cym; y<(int)height-cym; y++) for (int x=cxm; x<(int)width-cxm; x++) {
	  double val = 0, norm = 0;
	  for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++) {
	    const T cval = (*this)(x+xm,y+ym,z+zm,v);
	    val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
	    norm+= cval*cval;
	  }
	  dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
	}
	if (cond) cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0, norm = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++) {
              const T cval = neumann_pix3d(x+xm,y+ym,z+zm,v);
              val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
              norm+=cval*cval;
            }
            dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
          }
	else cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0, norm = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++)  for (int xm=-cxm; xm<=fxm; xm++) {
              const T cval = dirichlet_pix3d(x+xm,y+ym,z+zm,v,0);
              val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
              norm+= cval*cval;
            }
            dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
          }
      }
    return dest;
  }
  //! Correlate the image by a mask
  CImg& correl(const CImg& mask,const uint cond=1,const bool weighted_correl=false) { return get_correl(mask,cond,weighted_correl).swap(*this); }

  
  //! Return the convolution of the image by a mask
  CImg get_convol(const CImg& mask,const uint cond=1,const bool weighted_correl=false) const {
    cimg_test_scalar(mask);
    CImg dest(*this,false);
    const int cxm=mask.width/2, cym=mask.height/2, czm=mask.depth/2, fxm=cxm-1+(mask.width%2), fym=cym-1+(mask.height%2), fzm=czm-1+(mask.depth%2);
    cimg_mapV(*this,v) 
      if (!weighted_correl) {	// Classical convolution
	for (int z=czm; z<(int)depth-czm; z++) for (int y=cym; y<(int)height-cym; y++) for (int x=cxm; x<(int)width-cxm; x++) {
	  double val = 0;
	  for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++)
	    val+= (*this)(x-xm,y-ym,z-zm,v)*mask(cxm+xm,cym+ym,czm+zm,0);
	  dest(x,y,z,v)=(T)val;
	}
	if (cond) cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++)
              val+= neumann_pix3d(x-xm,y-ym,z-zm,v)*mask(cxm+xm,cym+ym,czm+zm,0);
            dest(x,y,z,v)=(T)val;
          }
	else cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++)  for (int xm=-cxm; xm<=fxm; xm++)
              val+= dirichlet_pix3d(x-xm,y-ym,z-zm,v,0)*mask(cxm+xm,cym+ym,czm+zm,0);
            dest(x,y,z,v)=(T)val;
          }
      } else {	// Weighted convolution
	for (int z=czm; z<(int)depth-czm; z++) for (int y=cym; y<(int)height-cym; y++) for (int x=cxm; x<(int)width-cxm; x++) {
	  double val = 0, norm = 0;
	  for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++) {
	    const T cval = (*this)(x-xm,y-ym,z-zm,v);
	    val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
	    norm+= cval*cval;
	  }
          dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
	}
	if (cond) cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0, norm = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++) for (int xm=-cxm; xm<=fxm; xm++) {
              const T cval = neumann_pix3d(x-xm,y-ym,z-zm,v);
              val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
              norm+=cval*cval;
            }
            dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
          }
	else cimg_mapYZV(*this,y,z,v)
          for (int x=0; x<(int)width; (y<cym || y>=(int)height-cym || z<czm || z>=(int)depth-czm)?x++:((x<cxm-1 || x>=(int)width-cxm)?x++:(x=(int)width-cxm))) {
            double val = 0, norm = 0;
            for (int zm=-czm; zm<=fzm; zm++) for (int ym=-cym; ym<=fym; ym++)  for (int xm=-cxm; xm<=fxm; xm++) {
              const T cval = dirichlet_pix3d(x-xm,y-ym,z-zm,v,0);
              val+= cval*mask(cxm+xm,cym+ym,czm+zm,0);
              norm+= cval*cval;
            }
            dest(x,y,z,v)=(norm!=0)?(T)(val/std::sqrt(norm)):0;
          }
      }
    return dest;
  }

  //! Convolve the image by a mask
  CImg& convol(const CImg& mask,const uint cond=1,const bool weighted_convol=false) { return get_convol(mask,cond,weighted_convol).swap(*this); }

  //! Return the convolution of the image by a 3x3 mask (fast implementation)
  CImg get_convol3x3(const double m[3][3],const bool weighted_convol=false) const {
    CImg res(*this,false);
    CImg_3x3(I,T);
    if (!weighted_convol) cimg_mapZV(*this,z,v) cimg_map3x3(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_conv3x3(I,m);
    else cimg_mapZV(*this,z,v) cimg_map3x3(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum3x3(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_conv3x3(I,m)/std::sqrt(norm)):0;
    }    
    return res;
  }
  //! Convolve the image by a 3x3 mask (fast implementation)
  CImg& convol3x3(const double m[3][3],const bool weighted_convol=false) { return get_convol3x3(m,weighted_convol).swap(*this); }
  //! Return the correlation of the image by a 3x3 mask (fast implementation)
  CImg get_correl3x3(const double m[3][3],const bool weighted_correl=false) const {
    CImg res(*this,false);
    CImg_3x3(I,T);
    if (!weighted_correl) cimg_mapZV(*this,z,v) cimg_map3x3(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_corr3x3(I,m);
    else cimg_mapZV(*this,z,v) cimg_map3x3(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum3x3(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_corr3x3(I,m)/std::sqrt(norm)):0;
    }
    return res;
  }
  //! Correlate the image by a 3x3 mask (fast implementation)
  CImg& correl3x3(const double m[3][3],const bool weighted_correl=false) { return get_correl3x3(m,weighted_correl).swap(*this); }

  //! Return the convolution of the image by a 5x5 mask (fast implementation)
  CImg get_convol5x5(const double m[5][5],const bool weighted_convol=false) const {
    CImg res(*this,false);
    CImg_5x5(I,T);
    if (!weighted_convol) cimg_mapZV(*this,z,v) cimg_map5x5(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_conv5x5(I,m);
    else cimg_mapZV(*this,z,v) cimg_map5x5(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum5x5(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_conv5x5(I,m)/std::sqrt(norm)):0;
    }    
    return res;
  }
  //! Convolve the image by a 5x5 mask (fast implementation)
  CImg& convol5x5(const double m[5][5],const bool weighted_convol=false) { return get_convol5x5(m,weighted_convol).swap(*this); }
  //! Return the correlation of the image by a 5x5 mask (fast implementation)
  CImg get_correl5x5(const double m[3][3],const bool weighted_correl=false) const {
    CImg res(*this,false);
    CImg_5x5(I,T);
    if (!weighted_correl) cimg_mapZV(*this,z,v) cimg_map5x5(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_corr5x5(I,m);
    else cimg_mapZV(*this,z,v) cimg_map5x5(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_corr5x5(I,m)/std::sqrt(norm)):0;
    }
    return res;
  }
  //! Correlate the image by a 5x5 mask (fast implementation)
  CImg& correl5x5(const double m[5][5],const bool weighted_correl=false) { return get_correl5x5(m,weighted_correl).swap(*this); }

  //! Return the convolution of the image by a 3x3x3 mask (fast implementation)
  CImg get_convol3x3x3(const double m[3][3][3],const bool weighted_convol=false) const {
    CImg res(*this,false);
    CImg_3x3x3(I,T);
    if (!weighted_convol) cimg_mapZV(*this,z,v) cimg_map3x3x3(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_conv3x3x3(I,m);
    else cimg_mapZV(*this,z,v) cimg_map3x3x3(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum3x3x3(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_conv3x3x3(I,m)/std::sqrt(norm)):0;
    }    
    return res;
  }
  //! Convolve the image by a 3x3x3 mask (fast implementation)
  CImg& convol3x3x3(const double m[3][3][3],const bool weighted_convol=false) { return get_convol3x3x3(m,weighted_convol).swap(*this); }

  //! Return the correlation of the image by a 3x3x3 mask (fast implementation)
  CImg get_correl3x3x3(const double m[3][3][3],const bool weighted_correl=false) const {
    CImg res(*this,false);
    CImg_3x3x3(I,T);
    if (!weighted_correl) cimg_mapZV(*this,z,v) cimg_map3x3x3(*this,x,y,z,v,I) res(x,y,z,v) = (T)cimg_corr3x3x3(I,m);
    else cimg_mapZV(*this,z,v) cimg_map3x3x3(*this,x,y,z,v,I) {
      const double norm = (double)cimg_squaresum(I);
      res(x,y,z,v) = (norm!=0)?(T)(cimg_corr3x3x3(I,m)/std::sqrt(norm)):0;
    }
    return res;
  }
  //! Correlate the image by a 3x3x3 mask (fast implementation)
  CImg& correl3x3x3(const double m[3][3][3],const bool weighted_correl=false) { return get_correl3x3x3(m,weighted_correl).swap(*this); }

  //! Add noise to the image
  CImg& noise(const double psigma=-20,const uint ntype=0) {
    cimg_test(*this);
    double sigma = psigma;
    static bool first_time = true;
    if (first_time) { std::srand((uint)time(NULL)); first_time = false; }
    CImgStats st;
    T *ptr;
    if (sigma==0) return *this;
    if (sigma<0 || ntype==2) st = CImgStats(*this,false);
    if (sigma<0) sigma = -sigma*(st.max-st.min)/100.0;
    switch (ntype) {
    case 0: cimg_map(*this,ptr) *ptr=(T)(*ptr+sigma*cimg::grand());	break;    // Gaussian noise
    case 1: cimg_map(*this,ptr) *ptr=(T)(*ptr+sigma*cimg::crand());	break;    // Uniform noise
    case 2:                                                                       // Salt & Pepper
      if (st.max==st.min) { st.min=0; st.max=255; }
      cimg_map(*this,ptr) if (cimg::rand()*100<sigma) *ptr=(T)(cimg::rand()<0.5?st.max:st.min);
      break;
    }
    return *this;
  }
  //! Return a noisy image
  CImg get_noise(const double sigma=-20,const uint ntype=0) const { return CImg<T>(*this).noise(sigma,ntype); }

  //! Apply a deriche filter on the image
  CImg& deriche(const float sigma=1,const int order=0,const char axe='x',const uint cond=1) {
    cimg_test(*this);
    if (sigma<0 || order<0 || order>2) throw CImgArgumentException("CImg<%s>::deriche() : Bad arguments (sigma=%g, order=%d)",pixel_type(),sigma,order);
    if (sigma<0.01f) return *this;
    const float alpha=sigma>0?1.695f/sigma:0,ea=(float)std::exp(alpha),ema=(float)std::exp(-alpha),em2a=ema*ema,b1=2*ema,b2=-em2a;
    float ek,ekn,parity,a1,a2,a3,a4,g0,sumg1,sumg0;
    double *Y,Y0,Y1,Y2;
    int i,offset,nb;
    T *ima,I1,I2;
    switch(order) {
    case 1:                 // first derivative
      ek = -(1-ema)*(1-ema)*(1-ema)/(2*(ema+1)*ema); a1 = a4 = 0;  a2 = ek*ema; a3 = -ek*ema; parity =-1;\
      if (cond) { sumg1 = (ek*ea) / ((ea-1)*(ea-1)); g0 = 0; sumg0 = g0+sumg1; } \
      else g0 = sumg0 = sumg1 = 0;
      break;
    case 2:               // second derivative
      ekn = ( -2*(-1+3*ea-3*ea*ea+ea*ea*ea)/(3*ea+1+3*ea*ea+ea*ea*ea) );
      ek = -(em2a-1)/(2*alpha*ema); a1 = ekn;  a2 = -ekn*(1+ek*alpha)*ema; a3 = ekn*(1-ek*alpha)*ema; a4 = -ekn*em2a; parity =1;
      if (cond) { sumg1 = ekn/2; g0 = ekn; sumg0 = g0+sumg1; }
      else g0=sumg0=sumg1=0;
      break;
    default:              // smoothing
      ek = (1-ema)*(1-ema) / (1+2*alpha*ema - em2a); a1 = ek;  a2 = ek*ema*(alpha-1); a3 = ek*ema*(alpha+1); a4 = -ek*em2a; parity = 1;
      if (cond) { sumg1 = ek*(alpha*ea+ea-1) / ((ea-1)*(ea-1)); g0 = ek; sumg0 = g0+sumg1; }
      else  g0=sumg0=sumg1=0;
      break;
    }
    // filter init
    Y = new double[cimg::max(width,height,depth)];
    switch(cimg::uncase(axe)) {
    case 'x': if (width>1)  { offset = 1;            nb = width;  cimg_mapYZV(*this,y,z,k) cimg_deriche_map(0,y,z,k,nb,offset,T); }	break;
    case 'y': if (height>1) { offset = width;        nb = height; cimg_mapXZV(*this,x,z,k) cimg_deriche_map(x,0,z,k,nb,offset,T); }	break;
    case 'z': if (depth>1)  { offset = width*height; nb = depth;  cimg_mapXYV(*this,x,y,k) cimg_deriche_map(x,y,0,k,nb,offset,T); }	break;
    default : cimg::warn(true,"CImg<%s>::deriche() : Unknown axe '%c'",pixel_type(),axe); break;
    }
    delete[] Y;
    return *this;
  }
  //! Return the result of the Deriche filter
  CImg get_deriche(const float sigma=1,const int order=0,const char axe='x',const uint cond=1) const {
    return CImg<T>(*this).deriche(sigma,order,axe,cond);
  }
  //! Blur the image with a Deriche filter (quasi-gaussian filter)
  CImg& blur(const float sigma=1,const uint cond=1) {
    cimg_test(*this);
    if (width>1)  deriche(sigma,0,'x',cond);
    if (height>1) deriche(sigma,0,'y',cond);
    if (depth>1)  deriche(sigma,0,'z',cond);
    return *this;
  }
  //! Return a blurred version of the image, using a Deriche filter (quasi gaussian filter)
  CImg get_blur(const float sigma=1,const uint cond=1) const { return CImg<T>(*this).blur(sigma,cond); }

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Matrix computation
  //@{
  //------------------------------------------
  //------------------------------------------

  //! Operator* (matrix product)
  template<typename t> CImg operator*(const CImg<t>& img) const {
    cimg_test_matrix(*this);
    cimg_test_matrix(img);
    if (width!=img.height) 
      throw CImgArgumentException("CImg<%s>::operator*() : can't multiply a matrix *this = (%dx%d) by a matrix (%dx%d)",
                                  pixel_type(),width,height,img.width,img.height);
    CImg res(height,img.width);
    double val;
    cimg_mapXY(res,i,j) { val=0; cimg_mapX(*this,k) val+=(*this)(k,i)*img(j,k); res(j,i) = (T)val; }
    return res;
  }
  //! Operator*= (matrix product)
  template<typename t> CImg& operator*=(const CImg<t>& img) { return ((*this)*img).swap(*this); }
  
  //! Return a new image corresponding to the vector located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg get_vector(const uint x=0,const uint y=0,const uint z=0) const {
    CImg dest(dim);
    cimg_mapV(*this,k) dest[k]=(*this)(x,y,z,k);
    return dest;
  }
  
  //! Return a new image corresponding to the \a square \a matrix located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg get_matrix(const uint x=0,const uint y=0,const uint z=0) const {
    const int n = (int)std::sqrt((double)dim);
    CImg dest(n,n);
    cimg_mapV(*this,k) dest[k]=(*this)(x,y,z,k);
    return dest;
  }
  
  //! Return a new image corresponding to the \a diffusion \a tensor located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg get_tensor(const uint x=0,const uint y=0,const uint z=0) const {
    CImg dest(dim==3?2:3,dim==3?2:3,1,1);
    if (dim==3) {
      dest[0] = (*this)(x,y,z,0);
      dest[1] = dest[2] = (*this)(x,y,z,1);
      dest[3] = (*this)(x,y,z,2);
    }
    else {
      dest[0] = (*this)(x,y,z,0);
      dest[1] = dest[3] = (*this)(x,y,z,1);
      dest[2] = dest[6] = (*this)(x,y,z,2);
      dest[4] = (*this)(x,y,z,3);
      dest[5] = dest[7] = (*this)(x,y,z,4);
      dest[8] = (*this)(x,y,z,5);
    }
    return dest;
  }
  //! Set the image \p vec as the \a vector \a valued pixel located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg& set_vector(const T* vec,const uint x=0,const uint y=0,const uint z=0) {
    cimg_mapV(*this,k) (*this)(x,y,z,k)=vec[k];
    return *this;
  }
  //! Set the image \p vec as the \a square \a matrix-valued pixel located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg& set_matrix(const CImg& mat,const uint x=0,const uint y=0,const uint z=0) {
    return set_vector(mat,x,y,z);
  }
  //! Set the image \p vec as the \a tensor \a valued pixel located at (\p x,\p y,\p z) of the current vector-valued image.
  CImg& set_tensor(const CImg& ten,const uint x=0,const uint y=0,const uint z=0) {
    if (ten.height==2) {
      (*this)(x,y,z,0)=ten[0];
      (*this)(x,y,z,1)=ten[1];
      (*this)(x,y,z,2)=ten[3];
    }
    else {
      (*this)(x,y,z,0)=ten[0];
      (*this)(x,y,z,1)=ten[1];
      (*this)(x,y,z,2)=ten[2];
      (*this)(x,y,z,3)=ten[4];
      (*this)(x,y,z,4)=ten[5];
      (*this)(x,y,z,5)=ten[8];
    }
    return *this;
  }
  //! Set the current matrix to be the matrix \p factor * \a Identity.
  CImg& eye(const T& factor=(T)1) {    
    cimg_test_square(*this);
    fill(0);
    for (uint x=0; x<width; x++) (*this)(x,x) = factor;
    return *this;
  }
  //! Return a matrix \p dim * \p dim equal to \p factor * \a Identity.
  static CImg get_eye(const uint dim,const T& factor=(T)1) {
    return CImg<T>(dim,dim).eye(factor);
  }
  
  //! Return the transpose version of the current matrix.
  CImg get_transpose() const {
    cimg_test_matrix(*this);
    CImg res(height,width);
    cimg_mapXY(res,x,y) res(x,y) = (*this)(y,x);
    return res;
  }
  //! Replace the current matrix by its transpose.
  CImg& transpose() { return get_transpose().swap(*this); }

  //! Get a diagonal matrix, whose diagonal coefficients are the coefficients of the input image
  CImg get_diagonal() const {
    cimg_test(*this);
    CImg res(size(),size(),1,1,0);
    cimg_mapoff(*this,off) res(off,off)=(*this)(off);
    return res;
  }
  //! Replace a vector by a diagonal matrix containing the original vector coefficients.
  CImg& diagonal() { return get_diagonal().swap(*this); }

  //! Inverse the current matrix.
  CImg& inverse() {
    cimg_test_square(*this);
    switch (width) {
    case 2:
      {
        const double 
          a = data[0], c = data[1],
          b = data[2], d = data[3],
          dete = det();
        if (dete) { 
          data[0] = (T)(d/dete);  data[1] = (T)(-c/dete);
          data[2] = (T)(-b/dete), data[3] = (T)(a/dete); 
        } else {
          cimg::warn(true,"CImg<%s>::inverse() : Matrix determinant is 0, can't invert matrix",pixel_type());
          fill(0);
        }
      }
      break;
    case 3:
      {
        const double
          a = data[0], d = data[1], g = data[2],
          b = data[3], e = data[4], h = data[5],
          c = data[6], f = data[7], i = data[8],
          dete = det();
        if (dete) {
          data[0] = (T)((i*e-f*h)/dete), data[1] = (T)((g*f-i*d)/dete), data[2] = (T)((d*h-g*e)/dete);
          data[3] = (T)((h*c-i*b)/dete), data[4] = (T)((i*a-c*g)/dete), data[5] = (T)((g*b-a*h)/dete);
          data[6] = (T)((b*f-e*c)/dete), data[7] = (T)((d*c-a*f)/dete), data[8] = (T)((a*e-d*b)/dete);
        } else {
          cimg::warn(true,"CImg<%s>::inverse() : Matrix determinant is 0, can't invert matrix",pixel_type());
          fill(0);
        }
      }
      break;
    default:
      {        
          int N = width, LWORK = 4*N, *IPIV = new int[N], INFO;
          double *A = new double[N*N], *WORK = new double[LWORK];
          for (uint k=0; k<(uint)N; k++) for (uint l=0; l<(uint)N; l++) A[k*N+l] = (*this)(k,l);
          dgetrf_(&N,&N,A,&N,IPIV,&INFO);
          cimg::warn(INFO!=0,"CImg<%s>::inverse() : LAPACK Error code = %d, from dgetrf_()",pixel_type(),INFO);
          if (!INFO) {
            dgetri_(&N,A,&N,IPIV,WORK,&LWORK,&INFO);
            cimg::warn(INFO!=0,"CImg<%s>::inverse() : LAPACK Error code = %d, from dgetri_()",pixel_type(),INFO);
          }
          if (!INFO) for (uint k=0; k<(uint)N; k++) for (uint l=0; l<(uint)N; l++) (*this)(k,l) = (T)(A[k*N+l]);
          else fill(0);
          delete[] IPIV; delete[] A; delete[] WORK;        
      }
    }
    return *this;
  }
  //! Return the inverse of the current matrix.
  CImg get_inverse() const { return CImg<T>(*this).inverse(); }

  //! Return the trace of the current matrix.
  double trace() const {
    cimg_test_square(*this);
    double res=0;
    cimg_mapX(*this,k) res+=(*this)(k,k);
    return res;
  }
  //! Return the dot product of the current vector/matrix with the vector/matrix \p img.
  double dot(const CImg& img) const {
    cimg_test(*this); cimg_test(img);
    const uint nb = cimg::min(size(),img.size());
    double res=0;
    for (uint off=0; off<nb; off++) res+=data[off]*img[off];
    return res;
  }
	
  //! Return the cross product between two 3d vectors
  CImg& cross(const CImg& img) {
    if (width!=1 || height<3 || img.width!=1 || img.height<3)
      throw CImgInstanceException("CImg<%s>::cross() : cannot get cross product between two matrices (%d,%d) and (%d,%d)",
                                  pixel_type(),width,height,img.width,img.height);
    const T x = (*this)[0], y = (*this)[1], z = (*this)[2];
    (*this)[0] = y*img[2]-z*img[1];
    (*this)[1] = z*img[0]-x*img[2];
    (*this)[2] = x*img[1]-y*img[0];
    return *this;
  }
  //! Return the cross product between two 3d vectors
  CImg get_cross(const CImg& img) const { return CImg<T>(*this).cross(img); }

  //! Return the determinant of the current matrix.
  double det() const {
    cimg_test_square(*this);
    switch (width) {
    case 1: return (*this)(0,0);
    case 2: return (*this)(0,0)*(*this)(1,1)-(*this)(0,1)*(*this)(1,0);
    case 3: 
      {
        const double
          a = data[0], d = data[1], g = data[2],
          b = data[3], e = data[4], h = data[5],
          c = data[6], f = data[7], i = data[8];
        return i*a*e-a*h*f-i*b*d+b*g*f+c*d*h-c*g*e;
      }
    }
    return 0;
  }
  //! Return the norm of the current vector/matrix. \p ntype = norm type (0=L2, 1=L1, -1=Linf).
  double norm(const int ntype=2) const {
    cimg_test(*this);
    double res = 0;
    switch (ntype) {
    case -1: { cimg_mapoff(*this,off) if (std::fabs((double)data[off])>res) res = std::fabs((double)data[off]); return res; }
    case 1 : { cimg_mapoff(*this,off) res+=std::fabs((double)data[off]); return res; }
    default: { return std::sqrt(dot(*this)); }
    }
    return 0;
  }
  //! Return the sum of all the pixel values in an image.
  double sum() const {
    cimg_test(*this);		
    double res=0;
    T *ptr;
    cimg_map(*this,ptr) res+=*ptr;
    return res;
  }
  //! Compute the eigenvalues and eigenvectors of a general matrix.
  template<typename t> const CImg<T>& eigen(CImg<t>& val, CImg<t> &vec) const {
    cimg_test_square(*this);
    if (val.size()<width) 
      throw CImgArgumentException("CImg<%s>::eigen() : Argument 'val' is not large enough to be filled with eigenvalues (size=%d, needed is %d)",
                                  pixel_type(),val.size(),width);
    if (vec.data && vec.size()<width*width) 
      throw CImgArgumentException("CImg<%s>::eigen() : Argument 'vec' is not large enough to be filled with eigenvectors (size=%d, needed is %d)",
                                  pixel_type(),val.size(),width*width);
    switch(width) {
    case 1:
      val[0]=(t)(*this)[0]; 
      if (vec.data) vec[0]=(t)1;
      break;
    case 2:
      {
        const double
          a = (*this)[0], b = (*this)[1],
          c = (*this)[2], d = (*this)[3],
          e = a+d;
        double f = e*e-4*(a*d-b*c);
        cimg::warn(f<0,"CImg<%s>::eigen() : Complex eigenvalues",pixel_type());
        f = std::sqrt(f);
        const double l1 = 0.5*(e-f), l2 = 0.5*(e+f);
        val[0]=(t)l1; val[1]=(t)l2;
        if (vec.data) {
          double u,v,n;
          if (std::fabs(b)>std::fabs(a-l1)) { u = 1; v = (l1-a)/b; }
          else { if (a-l1!=0) { u = -b/(a-l1); v = 1; } else { u = 1; v = 0; } }
          n = std::sqrt(u*u+v*v); u/=n; v/=n; vec[0] = (t)u; vec[1] = (t)v;
          if (std::fabs(b)>std::fabs(a-l2)) { u = 1; v = (l2-a)/b; }
          else { if (a-l2!=0) { u = -b/(a-l2); v = 1; } else { u = 0; v = 1; } }
          n = std::sqrt(u*u+v*v); u/=n; v/=n; vec[2] = (t)u; vec[3] = (t)v;
        }
      }
      break;
    default: 
      throw CImgInstanceException("CImg<%s>::eigen() : Eigenvalues computation of general matrices is limited to 2x2 matrices (given is %dx%d)",
                                  pixel_type(),width,height);
    }
    return *this;
  }

  //! Compute the eigenvalues of a general matrix.
  template<typename t> const CImg<T>& eigen(CImg<t>& val) const { CImg foo; return eigen(val,foo); }
  CImgl<T> get_eigen(const bool compute_vectors=true) const {
    cimg_test_square(*this);
    CImgl<T> res(1,1,width);
    if (compute_vectors) res.insert(CImg<T>(width,width));
    eigen(res[0],res[1]);
    return res;
  }

  //! Compute the eigenvalues and eigenvectors of a symmetric matrix.
  template<typename t> const CImg<T>& symeigen(CImg<t>& val, CImg<t>& vec) const {
    cimg_test_square(*this);
   if (val.size()<width) 
      throw CImgArgumentException("CImg<%s>::symeigen() : Argument 'val' is not large enough to be filled with eigenvalues (size=%d, needed is %d)",
                                  pixel_type(),val.size(),width);
    if (vec.data && vec.size()<width*width) 
      throw CImgArgumentException("CImg<%s>::symeigen() : Argument 'vec' is not large enough to be filled with eigenvectors (size=%d, needed is %d)",
                                  pixel_type(),val.size(),width*width);
    char JOBZ=vec.data?'V':'N', UPLO='U';
    int N,INFO=0,LWORK;
    double *WORK,*A,*VAL;
    if (width<3) return eigen(val,vec);
    N = width;
    LWORK = 5*N;
    A    = new double[N*N];
    WORK = new double[LWORK];
    VAL  = new double[width];
    for (uint k=0; k<(uint)N; k++) for (uint l=0; l<(uint)N; l++) A[k*N+l] = (*this)(k,l);
    dsyev_(&JOBZ,&UPLO,&N,A,&N,VAL,WORK,&LWORK,&INFO);
    cimg::warn(INFO!=0,"CImg<%s>::symeigen() : LAPACK Error code = %d, from ssyev_()",pixel_type(),INFO);
    cimg_mapX(*this,x) val(x) = (t)VAL[x];
    if (vec.data) cimg_mapXY(*this,x,y) vec(x,y) = (t)A[x+y*N];
    delete[] A; 
    delete[] WORK;
    delete[] VAL;
    return *this;
  }
  //! Compute the eigenvalues of a symmetric matrix.
  template<typename t> const CImg<T>& symeigen(CImg<t>& val) const { CImg foo; return symeigen(val,foo); }
  CImgl<T> get_symeigen(const bool compute_vectors=true) const {
    cimg_test_square(*this);
    CImgl<T> res(1,1,width);
    if (compute_vectors) res.insert(CImg<T>(width,width));
    symeigen(res[0],res[1]);
    return res;
  }

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Display functions
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Open a Window with the specified image and return a pointer to the corresponding CImgDisplay structure
  CImgDisplay* new_display(const char *title="",const int normalize=1,const uint event_type=3,const bool visible=true) const {
    return new CImgDisplay(*this,title,normalize,event_type,visible);
  }
  //! Display an image into a CImgDisplay window.
  const CImg& display(CImgDisplay& disp,const uint ymin=0,const uint ymax=(uint)(-1)) const { disp.display(*this,ymin,ymax);  return *this; }
  //! Display an image into a CImgDisplay window.
  const CImg& display(CImgDisplay* disp,const uint ymin=0,const uint ymax=(uint)(-1)) const { disp->display(*this,ymin,ymax); return *this; }
  //! Same as \ref cimg::wait_sync()
  const CImg& wait_sync(const uint milliseconds) const { cimg::wait_sync(milliseconds); return *this;  }
  
  //! Display an image in a window with a title \p title, and wait a 'closed' or 'keyboard' event.\n
  //! Parameters \p min_size and \p max_size set the minimum and maximum dimensions of the display window.
  //! If negative, they corresponds to a percentage of the original image size.
  const CImg& display(const char* title,const int min_size=64,const int max_size=1024) const {
    cimg_test(*this);
    CImgDisplay *disp;
    uint w = width, h = height, XYZ[3];
    print(title);
    if (depth!=0) { w+=depth; h+=depth; }
    const uint dmin = cimg::min(w,h), minsiz = min_size>=0?min_size:(-min_size)*dmin/100;
    if (dmin<minsiz) { w=w*minsiz/dmin; h=h*minsiz/dmin; }
    const uint dmax = cimg::max(w,h), maxsiz = max_size>=0?max_size:(-max_size)*dmax/100;
    if (dmax>maxsiz) { w=w*maxsiz/dmax; h=h*maxsiz/dmax; }
    if (w!=width || h!=height) disp = CImg<uchar>(w,h,1,1,0).new_display(title,0,3); 
    else disp = new_display(title,0,3);
    XYZ[0] = width/2; XYZ[1] = height/2; XYZ[2] = depth/2;
    while (!disp->closed && !disp->key) feature_selection(NULL,1,disp,XYZ);
    delete disp;
    return *this;
  }

  //! Display an image in a window, with a default title. See also \see display() for details on parameters.
  const CImg& display(const int min_size=64,const int max_size=1024) const { return display("CImg",min_size,max_size); }
  
  //! High-level interface to select features in images
  const CImg& feature_selection(int *const selection, const int feature_type, CImgDisplay& xdisp,
                                uint *const XYZ=NULL,const uchar* pcolor=NULL) const {
    feature_selection(selection,feature_type,&xdisp,XYZ,pcolor);
  }
  //! High-level interface to select features in images
  const CImg& feature_selection(int *const selection=NULL, const int feature_type=0, CImgDisplay *xdisp=NULL,
				uint *const XYZ=NULL,const uchar* pcolor=NULL) const {
    cimg_test(*this);
    uchar color[3]={255,255,105};
    if (pcolor) { color[0]=pcolor[0]; color[1]=pcolor[1]; color[2]=pcolor[2]; }
    CImgDisplay *disp = xdisp?xdisp:new CImgDisplay(width,height,"",0,3);                                                                                   
    int texty=0,X0,Y0,X1,Y1,chatch=0,hatch,clicked=0,zone,czone,xo,yo,x0,y0,z0,x1,y1,z1,xc,yc,zc,obutt,butt=0,mousex,mousey;
    CImg<uchar> visu0,visu;
    char text[1024];
    
    xc=yc=zc=x0=y0=z0=x1=y1=z1=zone=czone=-1;
    xo=yo=mousex=mousey=-2;
    uint lx,ly, xp = (XYZ?XYZ[0]:width/2)%width, yp = (XYZ?XYZ[1]:height/2)%height, zp = (XYZ?XYZ[2]:depth/2)%depth;
    if (depth==1) { lx=disp->width; ly=disp->height; }
    else { lx = width*disp->width/(width+depth); ly = height*disp->height/(height+depth); }

    while (!disp->key && !disp->closed && (!(feature_type==0 && clicked)) && (!(depth==1 && clicked==2)) && clicked<3) {

      // Handle mouse motion and buttons
      obutt = butt; xo=mousex; yo=mousey; mousex=disp->mousex; mousey=disp->mousey; butt=disp->button;
      if (mousex>=0 && mousey>=0) {
        if (mousex<(int)lx  && mousey<(int)ly ) { xc = mousex*width/lx; yc = mousey*height/ly; zc = zp;                    zone=0; }
        if (mousex>=(int)lx && mousey<(int)ly ) { xc = xp; yc = mousey*height/ly; zc = (mousex-lx)*depth/(disp->width-lx); zone=1; }
        if (mousex<(int)lx  && mousey>=(int)ly) { xc = mousex*width/lx; yc = yp; zc = (mousey-ly)*depth/(disp->height-ly); zone=2; }
        if (mousex>=(int)lx && mousey>=(int)ly) { xc = xp; yc = yp; zc = zp;                                               zone=3; }
        switch(clicked) {
        case 0: x0 = x1 = xc; y0 = y1 = yc; z0 = z1 = zc; break;
        case 1:
          if (zone==czone) switch(czone) {
          case 0:  x1 = xc; y1 = yc; break;
          case 1:  y1 = yc; z1 = zc; break;
          default: z1 = zc; x1 = xc; break;
          }
          break;
        case 2:
          switch(czone) {
          case 0: xc = x1; yc = y1; break;
          case 1: yc = y1; zc = z1; break;
          default: zc = z1; xc = x1; break;
          }
          x1 = xc; y1 = yc; z1 = zc; break;
        }
        if ((clicked==1 && czone==zone) || clicked==2 || butt&2) { xp = xc; yp = yc; zp = zc; }
        if (butt&4) clicked = 0;
        if (((clicked!=1 && butt&1) || (clicked==1 && !(obutt&1))) && zone<3) { czone=zone; clicked++; }
      }

      // Drawing
      if (xo!=mousex || yo!=mousey || clicked || butt) {
        text[0]='\0';
        if (depth==1 && !visu0.data) visu0 = get_resize(disp->width,disp->height,1,dim<3?dim:3,1).normalize((T)0,(T)255);
        if (depth!=1 && (!visu0.data || clicked || (butt&2)))
          visu0 = get_3dplanes(xp,yp,zp).normalize((T)0,(T)255).resize(disp->width,disp->height,1,dim<3?dim:3,1);
        visu = visu0;
        if ((clicked || !feature_type) && x0>=0) {
          if (!feature_type) hatch=0x100; else hatch = 0x30 + (chatch<<8);
					
          if (feature_type==1) {
            X0 = (int)((x0+0.5)*lx/width);  X1 = (int)((x1+0.5)*lx/width);
            Y0 = (int)((y0+0.5)*ly/height); Y1 = (int)((y1+0.5)*ly/height);
            visu.draw_line(X0,Y0,X1,Y1,color,hatch);
            if (depth==1) std::sprintf(text,"Vect (%d,%d)-(%d,%d):Norm=%g",x0,y0,x1,y1,
                                       std::sqrt((float)((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1))));
          }
          else {
            if (x1>=x0) {	X0 = x0*lx/width;  X1 = (x1+1)*lx/width-1; } else {	X0 = x1*lx/width;  X1 = (x0+1)*lx/width-1; }
            if (y1>=y0) {	Y0 = y0*ly/height; Y1 = (y1+1)*ly/height-1;} else {	Y0 = y1*ly/height; Y1 = (y0+1)*ly/height-1;}
            visu.draw_line(X0,Y0,X1,Y0,color,hatch);
            visu.draw_line(X1,Y0,X1,Y1,color,hatch);
            visu.draw_line(X1,Y1,X0,Y1,color,hatch);
            visu.draw_line(X0,Y1,X0,Y0,color,hatch);
            if (depth==1 && feature_type)
              std::sprintf(text,"Box (%d,%d)-(%d,%d):Size=(%d,%d)",x0<x1?x0:x1,y0<y1?y0:y1,x0>x1?x0:x1,y0>y1?y0:y1,
                           std::abs(x0-x1)+1,std::abs(y0-y1)+1);
          }
          if (depth!=1) {
            if (feature_type==1) {
              X0 = (int)(lx + (z0+0.5)*(disp->width-lx)/depth);
              X1 = (int)(lx + (z1+0.5)*(disp->width-lx)/depth);
              visu.draw_line(X0,Y0,X1,Y1,color,hatch);
              X0 = x0*lx/width; Y0 = ly + z0*(disp->height-ly)/depth;
              X1 = x1*lx/width; Y1 = ly + z1*(disp->height-ly)/depth;
              visu.draw_line(X0,Y0,X1,Y1,color,hatch);
              std::sprintf(text,"Vect (%d,%d,%d)-(%d,%d,%d):Norm=%g",x0,y0,z0,x1,y1,z1,
                           std::sqrt((float)((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1)+(z0-z1)*(z0-z1))));
            }
            else {
              if (z1>=z0) { X0 = lx + z0*(disp->width-lx)/depth;  X1 = lx + (z1+1)*(disp->width-lx)/depth-1; }
              else        { X0 = lx + z1*(disp->width-lx)/depth;  X1 = lx + (z0+1)*(disp->width-lx)/depth-1; }
              visu.draw_line(X0,Y0,X1,Y0,color,hatch);
              visu.draw_line(X1,Y0,X1,Y1,color,hatch);
              visu.draw_line(X1,Y1,X0,Y1,color,hatch);
              visu.draw_line(X0,Y1,X0,Y0,color,hatch);
              if (x1>=x0) {
                X0 = x0*lx/width; X1 = (x1+1)*lx/width-1;
              }
              else {
                X0 = x1*lx/width; X1 = (x0+1)*lx/width-1;
              }
              if (y1>=y0) { Y0 = ly + z0*(disp->height-ly)/depth; Y1 = ly + (z1+1)*(disp->height-ly)/depth-1; }
              else        { Y0 = ly + z1*(disp->height-ly)/depth; Y1 = ly + (z0+1)*(disp->height-ly)/depth-1; }
              visu.draw_line(X0,Y0,X1,Y0,color,hatch);
              visu.draw_line(X1,Y0,X1,Y1,color,hatch);
              visu.draw_line(X1,Y1,X0,Y1,color,hatch);
              visu.draw_line(X0,Y1,X0,Y0,color,hatch);
              if (feature_type) std::sprintf(text,"Box (%d,%d,%d)-(%d,%d,%d):Size=(%d,%d,%d)",x0<x1?x0:x1,y0<y1?y0:y1,z0<z1?z0:z1,
                                             x0<x1?x1:x0,y0<y1?y1:y0,z0<z1?z1:z0,
                                             std::abs(x0-x1)+1,std::abs(y0-y1)+1,std::abs(z0-z1)+1);
            }
          }
        }
        if (xc>=0) {
          if (!text[0]) {
            if (depth!=1) std::sprintf(text,"Coords (%d,%d,%d)={ ",xc,yc,zc); else std::sprintf(text,"Coords (%d,%d)={ ",xc,yc);
            cimg_mapV(*this,k) std::sprintf(text+cimg::strlen(text),"%g ",(double)((*this)(xc,yc,zc,k)));
            std::sprintf(text+cimg::strlen(text),"}");
          }
          if (disp->mousey<10 && texty==0) texty=disp->height-11;
          if (disp->mousey>(int)(texty)-2 && texty!=0) texty=0;
          X0 = cimg::strlen(text)*7;
          cimg_mapV(visu,k)
            for (uint y=texty<0?0:texty; y<(texty+11<(int)visu.height?texty+11:visu.height); y++)
              for (uint x=0; x<(X0<(int)visu.width?X0:visu.width); x++) visu(x,y,0,k)>>=1;
          visu.draw_text(0,texty,text,color);
        }
        disp->display(visu).wait_sync(20);
        chatch+=32;
      } else disp->wait();
    }

    // Return result
    if (XYZ) { XYZ[0] = xp; XYZ[1] = yp; XYZ[2] = zp; }
    if (feature_type==2) {
      if (x0>x1) { const int tmp=x0; x0=x1; x1=tmp; }
      if (y0>y1) { const int tmp=y0; y0=y1; y1=tmp; }
      if (z0>z1) { const int tmp=z0; z0=z1; z1=tmp; }
    }
    if (selection) {
      if (disp->closed) x0=y0=z0=x1=y1=z1=-1;
      switch(feature_type) {
      case 1:
      case 2:  selection[3] = x1; selection[4] = y1; selection[5] = z1;
      default: selection[0] = x0; selection[1] = y0; selection[2] = z0;
      }
    }
    disp->button=0;
    if (!xdisp) delete disp;
			
    return *this;
  }

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Input-Output functions
  //@{
  //------------------------------------------
  //------------------------------------------

  //! Load the image from a file. The used file format is defined by the file extension in the filename \p filename.
  static CImg load(const char *filename) {
    const char *ext = cimg::file_split(filename);
    if (!filename) throw CImgArgumentException("CImg<%s>::load() : Can't load NULL filename",pixel_type());
    if (!cimg::strcasecmp(ext,"inr")) return load_inr(filename);
    if (!cimg::strcasecmp(ext,"asc")) return load_ascii(filename);
    if (!cimg::strcasecmp(ext,"dlm")) return load_dlm(filename);
    if (!cimg::strcasecmp(ext,"hdr")) return load_analyze(filename);
    if (!cimg::strcasecmp(ext,"raw") || ext[0]=='\0') return load_raw(filename);
    if (!cimg::strcasecmp(ext,"ppm") || !cimg::strcasecmp(ext,"pgm") || !cimg::strcasecmp(ext,"pnm")) return load_pnm(filename);
    return load_convert(filename);
  }

  //! Load the image from an INRIMAGE-4 file.
  static CImg load_inr(const char *filename, float *voxsize = NULL) {
    int fopt[8];
    CImg dest;
    std::FILE *file = cimg::fopen(filename,"rb");
    if (voxsize) voxsize[0]=voxsize[1]=voxsize[2]=1;
    cimg::inr_header_read(file,fopt,voxsize);
    dest = CImg<T>(fopt[0],fopt[1],fopt[2],fopt[3]);
    cimg_inr_case(0,0,8, uchar, T) {
      cimg_inr_case(0,1,8, char,  T) {
        cimg_inr_case(0,0,16,ushort,T) {
          cimg_inr_case(0,1,16,short, T) {
            cimg_inr_case(0,0,32,uint,  T) {
              cimg_inr_case(0,1,32,int,   T) {
                cimg_inr_case(1,0,32,float, T) {
                  cimg_inr_case(1,1,32,float, T) {
                    cimg_inr_case(1,0,64,double,T) {
                      cimg_inr_case(1,1,64,double,T) {
                        throw CImgIOException("CImg<%s>::load_inr() : file '%s', can't read images of the type specified in the file",
                                              pixel_type(),filename);
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    cimg::fclose(file);
    return dest;
  }

  //! Load the image from a PNM file
  static CImg load_pnm(const char *filename) {
    uint ppm_type,width,height,colormax=255;
    uchar *raw=NULL,*raw2=NULL;
    char item[256];
    T *rdata,*gdata,*bdata;
    CImg dest;
    std::FILE *file;
    int rval,gval,bval;
    file=cimg::fopen(filename,"rb");
    while (fscanf(file," %1023[^\n]",item)==1 && item[0]=='#'); fgetc(file);
    if(std::sscanf(item,"P%d",&ppm_type)!=1) 
      throw CImgIOException("CImg<%s>::load_pnm() : file '%s',PPM header 'P?' not found",pixel_type(),filename);
    while (fscanf(file," %1023[^\n]",item)==1 && item[0]=='#'); fgetc(file);
    if (std::sscanf(item,"%d %d",&width,&height)!=2)
      throw CImgIOException("CImg<%s>::load_pnm() : file '%s',WIDTH and HEIGHT not defined",pixel_type(),filename);
    while (fscanf(file," %1023[^\n]",item)==1 && item[0]=='#'); fgetc(file);
    cimg::warn(std::sscanf(item,"%d",&colormax)!=1,"CImg<%s>::load_pnm() : file '%s',COLORMAX not defined",pixel_type(),filename);
    cimg::warn(colormax!=255,"CImg<%s>::load_pnm() : file '%s', COLORMAX=%d mode is not supported",pixel_type(),filename,colormax);
    switch (ppm_type) {
    case 2:                 /* Grey Ascii */
      {
        dest = CImg<T>(width,height,1,1);
        rdata= dest.ptr();
        for (uint xy=0; xy<width*height; xy++) { fscanf(file,"%d",&rval); *(rdata++)=(uchar)rval; }
      }
      break;
    case 3:               /* Color Ascii */
      {
        dest = CImg<T>(width,height,1,3);
        rdata=dest.ptr(0,0,0,0); gdata=dest.ptr(0,0,0,1); bdata=dest.ptr(0,0,0,2);
        for (uint xy=0; xy<width*height; xy++) { fscanf(file,"%d %d %d",&rval,&gval,&bval); *(rdata++)=(uchar)rval; *(gdata++)=(uchar)gval; *(bdata++)=(uchar)bval; }
      }
      break;
    case 5:               /* Grey Binary */
      {
        raw2 = raw = new uchar[width*height];
        cimg::fread(raw,sizeof(uchar),width*height,file);
        dest = CImg<T>(width,height,1,1);
        rdata = dest.ptr(); for (uint xy=0; xy<dest.size(); xy++) *(rdata++)=(T)*(raw2++);
        delete[] raw;
      }
      break;
    case 6:               /* Color Binary */
      {
        raw2 = raw = new uchar[width*height*3];
        cimg::fread(raw,sizeof(uchar),width*height*3,file);
        dest = CImg<T>(width,height,1,3);
        rdata=dest.ptr(0,0,0,0); gdata=dest.ptr(0,0,0,1); bdata=dest.ptr(0,0,0,2);
        for (uint xy=0; xy<width*height; xy++) { *(rdata++)=(T)*(raw2++); *(gdata++)=(T)*(raw2++); *(bdata++)=(T)*(raw2++); }
        delete[] raw;
      }
      break;
    default:
      {
        cimg::fclose(file);
        throw CImgIOException("CImg<%s>::load_pnm() : file '%s', PPM type 'P%d' not supported",pixel_type(),filename,ppm_type);
      }
    }
    cimg::fclose(file);
    return dest;
  }

  //! Load the image from an ASCII file
  static CImg load_ascii(const char *filename) {
    int err=1,xy,w,h,z,k;
    CImg dest;
    std::FILE *file;
    double v;
    file=cimg::fopen(filename,"r");
    fscanf(file,"%d %d %d %d\n",&w,&h,&z,&k);
    dest=CImg<T>(w,h,z,k);
    for (xy=0; xy<w*h*z*k && err==1; xy++) { err=fscanf(file,"%lf ",&v); dest.data[xy]=(T)v; }
    cimg::warn(xy<w*h*z*k,"CImg<%s>::load_ascii() : file '%s', %d values read, instead of %d",pixel_type(),filename,xy,w*h*z*k);
    cimg::fclose(file);
    return dest;
  }

  //! Load the image from a DLM file
  static CImg load_dlm(const char *filename) {
    std::FILE *file = cimg::fopen(filename,"r");
    uint cdimx,dimx=0,dimy=0;
    double tmpf;
    char c;
    for (int err=0; err!=EOF; ) {
      cdimx=0;
      do { if ((err=fscanf(file,"%lg%c",&tmpf,&c))>0) cdimx++; }
      while (err==2 && c!='\n');
      if (cdimx) {
        if (!dimx) dimx=cdimx;
        if (cdimx!=dimx) throw CImgIOException("CImg<%s>::load_dlm() : File '%s', Line %d contains %d elements, instead of %d",
                                               pixel_type(),filename,dimy+1,cdimx,dimx);
        dimy++;
      } else err=EOF;
    }
    rewind(file);
    CImg<T> dest(dimx,dimy);
    T *ptr_data = dest.data;
    cimg_mapoff(dest,off) if (fscanf(file,"%lg",&tmpf)) *(ptr_data++) = (T)tmpf;  else { fgetc(file); off--; }

    cimg::fclose(file);
    return dest;
  }

  //! Load the image from an ANALYZE7.5 file
  static CImg load_analyze(const char *filename, float *voxsize = NULL) {
    CImg dest;
    float scalefactor;
    char tmp[1024];
    const char *ext;
    int header_size,endian=0;
    short dimx=1,dimy,dimz,dimv,*dim,bitpix,datatype;
    uchar *header;
    std::FILE *file_header=NULL,*file=NULL;
    ext = cimg::file_split(filename);
    if (!cimg::strncasecmp(ext,"hdr",3)) {
       file_header = cimg::fopen(filename,"rb");
       std::strcpy(tmp,filename);
       std::sprintf(tmp+cimg::strlen(tmp)-3,"img");
       file = cimg::fopen(tmp,"rb");
    }
    else {
      if (!cimg::strncasecmp(ext,"img",3)) {
        file = cimg::fopen(filename,"rb");
        std::strcpy(tmp,filename);
        std::sprintf(tmp+cimg::strlen(tmp)-3,"hdr");
        file_header = cimg::fopen(tmp,"rb");
      } else throw CImgIOException("CImg<%s>::load_analyze() : Cannot load filename '%s' as an analyze format",pixel_type(),filename);
    }
    cimg::fread(&header_size,sizeof(int),1,file_header);
    if ((uint)header_size>=4096) { endian = 1; cimg::bswap(&header_size,1,sizeof(int)); }
    header = new uchar[header_size];
    cimg::fread(header+sizeof(int),1,header_size-sizeof(int),file_header);
    cimg::fclose(file_header);
    dim = (short*)(header+40);
    if (endian) { cimg::bswap(dim,5,sizeof(short)); }
    if (dim[0]<=0) throw CImgIOException("CImg<%s>::load_analyze() : Specified image dimension is %d\n",pixel_type(),dim[0]);
    cimg::warn(dim[0]>4,"CImg<%s>::load_analyze() : Image dimension is %d, reading only 4 first dimensions",pixel_type(),dim[0]);
    if (dim[0]>=1) dimx = dim[1];
    if (dim[0]>=1) dimy = dim[2]; else dimy = 1;
    if (dim[0]>=2) dimz = dim[3]; else dimz = 1;
    if (dim[0]>=3) dimv = dim[4]; else dimv = 1;
    if (endian) {
      cimg::bswap(header+112,1,sizeof(float));
      cimg::bswap(header+70,1,sizeof(short)); 
      cimg::bswap(header+72,1,sizeof(short)); 
      cimg::bswap(header+76,4,sizeof(float));
    }
    scalefactor = *(float*)(header+112); if (scalefactor==0) scalefactor=1;
    datatype = *(short*)(header+70); bitpix = *(short*)(header+72);
    if (voxsize) { float *vsize = (float*)(header+76); voxsize[0] = vsize[1];	voxsize[1] = vsize[2]; voxsize[2] = vsize[3]; }
    delete[] header;
    dest = CImg<T>(dimx,dimy,dimz,dimv);
    switch (datatype) {
    case 2:
      {
        uchar *buffer = new uchar[dimx*dimy*dimz*dimv];
        cimg::fread(buffer,sizeof(uchar),dimx*dimy*dimz*dimv,file);
        cimg_mapoff(dest,off) dest.data[off] = (T)(buffer[off]*scalefactor);
        delete[] buffer; break;
      }
    case 4:
      {
        short *buffer = new short[dimx*dimy*dimz*dimv];
        cimg::fread(buffer,sizeof(short),dimx*dimy*dimz*dimv,file);
        if (endian) cimg::bswap(buffer,dimx*dimy*dimz*dimv,sizeof(short));
        cimg_mapoff(dest,off) dest.data[off] = (T)(buffer[off]*scalefactor);
        delete[] buffer; break;
      }
    case 8:
      {
        int *buffer = new int[dimx*dimy*dimz*dimv];
        cimg::fread(buffer,sizeof(int),dimx*dimy*dimz*dimv,file);
        if (endian) cimg::bswap(buffer,dimx*dimy*dimz*dimv,sizeof(int));
        cimg_mapoff(dest,off) dest.data[off] = (T)(buffer[off]*scalefactor);
        delete[] buffer; break;
      }
    case 16:
      {
        float *buffer = new float[dimx*dimy*dimz*dimv];
        cimg::fread(buffer,sizeof(float),dimx*dimy*dimz*dimv,file);
        if (endian) cimg::bswap(buffer,dimx*dimy*dimz*dimv,sizeof(float));
        cimg_mapoff(dest,off) dest.data[off] = (T)(buffer[off]*scalefactor);
        delete[] buffer; break;
      }
    case 64:
      {
        double *buffer = new double[dimx*dimy*dimz*dimv];
        cimg::fread(buffer,sizeof(double),dimx*dimy*dimz*dimv,file);
        if (endian) cimg::bswap(buffer,dimx*dimy*dimz*dimv,sizeof(double));
        cimg_mapoff(dest,off) dest.data[off] = (T)(buffer[off]*scalefactor);
        delete[] buffer; break;
      }
    default: throw CImgIOException("CImg<%s>::load_analyze() : Cannot read images width 'datatype = %d'",pixel_type(),datatype);
    }
    cimg::fclose(file);
    return dest;
  }

  //! Load the image from a RAW file
  static CImg load_raw(const char *filename) {
    std::FILE *file;
    char tmp[256],tmp2[256];
    int i;
    uint j,n,w,h,z,k,err;
    file=cimg::fopen(filename,"rb");
    j=0; while((i=fgetc(file))!='\n' && i!=EOF) tmp[j++]=(char)i; tmp[j]='\0'; err=std::sscanf(tmp,"%d#%[A-Za-z]",&n,tmp2);
    if (err!=2) throw CImgIOException("CImg<%s>::load_raw() : file '%s', Unknow .raw header",pixel_type(),filename);
    cimg::warn(n!=1, "CImg<%s>::load_raw() : file '%s' contains %d images, only the first will be loaded",pixel_type(),filename,n);
    CImg<T> img, *res = &img;
    cimg_raw_case(uchar) {
      cimg_raw_case(char) {
        cimg_raw_case(ushort) {
          cimg_raw_case(short) {
            cimg_raw_case(uint) {
              cimg_raw_case(int) {
                cimg_raw_case(float) {
                  cimg_raw_case(double) {
                    throw CImgIOException("CImg<%s>::load_raw() : file '%s', can't read images of %s",pixel_type(),filename,tmp2);
                  }
                }
              }
            }
          }
        }
      }
    }
    cimg::fclose(file);
    return img;
  }

  //! Function that loads the image for other file formats that are not natively handled by CImg, using the tool 'convert' from the ImageMagick package.\n
  //! This is the case for all compressed image formats (GIF,PNG,JPG,TIF,...). You need to install the ImageMagick package in order to get
  //! this function working properly (see http://www.imagemagick.org ).
  static CImg load_convert(const char *filename) {
    srand((uint)time(NULL));
    char command[512], filetmp[512];
    std::sprintf(filetmp,"%s/CImg%.4d.ppm",cimg::temporary_path(),rand()%10000);
    std::sprintf(command,"%s %s %s",cimg::convert_path(),filename,filetmp);
    system(command);
    std::FILE *file = std::fopen(filetmp,"rb");
    if (!file) {
      std::fclose(cimg::fopen(filename,"r"));
      throw CImgIOException("CImg<%s>::load_convert() : Failed to open image '%s' with 'convert'.\n\
(1) Check that you have installed the ImageMagick package.\n\
(2) If true, you have to set the macro 'cimg_convert_path' to the correct path of the ImageMagick's convert executable :\n\
#define cimg_convert_path \"path/convert.exe\" (before including 'CImg.h')",pixel_type(),filename);
    } else cimg::fclose(file);
    const CImg dest(filetmp);
    std::remove(filetmp);
    return dest;
  }

  //! Save the image as a file. The used file format is defined by the file extension in the filename \p filename.\n
  //! Parameter \p number can be used to add a 6-digit number to the filename before saving.\n
  //! If \p normalize is true, a normalized version of the image (between [0,255]) is saved.
  const CImg& save(const char *filename,const int number=-1,const bool normalize=true) const {
    cimg_test(*this);
    const char *ext = cimg::file_split(filename);
    char nfilename[1024];
    if (number>=0) filename = cimg::file_number(filename,number,6,nfilename);
    if (!cimg::strcasecmp(ext,"inr")) return save_inr(filename);
    if (!cimg::strcasecmp(ext,"asc")) return save_ascii(filename);
    if (!cimg::strcasecmp(ext,"dlm")) return save_dlm(filename);
    if (!cimg::strcasecmp(ext,"hdr")) return save_analyze(filename);
    if (!cimg::strcasecmp(ext,"raw") || ext[0]=='\0') return save_raw(filename);
    if (!cimg::strcasecmp(ext,"pgm") || !cimg::strcasecmp(ext,"ppm") || !cimg::strcasecmp(ext,"pnm")) 
      return normalize?get_normalize((T)0,(T)255).save_pnm(filename):save_pnm(filename);
    return normalize?get_normalize((T)0,(T)255).save_convert(filename):save_convert(filename);
  }
  
  //! Save the image as an ASCII file.
  const CImg& save_ascii(const char *filename) const {
    cimg_test(*this);
    std::FILE *file = cimg::fopen(filename,"w");
    std::fprintf(file,"%d %d %d %d\n",width,height,depth,dim);
    for (uint xy=0; xy<size(); xy++)  std::fprintf(file,"%g ",(double)data[xy]);
    cimg::fclose(file);
    return *this;
  }

  //! Save the image as a DLM file.
  const CImg& save_dlm(const char *filename) const {
    cimg_test(*this);
    std::FILE *file;
    uint x;
    file=cimg::fopen(filename,"w");
    cimg_mapY(*this,y) {
      for (x=0; x<width-1; x++) std::fprintf(file,"%g,",(double)(*this)(x,y));
      std::fprintf(file,"%g\n",(double)(*this)(x,y));
    }
    cimg::fclose(file);
    return *this;
  }

  //! Save the image as a PNM file.
  const CImg& save_pnm(const char *filename) const {
    cimg_test(*this);
    const char *ext = cimg::file_split(filename);
    uchar *xptr,*xdata;
    T *r,*g,*b;
    std::FILE *file;
    if (depth!=1)  return get_3dplanes(width/2,height/2,depth/2).save_pnm(filename);
    switch(dim) {
    case 1:
      {
        if (!cimg::strcasecmp(ext,"ppm")) { get_resize(width,height,1,3,2).save_pnm(filename); return *this; }
        xptr = xdata = new uchar[size()];
        r = data;
        file = cimg::fopen(filename,"wb");
        std::fprintf(file,"P5\n# CREATOR: CImg : Original size=%dx%dx%dx%d\n%d %d\n255\n",width,height,depth,dim,width,height);
        for (uint xy = 0; xy<size(); xy++) *(xptr++) = (uchar)(*(r++));
        cimg::fwrite(xdata,sizeof(uchar),size(),file);
        cimg::fclose(file);
        delete[] xdata;
      }
      break;
    case 2:
      {
        get_resize(width,height,1,3,0).save_pnm(filename);
        return *this;
      }
    default:
      {
        if (!cimg::strcasecmp(ext,"pgm")) { get_norm_pointwise(2).normalize(0,255).save_pnm(filename); return *this; }
        xptr = xdata = new uchar[size()];
        r = data; g = ptr(0,0,0,1); b = ptr(0,0,0,2);
        file = cimg::fopen(filename,"wb");
        std::fprintf(file,"P6\n# CREATOR: CImg : Original size=%dx%dx%dx%d\n%d %d\n255\n",width,height,depth,dim,width,height);
        for (uint xy = 0; xy<size(); xy+=3) { *(xptr++) = (uchar)(*(r++)); *(xptr++) = (uchar)(*(g++)); *(xptr++) = (uchar)(*(b++)); }
        cimg::fwrite(xdata,sizeof(uchar),size(),file);
        cimg::fclose(file);
        delete[] xdata;
      }
      break;
    }
    return *this;
  }

  //! Save the image as an ANALYZE7.5 file.
  const CImg& save_analyze(const char *filename,const float *const voxsize=NULL) const {
    cimg_test(*this);
    std::FILE *file;
    char header[348],hname[1024],iname[1024];
    const char *ext = cimg::file_split(filename);
    short datatype=-1;
    std::memset(header,0,348);
    if (!ext[0]) { std::sprintf(hname,"%s.hdr",filename); std::sprintf(iname,"%s.img",filename); }
    if (!cimg::strncasecmp(ext,"hdr",3)) { std::strcpy(hname,filename); std::strcpy(iname,filename); std::sprintf(iname+cimg::strlen(iname)-3,"img"); }
    if (!cimg::strncasecmp(ext,"img",3)) { std::strcpy(hname,filename); std::strcpy(iname,filename); std::sprintf(hname+cimg::strlen(iname)-3,"hdr"); }
    ((int*)(header))[0] = 348;
    std::sprintf(header+4,"CImg");
    std::sprintf(header+14," ");
    ((short*)(header+36))[0] = 4096;
    ((char*)(header+38))[0] = 114;
    ((short*)(header+40))[0] = 4;
    ((short*)(header+40))[1] = width;
    ((short*)(header+40))[2] = height;
    ((short*)(header+40))[3] = depth;
    ((short*)(header+40))[4] = dim;
    if (!cimg::strcasecmp(pixel_type(),"uchar"))  datatype = 2;
    if (!cimg::strcasecmp(pixel_type(),"short"))  datatype = 4;
    if (!cimg::strcasecmp(pixel_type(),"int"))    datatype = 8;
    if (!cimg::strcasecmp(pixel_type(),"float"))  datatype = 16;
    if (!cimg::strcasecmp(pixel_type(),"double")) datatype = 64;
    ((short*)(header+70))[0] = datatype;
    ((short*)(header+72))[0] = sizeof(T);
    ((float*)(header+112))[0] = 1;
    ((float*)(header+76))[0] = 0;
    if (voxsize) {
      ((float*)(header+76))[1] = voxsize[0];
      ((float*)(header+76))[2] = voxsize[1];
      ((float*)(header+76))[3] = voxsize[2];
    } else ((float*)(header+76))[1] = ((float*)(header+76))[2] = ((float*)(header+76))[3] = 1;
    file = cimg::fopen(hname,"wb");
    cimg::fwrite(header,sizeof(char),348,file);
    cimg::fclose(file);
    file = cimg::fopen(iname,"wb");
    cimg::fwrite(data,sizeof(T),size(),file);
    cimg::fclose(file);
    return *this;
  }

  //! Save the image as a RAW file
  const CImg& save_raw(const char *filename) const {
    cimg_test(*this);
    std::FILE *file;
    file=cimg::fopen(filename,"wb");
    std::fprintf(file,"1#%s\n%d %d %d %d\n",pixel_type(),width,height,depth,dim);
    if (!cimg::CPU_endian()) {
      CImg<T> tmp(*this);
      cimg::bswap(tmp.data,tmp.size(),sizeof(T));
      cimg::fwrite(tmp.data,sizeof(T),width*height*depth*dim,file);
    }
    else {
      cimg::fwrite(data,sizeof(T),width*height*depth*dim,file);
    }
    cimg::fclose(file);
    return *this;
  }

  //! Function that saves the image for other file formats that are not natively handled by CImg, using the tool 'convert' from the ImageMagick package.\n
  //! This is the case for all compressed image formats (GIF,PNG,JPG,TIF,...). You need to install the ImageMagick package in order to get
  //! this function working properly (see http://www.imagemagick.org ).
  const CImg& save_convert(const char *filename) const {
    cimg_test(*this);
    srand((unsigned int)time(NULL));
    char command[512],filetmp[512];
    std::sprintf(filetmp,"%s/CImg%.4d.ppm",cimg::temporary_path(),rand()%10000);
    save_pnm(filetmp);
    std::sprintf(command,"%s -quality 100%% %s %s",cimg::convert_path(),filetmp,filename);
    system(command);
    std::FILE *file = std::fopen(filename,"rb");
    if (!file) throw CImgIOException("CImg<%s>::save_convert() : Failed to save image '%s' with 'convert'.\n\
(1) Check that you have installed the ImageMagick package.\n\
(2) If true, you have to set the macro 'cimg_convert_path' to the correct path of the ImageMagick's convert executable :\n\
 #define cimg_convert_path \"path/convert.exe\" (before including 'CImg.h')",pixel_type(),filename);
    if (file) cimg::fclose(file);
    std::remove(filetmp);
    return *this;
  }
  
  //! Save the image as an INRIMAGE-4 file.
  const CImg& save_inr(const char *filename,const float *const voxsize = NULL) const {
    cimg_test(*this);
    std::FILE *file;
    int inrpixsize=-1,err;
    char header[257];
    const char *inrtype = "unsigned fixed\nPIXSIZE=8 bits\nSCALE=2**0";
    if (!cimg::strcasecmp(pixel_type(),"uchar")) { inrtype = "unsigned fixed\nPIXSIZE=8 bits\nSCALE=2**0"; inrpixsize = 1; }
    else { if (!cimg::strcasecmp(pixel_type(),"char")) {	inrtype = "fixed\nPIXSIZE=8 bits\nSCALE=2**0"; inrpixsize = 1; }
      else { if (!cimg::strcasecmp(pixel_type(),"ushort")) {	inrtype = "unsigned fixed\nPIXSIZE=16 bits\nSCALE=2**0";inrpixsize = 2; }
        else { if (!cimg::strcasecmp(pixel_type(),"short")) { inrtype = "fixed\nPIXSIZE=16 bits\nSCALE=2**0"; inrpixsize = 2; }
          else { if (!cimg::strcasecmp(pixel_type(),"uint")) { inrtype = "unsigned fixed\nPIXSIZE=32 bits\nSCALE=2**0";inrpixsize = 4; }
            else { if (!cimg::strcasecmp(pixel_type(),"int")) { inrtype = "fixed\nPIXSIZE=32 bits\nSCALE=2**0"; inrpixsize = 4; }
              else { if (!cimg::strcasecmp(pixel_type(),"float")) { inrtype = "float\nPIXSIZE=32 bits"; inrpixsize = 4; }
                else { if (!cimg::strcasecmp(pixel_type(),"double")) { inrtype = "float\nPIXSIZE=64 bits"; inrpixsize = 8; }
                }
              }
            }
          }
        }
      }
    }
    if (inrpixsize<=0) throw CImgIOException("CImg<%s>::save_inr() : Don't know how to save images of <%s>",pixel_type(),pixel_type());
    file = cimg::fopen(filename,"wb");
    err = std::sprintf(header,"#INRIMAGE-4#{\nXDIM=%d\nYDIM=%d\nZDIM=%d\nVDIM=%d\n",width,height,depth,dim);
    if (voxsize) err += std::sprintf(header+err,"VX=%g\nVY=%g\nVZ=%g\n",voxsize[0],voxsize[1],voxsize[2]);
    err += std::sprintf(header+err,"TYPE=%s\nCPU=%s\n",inrtype,cimg::CPU_endian()?"decm":"sun");
    std::memset(header+err,'\n',252-err);
    std::memcpy(header+252,"##}\n",4);
    cimg::fwrite(header,sizeof(char),256,file);
    cimg_mapXYZ(*this,x,y,z) cimg_mapV(*this,k) cimg::fwrite(&((*this)(x,y,z,k)),inrpixsize,1,file);
    cimg::fclose(file);
    return *this;
  }

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Other functions
  //@{
  //------------------------------------------
  //------------------------------------------
  CImg& swap(CImg& img) {
    int tmp = img.width; img.width = width;   width = tmp;
    tmp = img.height;    img.height = height; height = tmp;
    tmp = img.depth;     img.depth = depth;   depth = tmp;
    tmp = img.dim;       img.dim = dim;       dim = tmp;
    T* const ptr = img.data; img.data = data; data = ptr;
    return img;
  }

  //@}
};


/*-------------------------------------------------------




  Definition of the CImgl<> structure




  ------------------------------------------------------*/
//! This class represents list of images CImg<T>
template<typename T> struct CImgl {
  uint size;                  //!< Number of images in the list.
  CImg<T> *data;              //!< Pointer to the first image of the image list.
  
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Constructors - Destructor - Copy
  //@{
  //------------------------------------------
  //------------------------------------------

  //! Return a string describing the type of the image pixels in the list (template parameter \p T).
  static const char* pixel_type() { T val; return cimg::template_type(val); }

  //! Create a list of \p n new images, each having size (\p width,\p height,\p depth,\p dim).
  CImgl(const uint n=0,const uint width=0,const uint height=1,const uint depth=1, const uint dim=1):size(n),data(NULL) {
    if (n>0) {
      data = new CImg<T>[(n/cimg::lblock+1)*cimg::lblock];
      cimgl_map(*this,l) {
        CImg<T> img(width,height,depth,dim);
        img.swap(data[l]);
      }
    }
  }
  //! Create a list of \p n new images, each having size (\p width,\p height,\p depth,\p dim) and filled with the value \p val
  CImgl(const uint n,const uint width,const uint height,const uint depth, const uint dim, const T& val):size(n),data(NULL) {
    if (n>0) {
      data = new CImg<T>[(n/cimg::lblock+1)*cimg::lblock];
      cimgl_map(*this,l) {
        CImg<T> img(width,height,depth,dim,val);
        img.swap(data[l]);
      }
    }
  }
  //! Copy constructor.
  template<typename t> CImgl(const CImgl<t>& list):size(list.size),data(NULL) {
    if (size>0) {
      data = new CImg<T>[(size/cimg::lblock+1)*cimg::lblock];
      cimgl_map(*this,l) data[l] = list[l];
    }
  }
  //! Copy constructor.
  CImgl(const CImgl<T>& list):size(list.size),data(NULL) {
    if (size>0) {
      data = new CImg<T>[(size/cimg::lblock+1)*cimg::lblock];
      cimgl_map(*this,l) data[l] = list[l];
    }
  }
  //! Create a list from a single image \p img (image is copied).
  CImgl(const CImg<T>& img):size(1) {
    data = new CImg<T>[cimg::lblock];
    data[0] = img;
  }
  //! Create a list of \p n identical images \p img (images are copied).
  CImgl(const uint n, const CImg<T>& img):size(n>0?n:0) {
    if (n==0) data = NULL;
    else {
      data = new CImg<T>[(size/cimg::lblock+1)*cimg::lblock];
      cimgl_map(*this,l) data[l] = img;
    }
  }
  //! Create a list by loading a file.
  CImgl(const char* filename):size(0),data(NULL) { load(filename).swap(*this); }
  //! Create a list from two images \p img1 and \p img2 (images are copied).
  CImgl(const CImg<T>& img1,const CImg<T>& img2):size(2) {
    data = new CImg<T>[cimg::lblock];
    data[0] = img1;
    data[1] = img2;
  }
  //! Create a list from three images \p img1,\p img2 and \p img3 (images are copied).
  CImgl(const CImg<T>& img1,const CImg<T>& img2,const CImg<T>& img3):size(3) {
    data = new CImg<T>[cimg::lblock];
    data[0] = img1;
    data[1] = img2;
    data[2] = img3;
  }
  //! Create a list from four images \p img1,\p img2,\p img3 and \p img4 (images are copied).
  CImgl(const CImg<T>& img1,const CImg<T>& img2,const CImg<T>& img3,const CImg<T>& img4):size(4) {
    data = new CImg<T>[cimg::lblock];
    data[0] = img1;
    data[1] = img2;
    data[2] = img3;
    data[3] = img4;
  }
  //! Copy a list into another one.
  template<typename t> CImgl& operator=(const CImgl<t>& list) {
    const uint s = list.size;
    if (s) {
      if (s!=size) { if (data) delete[] data; data = new CImg<T>[(s/cimg::lblock+1)*cimg::lblock]; }
      size = s;
      cimgl_map(*this,l) data[l] = list[l];
    } else {
      if (data) delete[] data;
      data = NULL;
      size = 0;
    }
    return *this;
  }
  //! Copy a list into another one (optimized for same pixel type \p T)
  CImgl& operator=(const CImgl<T>& list) {
    if (&list==this) return *this;
    const uint s = list.size;
    if (s) {
      if (s!=size) { if (data) delete[] data; data = new CImg<T>[(s/cimg::lblock+1)*cimg::lblock]; }
      size = s;
      cimgl_map(*this,l) data[l] = list[l];
    } else {
      if (data) delete[] data;
      data = NULL;
      size = 0;
    }
    return *this;
  }
  //! Destructor
  ~CImgl() { if (data) delete[] data; }
  
  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name Arithmetics operators
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Add each image of the current list with the corresponding image in the list \p list.
  template<typename t> CImgl& operator+=(const CImgl<t>& list) {
    uint sizemax = cimg::min(size,list.size);
    for (uint l=0; l<sizemax; l++) (*this)[l]+=list[l];
    return *this;
  }
  //! Subtract each image of the current list with the corresponding image in the list \p list.
  template<typename t> CImgl& operator-=(const CImgl<t>& list) {
    uint sizemax = cimg::min(size,list.size);
    for (uint l=0; l<sizemax; l++) (*this)[l]-=list[l];
    return *this;
  }
  //! Add each image of the current list with a value \p val.
  CImgl& operator+=(const T& val) { cimgl_map(*this,l) (*this)[l]+=val; return *this; }
  //! Substract each image of the current list with a value \p val.
  CImgl& operator-=(const T& val) { cimgl_map(*this,l) (*this)[l]-=val; return *this; }
  //! Multiply each image of the current list by a value \p val.
  CImgl& operator*=(const double val) { cimgl_map(*this,l) (*this)[l]*=val; return *this; }
  //! Divide each image of the current list by a value \p val.
  CImgl& operator/=(const double val) { cimgl_map(*this,l) (*this)[l]/=val; return *this; }
  //! Return a new image list corresponding to the addition of each image of the current list with a value \p val.
  CImgl operator+(const T& val) const      { return CImgl<T>(*this)+=val;  }
  //! Return a new image list corresponding to the multiplication of each image of the current list by a value \p val.
  CImgl operator*(const double val) const  { return CImgl<T>(*this)*=val;  }
  //! Return a new image list corresponding to the substraction of each image of the current list with a value \p val.
  CImgl operator-(const T& val) const      { return CImgl<T>(*this)-=val;  }
  //! Return a new image list corresponding to the division of each image of the current list by a value \p val.
  CImgl operator/(const double val) const  { return CImgl<T>(*this)/=val;  }
  //! Return a new image list corresponding to the addition of each image of the current list with the corresponding image in the list \p list.
  CImgl operator+(const CImgl& list) const { return CImgl<T>(*this)+=list; }
  //! Return a new image list corresponding to the substraction of each image of the current list with the corresponding image in the list \p list.
  CImgl operator-(const CImgl& list) const { return CImgl<T>(*this)-=list; }
  //! Return a new image list corresponding to the addition of each image of the current list with a value \p val;
  friend CImgl operator+(const T& val, const CImgl& list)     { return CImgl<T>(list)+=val; }
  //! Return a new image list corresponding to the scalar multiplication of each image of the current list by a value \p val.
  friend CImgl operator*(const double val, const CImgl& list) { return CImgl<T>(list)*=val; }
  
  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name List operations
  //@{
  //------------------------------------------
  //------------------------------------------

  //! Return a reference to the i-th element of the image list.
  CImg<T>& operator[](const uint pos) const {
#if cimg_debug>1
    if (pos>=size) {
      cimg::warn(true,"CImgl<%s>::operator[] : bad list position %d, in a list of %d images",pixel_type(),pos,size);
      return *data;
    }
#endif
    return data[pos];
  }
  //! Equivalent to CImgl<T>::operator[]
  CImg<T>& operator()(const uint pos) const {
    return (*this)[pos];
  }
  
  //! Insert a copy of the image \p img into the current image list, at position \p pos.
  CImgl& insert(const CImg<T>& img,const uint pos) {
    if (pos>size) throw CImgArgumentException("CImgl<%s>::insert() : Can't insert at position %d into a list with %d elements",pixel_type(),pos,size);
    CImg<T> *new_data = (!((++size)%cimg::lblock) || !data)?new CImg<T>[(size/cimg::lblock+1)*cimg::lblock]:NULL;
    if (!data) { data=new_data; *data=img; }
    else {
      if (new_data) {
        std::memcpy(new_data,data,sizeof(CImg<T>)*pos);
        if (pos!=size-1) std::memcpy(new_data+pos+1,data+pos,sizeof(CImg<T>)*(size-1-pos));
        std::memset(data,0,sizeof(CImg<T>)*(size-1));
        delete[] data;
        data = new_data;
      }
      else if (pos!=size-1) memmove(data+pos+1,data+pos,sizeof(CImg<T>)*(size-1-pos));
      data[pos].data = NULL;
      data[pos] = img;
    }
    return *this;
  }
  //! Append a copy of the image \p img at the current image list.
  CImgl& insert(const CImg<T>& img) { return insert(img,size); }
  //! Insert a copy of the image list \p list into the current image list, starting from position \p pos.
  CImgl& insert(const CImgl<T>& list,const uint pos) { cimgl_map(list,l) insert(list[l],pos+l); return *this; }
  //! Append a copy of the image list \p list at the current image list.
  CImgl& insert(const CImgl<T>& list) { return insert(list,size); }
  
  //! Remove the image at position \p pos from the image list.
  CImgl& remove(const uint pos) {
    if (pos>=size) { 
      cimg::warn(true,"CImgl<%s>::remove() : Can't remove an image from a list (%p,%d), at position %d",pixel_type(),data,size,pos);
      return *this;
    }
    CImg<T> tmp; tmp.swap(data[pos]); // the image to remove will be freed
    size--;
    if (pos!=size) { 
      memmove(data+pos,data+pos+1,sizeof(CImg<T>)*(size-pos));
      CImg<T> &tmp = data[size];
      tmp.width = tmp.height = tmp.depth = tmp.dim = 0;
      tmp.data = NULL;
    }
    return *this;
  }
  //! Remove the last image from the image list.
  CImgl& remove() { return remove(size); }  

  //@}
  //------------------------------------------
  //------------------------------------------
  //
  //! \name IO and display functions
  //@{
  //------------------------------------------
  //------------------------------------------
  
  //! Print informations about the list on the standart error stream.
  const CImgl& print(const char* title=NULL,const int print_flag=1) const { 
    char tmp[1024];
    std::fprintf(stderr,"%-8s(%p) : (%d,%p)\n",title?title:"CImgl",this,size,data);
    if (print_flag>0)	cimgl_map(*this,l) {
      std::sprintf(tmp,"%s[%d]",title?title:"CImgl",l);
      data[l].print(tmp,print_flag);
    }
    return *this;
  }
  //! Load an image list from a file (.raw format).
  static CImgl load_raw(const char *filename) {
    std::FILE *file;
    char tmp[256],tmp2[256];
    int i;
    uint n,j,w,h,z,k,err;
    file=cimg::fopen(filename,"rb");
    j=0; while((i=fgetc(file))!='\n' && i!=EOF) tmp[j++]=i; tmp[j]='\0'; err=std::sscanf(tmp,"%d#%[A-Za-z]",&n,tmp2);
    if (err!=2) throw CImgIOException("CImgl<%s>::load_raw() : file '%s', Unknow .raw header",pixel_type(),filename);
    CImgl<T> res(n);
    cimg_raw_case(uchar) {
      cimg_raw_case(char) {
        cimg_raw_case(ushort) {
          cimg_raw_case(short) {
            cimg_raw_case(uint) {
              cimg_raw_case(int) {
                cimg_raw_case(float) {
                  cimg_raw_case(double) {
                    throw CImgIOException("CImgl<%s>::load_raw() : file '%s', can't read images of %s",pixel_type(),filename,tmp2);
                  }
                }
              }
            }
          }
        }
      }
    }
    cimg::fclose(file);
    return res;
  }
  //! Load an image list from a file. The file should be a '.raw' format, else only one image will be loaded into the list.
  static CImgl load(const char *filename) {
    CImgl res;
    const char *ext = cimg::file_split(filename);
    if (!cimg::strcasecmp(ext,"raw") || !ext[0]) return load_raw(filename); else return CImg<T>(filename);
  }
  //! Save an image list into a file (.raw format).
  const CImgl& save_raw(const char *filename) const {
    cimgl_test(*this);
    CImg<T> img,tmp;
    std::FILE *file;
    file=cimg::fopen(filename,"wb");
    std::fprintf(file,"%d#%s\n",size,data[0].pixel_type());
    cimgl_map(*this,l) {
      img = data[l];
      std::fprintf(file,"%d %d %d %d\n",img.width,img.height,img.depth,img.dim);
      if (!cimg::CPU_endian()) {
        tmp = img;
        cimg::bswap(tmp.data,tmp.size(),sizeof(T));
        cimg::fwrite(tmp.data,sizeof(T),img.width*img.height*img.depth*img.dim,file);
      }
      else {
        cimg::fwrite(img.data,sizeof(T),img.width*img.height*img.depth*img.dim,file);
      }
    }
    cimg::fclose(file);
    return *this;
  }

  //! Save an image list into a file. The file format will be dynamically determined by the filename extension.
  //! Use the '.raw' format to save the list into a single file, else images are numbered and saved independently.
  const CImgl& save(const char *filename,const bool normalize=true) const {
    cimgl_test(*this);
    const char *ext = cimg::file_split(filename);
    if (!cimg::strcasecmp(ext,"raw") || !ext[0]) return save_raw(filename);
    else {
      if (size==1) data[0].save(filename,-1,normalize);
      else cimgl_map(*this,l) data[l].save(filename,l,normalize);
    }
    return *this;
  }
  //! Append images of a list into a single image (which is returned), by concatenating them along the specified axe \p axe,
  // centering them using the position \position.
  CImg<T> get_append(const char axe='x',const char position='c') const {
    cimgl_test(*this);
    uint dx=0,dy=0,dz=0,dv=0,pos=0;
    CImg<T> res;
    switch(axe) {
    case 'x':
      {
        cimgl_map(*this,l) {
          const CImg<T>& img = (*this)[l];
          dx += img.width;
          dy = cimg::max(dy,img.height);
          dz = cimg::max(dz,img.depth);
          dv = cimg::max(dv,img.dim);
        }
        res = CImg<T>(dx,dy,dz,dv).fill(0);
        switch (position) {
          case 'p' : { cimgl_map(*this,ll) { res.draw_image((*this)[ll],pos,0,0,0); pos+=(*this)[ll].width; }} break;
          case 'n' : { cimgl_map(*this,ll) { 
               res.draw_image((*this)[ll],pos,dy-(*this)[ll].height,dz-(*this)[ll].depth,dv-(*this)[ll].dim); pos+=(*this)[ll].width;
               }} break;
          default  : { cimgl_map(*this,ll) {
               res.draw_image((*this)[ll],pos,(dy-(*this)[ll].height)/2,(dz-(*this)[ll].depth)/2,(dv-(*this)[ll].dim)/2);
               pos+=(*this)[ll].width; 
               }} break;
        }
      }
      break;
    case 'y':
      {
        cimgl_map(*this,l) {
          const CImg<T>& img = (*this)[l];
          dx = cimg::max(dx,img.width);
          dy += img.height;
          dz = cimg::max(dz,img.depth);
          dv = cimg::max(dv,img.dim);
        }
        res = CImg<T>(dx,dy,dz,dv).fill(0);
        switch (position) {
           case 'p': { cimgl_map(*this,ll) { res.draw_image((*this)[ll],0,pos,0,0); pos+=(*this)[ll].height; }} break;
           case 'n': { cimgl_map(*this,ll) { 
             res.draw_image((*this)[ll],dx-(*this)[ll].width,pos,dz-(*this)[ll].depth,dv-(*this)[ll].dim); pos+=(*this)[ll].height;
                     }} break;
           default : { cimgl_map(*this,ll) { 
               res.draw_image((*this)[ll],(dx-(*this)[ll].width)/2,pos,(dz-(*this)[ll].depth)/2,(dv-(*this)[ll].dim)/2);
               pos+=(*this)[ll].height; 
                     }} break;
        }
      }
      break;
    case 'z':
      {
        cimgl_map(*this,l) {
          const CImg<T>& img = (*this)[l];
          dx = cimg::max(dx,img.width);
          dy = cimg::max(dy,img.height);
          dz += img.depth;
          dv = cimg::max(dv,img.dim);
        }
        res = CImg<T>(dx,dy,dz,dv).fill(0);
        switch (position) {
          case 'p': { cimgl_map(*this,ll) { res.draw_image((*this)[ll],0,0,pos,0); pos+=(*this)[ll].depth; }} break;
          case 'n': { cimgl_map(*this,ll) { 
            res.draw_image((*this)[ll],dx-(*this)[ll].width,dy-(*this)[ll].height,pos,dv-(*this)[ll].dim); pos+=(*this)[ll].depth;
                    }} break;
          case 'c': { cimgl_map(*this,ll) { 
            res.draw_image((*this)[ll],(dx-(*this)[ll].width)/2,(dy-(*this)[ll].height)/2,pos,(dv-(*this)[ll].dim)/2);
            pos+=(*this)[ll].depth; 
                    }} break;
        }
      }
      break;
    case 'v':
      {
        cimgl_map(*this,l) {
          const CImg<T>& img = (*this)[l];
          dx = cimg::max(dx,img.width);
          dy = cimg::max(dy,img.height);
          dz = cimg::max(dz,img.depth);
          dv += img.dim;
        }
        res = CImg<T>(dx,dy,dz,dv).fill(0);
        switch (position) {
          case 'p': { cimgl_map(*this,ll) { res.draw_image((*this)[ll],0,0,0,pos); pos+=(*this)[ll].dim; }} break;
          case 'n': { cimgl_map(*this,ll) { 
            res.draw_image((*this)[ll],dx-(*this)[ll].width,dy-(*this)[ll].height,dz-(*this)[ll].depth,pos); pos+=(*this)[ll].dim;
                    }} break;
          case 'c': { cimgl_map(*this,ll) { 
            res.draw_image((*this)[ll],(dx-(*this)[ll].width)/2,(dy-(*this)[ll].height)/2,(dz-(*this)[ll].depth)/2,pos);
            pos+=(*this)[ll].dim; 
                    }} break;
        }
      }
      break;
    }
    return res;
  }
  
  //! Open and return a new window display, with a title bar \p title, showing the images of the list.\n
  //! Parameter \p normalize set the way the images are normalized before being displayed (0=no normalization, 1=always normalized, 2=first-time normalized).\n
  //! Parameter \p event_type set how system events are catched by the window (0=no events, 1+=window closed, 2+=mouse and keys down 3+=keys up).
  //! If the 3rd bit of \p event_type is set, the display is done in fullscreen mode.\n
  //! Parameters \p axe (can be 'x','y','z' or 'v') and \p position (can be 'p','c' or 'n')
  //! specify the way the multiple images of the list are concatenated and centered.
  CImgDisplay* new_display(const char *title="",const int normalize=1,const uint event_type=3,const char axe='x',const char position='c') const	{
    return get_append(axe,position).new_display(title,normalize,event_type);
  }
  //! Display an image list into an existing display \p disp.\n
  //! Parameters \p axe (can be 'x','y','z' or 'v') and \p position (can be 'p','c' or 'n')
  //! specify the way the multiple images of the list are concatenated and centered.
  const CImgl& display(CImgDisplay& disp,const char axe='x',const char position='c') const { get_append(axe,position).display(disp); return *this; }
  //! Display an image list into an existing display \p *disp.\n
  //! Parameters \p axe (can be 'x','y','z' or 'v') and \p position (can be 'p','c' or 'n')
  //! specify the way the multiple images of the list are concatenated and centered.
  const CImgl& display(CImgDisplay* disp,const char axe='x',const char position='c') const { return display(*disp,axe,position); }
  //! Display an image list into a new window with specified title \p title.\n
  //! Parameters \p axe (can be 'x','y','z' or 'v') and \p position (can be 'p','c' or 'n')
  //! specify the way the multiple images of the list are concatenated and centered.
  const CImgl& display(const char* title,const char axe='x',const char position='c',const int min_size=-100,const int max_size=1024) const {
    get_append(axe,position).display(title,min_size,max_size);
    return *this;
  }
  //! Display an image list into a new window.\n
  //! Parameters \p axe (can be 'x','y','z' or 'v') and \p position (can be 'p','c' or 'n')
  //! specify the way the multiple images of the list are concatenated and centered.
  const CImgl& display(const char axe='x',const char position='c',const int min_size=-100,const int max_size=1024) const {
    return display("CImgl",axe,position,min_size,max_size); 
  }

  CImgl& swap(CImgl& list) {
    int tmp = list.size; list.size = size; size = tmp;
    CImg<T>* ptr = list.data; list.data = data; data = ptr;
    return list;
  }

  //@}
};


//! This class is only used by the functions CImg::ref_*() in order to return references to part of the original image.\n
//! It should be used as a <tt>const CImg<T></tt> class.
template<typename T> struct CImgROI : CImg<T> {
  CImgROI(const uint dx,const uint dy,const uint dz,const uint dv,T *const pdata) {
     width = dx; height = dy; depth = dz; dim = dv; data = pdata;
  }
  CImgROI(const CImgROI& roi) {
     width = roi.width; height = roi.height; depth = roi.depth; dim = roi.dim; data = roi.data;
  }
  ~CImgROI() { width=height=depth=dim=0; data=NULL;}
};


// Overcome VisualC++ 6.0 and DMC compilers namespace bug
#if ( defined(_MSC_VER) || defined(__DMC__) ) && defined(std)
#undef std
#endif

/*---------------------------------------------------------------------



  Additional documentation for the generation of the reference page
  (using doxygen)



  --------------------------------------------------------------------*/
/**
   \mainpage
   
   \section introduction Reference Documentation

   This is the reference documentation of the CImg Library.
   This documentation has been generated directly from the CImg sources, using \c doxygen.
   It contains a detailed description of all classes and functions of the CImg Library.
   This reference page is intended to help programmers using the CImg Library.
   You can navigate through the documentation, using the menu above.

   \section gettingstarted Getting Started ?

   As a good start, you are invited to visit these links first :

   - \ref cimg_structure
   - \ref cimg_tutorial
   - \ref cimg_environment
   - The reference of the main \ref CImg class, which represents an image.

*/
//------------------------------------------------------------------------------------------------------
/** \addtogroup cimg_environment Environment variables. */
/*@{
\page foo1

  The CImg library is a multiplatform library, working on a wide variety of systems.
  This implies the existence of some environment variables that must be correctly defined
  depending on your current system.
  Fortunately, the CImg Library defines these variables automatically most of the time
  (for standart systems). Anyway, it may happens that some systems are not recognized,
  and then, you have to set the environment variables by hand.\n
  
  Setting the environment variables is done with the <tt>#define</tt> keyword.
  This setting must be done <i>before including the file CImg.h</i> in your source code.
  For instance,
  defining the environment variable \c cimg_display_type would be done like this :\n

  <tt>#define cimg_display_type 0\n#include "CImg.h"\n...</tt>\n

  Here are the different environment variables used by the CImg Library :
  
  - \b \c cimg_OS : This variable defines the type of your current OS. It can be set to 0 (Solaris), 1 (Linux),
  2 (Windows), 3 (Mac OS X) or -1 (Other configuration).
  It should be actually auto-detected by the CImg library. If this is not the case (cimg_OS=-1), then you
  will probably have to tune the environment variables described below.
  - \b \c cimg_display_type : This variable defines the type of display library that will be used to
  display images in windows. It can be set to 0 (no display), 1 (X11-based display) or 2 (WindowsGDI based display).
  If you are running a system without X11 or WindowsGDI ability, please set this variable to 0. This will actually
  disable the display support, since the CImg Library doesn't contain the necessary code to use your current
  graphic system in order to display windows.
  - \b \c cimg_color_term : This variable tells the library if the system terminal has VT100 color capabilities. It can be
  defined or not defined. Define this variable to get colored output on your terminal, when using the CImg Library.
  - \b \c cimg_debug : This variable defines the level of run-time debug messages that will be displayed by
  the CImg Library. It can be set to 0 (no debugging messages), 1 (normal debugging messages, which is
  the default value), or 2 (high debugging messages). Please note that setting this value to 2 may slow down your
  program since more debugging tests are made by the library (particularly to check if pixel access is made outside
  image boundaries).
  - \b \c cimg_lapack : This variable tells the library if the LAPACK library is linked with the code.
  It can be defined or not defined (default). Define this variable if you want to use CImg-specific functions
  based on the LAPACK library. You will have to link your code with the LAPACK library.

  Finally, these compilation variables can be checked, using the function <tt>cimg::info()</tt>, which
  displays on the standart error output a list of the different configuration variables and their values.
*/
/*@}*/
//------------------------------------------------------------------------------------------------------

/** \addtogroup cimg_structure Library structure. */
/*@{
  \page foo2

  The CImg Library only consists in one header file \e CImg.h, containing all the classes and functions
  that compose the library. It is organized as follows :
  
  - All the CImg classes are defined in the global namespace <tt>::</tt>.
  - The namespace <tt>\ref cimg</tt> contains some functions and variables used by the library.
  Documented functions in this namespace can be safely used in your own program.
  - The class \c CImg<T> is the main class that represents an image, up to 4-dimensions wide, containing pixels of type \c T.
  - The class \c CImgl<T> represents a list of \c CImg<T> images.
  - The class \c CImgDisplay is the class that is used to display \c CImg<T> images into windows. The code of
  this class is depending on your current system, and on the value of the environment variable <tt>cimg_display_type</tt>
  (see also \ref cimg_environment ).
  - The class \c CImgStats is used to represent basic image statistics. Use it to compute
  min, max, mean and variance of CImg<T> images.
  - The class \c CImgException (and all its subclasses) is used to throw exceptions by the CImg Library.
  You can catch these exceptions with a bloc <tt>try { ..} catch (CImgException) { .. }</tt> in your code.
  Subclasses of \c CImgException are used to define the type of the problem that has occured.

 A CImg image is defined as a template class CImg<T> containing a pixel data field with a maximum of 4 dimensions :
 the 3 first dimensions are usually used to describe the spatial coordinates <tt>(x,y,z)</tt> in the image, while the last one
 is often used as a vector-valued pixel dimension (color channel for instance). Then, this class can handle the case of 3D volumes
 of vector-valued pixels, and all images that requires less dimensions (gray-valued or color 2D images for instance). 
 It also means that (almost) all member functions of the class CImg<T> are designed to handle the maximum case of these (3+1) dimensions.

 Moreover, the pixel type is given by the template parameter T, which means that you can define images with different pixel types T.
 Fully supported template types are the basic C++ types : <tt>unsigned char, char, short, unsigned int, int, float, double, ... </tt>
 Using your own template types is possible however, but you will have to redefine the complete set of arithmetic and logical operators.
 Typically, fast image display would be done using <tt>CImg<uchar></tt> images, while complex image processing algorithms would be coded
 using <tt>CImg<float></tt> or <tt>CImg<double></tt> images which have floating-point pixel values. Only two others classes are defined
 in the <tt>CImg.h</tt> file : <tt>CImgStats</tt> that is used to represent image statistics, and <tt>CImgDisplay</tt> used to open 
 windows where images are displayed, and handle keyboard and mouse events.

 CImg has been designed to be 'user-friendly', meaning that the underlying structure of the classes is always quite basic to understand.
 For instance, the <tt>CImg<T></tt> class members (which are defined as public) are only the dimensions
 <tt>width, height, depth, dim</tt>, and the pixel data <tt>data</tt>. Very useful when you want to access the raw pixel buffer
 for calling with functions of other libraries, or when you want to convert your raw data buffer into a <tt>CImg</tt>,
 in order to visualize it for instance. Moreover, most of the classical arithmetic and logical operators have been redefined
 in the <tt>CImg<T> class</tt>, which means that accessing a pixel is as simple as <tt>image(x,y)</tt> or <tt>image(x,y,z,v)</tt> (in 4D),
 and beautiful things are possible such as <tt>(img1+3*img2).display();</tt>

 All the functions and classes are defined in a single header file <tt>CImg.h</tt>.
 This sounds strange, but it is actually one of the major advantage of the CImg library :

 - The compilation is then done on the fly : Only functions really used in your program are compiled and appear in the final binary. 
 This leads to very compact executables, without any unused stuffs.
 - Class members and functions are then inlined, leading to better performance during the execution.
 - No complex dependancies have to be handled : Just include the <tt>CImg.h</tt> file, and you get a working toolkit that can process images.
 - It also eases the cohabitation with other image processing libraries. As only used functions are compiled,
 you can include the CImg library and use only one function of if necessary. It won't weight your code at all.\n

 <b><tt>CImg<>, CImgl<>, CImgDisplay, CImgStats</tt></b> : That's the only four classes you have to know in order to use
 the CImg library !\n\n

 Finally, a lot of examples are provided in the CImg package that demonstrate how to use the library.
 Please take some time to look at it. It will surely teaches you a lot of things about using functions
 and classes of the CImg Library.\n
 You can also look at \ref cimg_tutorial, which gives you a step-by-step explanation of a working program.
*/
/*@}*/

//------------------------------------------------------------------------------------------------------

/** \addtogroup cimg_tutorial Tutorial : Getting started. */
/*@{
  \page foo3
  
  Let's start to write our first program to get the idea. This will demonstrate how to load and create images, as well as handle image 
  display and mouse events.
  Assume we want to load a color image 'toto.jpg', smooth it, display it in a windows, and enter an event loop so that clicking a
  point in the image with the mouse will draw the intensity profiles of (R,G,B) of the corresponding image line (in another window).
  Yes, that sounds quite complex for a first code, but don't worry, it will be very simple using the CImg library ! Well, just look
  at the code below, it does the task :

<tt>
    1.  <b>#include "CImg.h"</b>\n
    2.  int main() {\n
    3.  &nbsp; <b>CImg<uchar></b> image("toto.jpg"), visu(500,400,1,3,0);\n
    4.  &nbsp; const uchar red[3]={255,0,0}, green[3]={0,255,0}, blue[3]={0,0,255};\n
    5.  &nbsp; image.<b>blur</b>(2.5);\n
    6.  &nbsp; <b>CImgDisplay</b> main_disp(image,"Click a point"), draw_disp(visu,"Intensity profile");\n
    7.  &nbsp; while (!main_disp.<b>closed</b> && !draw_disp.<b>closed</b>) {\n
    8.  &nbsp;&nbsp;&nbsp; main_disp.<b>wait</b>();\n
    9.  &nbsp;&nbsp;&nbsp; if (main_disp.<b>button</b>) {\n
    10. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; const int y = main_disp.<b>mousey</b>;\n
    11. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; visu.<b>fill</b>(0).<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,0,image.<b>dimx</b>()-1,y,0,0),red,0,256);\n
    12. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; visu.<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,1,image.<b>dimx()</b>-1,y,0,1),green,0,256);\n
    13. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; visu.<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,2,image.<b>dimx()</b>-1,y,0,2),blue,0,256).<b>display</b>(draw_disp);\n
    14. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;  }\n
    15. &nbsp;&nbsp;&nbsp; }\n
    16. &nbsp; return 0;\n
    17.  }\n
</tt>\n
Note that the CImg specific functions has been set to bold face.
<img SRC="../img/tutorial.jpg">

    Here is the complete explanation of the source, line by line :

    <ol>
       <li><tt><b>#include "CImg.h"</b></tt>\n
          Include the main and only header file of the CImg library.
       <li><tt><b>int main() {</b></tt>\n
          Definition of the main function.
       <li><tt><b>CImg<uchar> image("toto.jpg"), visu(500,400,1,3,0);</b></tt>\n
          Creation of two instances of images of unsigned char pixels.\n
          The first one image is initialized by reading an image file from the disk. 
          Here, <tt>toto.jpg</tt> must be in the same directory than the current program.
          Note that you must also have installed the \e ImageMagick package in order to be able to read JPG images.
          The second one visu is initialized as a black color image with dimension <tt>dx=500</tt>, <tt>dy=400</tt>, 
          <tt>dz=1</tt> (here, it is a 2D image, not a 3D one), and <tt>dv=3</tt> (each pixel has 3 'vector' channels R,G,B).
          The last argument in the constructor defines the default value of the pixel values
          (here 0, which means that visu will be initially black).
       <li><tt><b>const uchar red[3]={255,0,0}, green[3]={0,255,0}, blue[3]={0,0,255};</b></tt>\n
          Definition of three different colors as array of unsigned char. This will be used to draw plots with different colors.
       <li><tt><b>image.<b>blur</b>(2.5);</b></tt>\n
          Blur the image, with a gaussian blur and a variance of 2.5. Note that most of the CImg functions have two versions :
          one that acts in-place (which is the case of blur), and one that returns the result as a new image (the name of the function 
          begins then with <tt>get_</tt>&nbsp;). In this case, one could have also written <tt>image = image.get_blur(2.5);</tt>
          (more expensive, since it needs an additional copy operation).
       <li><tt><b><b>CImgDisplay</b> main_disp(image,"Click a point"), draw_disp(visu,"Intensity profile");</b></tt>\n
          Creation of two display windows, one for the input image image, and one for the image visu which will be display intensity profiles.
          By default, CImg displays handles events (mouse,keyboard,..). On Windows, there is a way to create fullscreen displays.
       <li><tt><b>while (!main_disp.<b>closed</b> && !draw_disp.<b>closed</b>) {</b></tt>\n
          Enter the event loop, the code will exit when one of the two display windows is closed.
       <li><tt><b>main_disp.<b>wait</b>();</b></tt>\n
          Wait for an event (mouse, keyboard,..) in the display window <tt>main_disp</tt>.
       <li><tt><b>if (main_disp.<b>button</b>) {</b></tt>\n
          Test if the mouse button is clicked. One may distinguish between the 3 different mouse buttons,
          but in this case it is not necessary
       <li><tt><b>const int y = main_disp.<b>mousey</b>;</b></tt>\n
          Get the image line y-coordinate that has been clicked.
       <li><tt><b>visu.<b>fill</b>(0).<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,0,image.<b>dimx</b>()-1,y,0,0),red,0,256);</b></tt>\n
          This line illustrates the pipeline property of most of the CImg class functions. The first function <tt>fill(0)</tt> simply sets
          all pixel values with 0 (i.e. clear the image <tt>visu</tt>). The interesting thing is that it returns a reference to
          <tt>visu</tt> and then, can be pipelined with the function <tt>draw_plot()</tt> which draws a plot in the image <tt>visu</tt>.
          The plot data are given by another image (the first argument of <tt>draw_plot()</tt>). In this case, the given image is 
          the red-component of the line y of the original image, retrieved by the function <tt>get_sprite()</tt> which returns a
          sub-image of the image <tt>image</tt>. Remember that images coordinates are 4D (x,y,z,v) and for color images,
          the R,G,B channels are respectively given by <tt>v=0, v=1 and v=2</tt>.
       <li><tt><b>visu.<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,1,image.<b>dimx()</b>-1,y,0,1),green,0,256);</b></tt>\n
          Plot the intensity profile for the green channel of the clicked line.
       <li><tt><b>visu.<b>draw_plot</b>(image.<b>get_sprite</b>(0,y,0,2,image.<b>dimx()</b>-1,y,0,2),blue,0,256).<b>display</b>(draw_disp);</b></tt>\n
          Same thing for the blue channel. Note how the function (which return a reference to visu) is pipelined with the function
          <tt>display()</tt> that just paints the image visu in the corresponding display window.
       <li><tt><b>till the end</b></tt>\n
          I don't think you need more explanations !
      </ol>

    As you have noticed, the CImg library allows to write very small and intuitive code. Note also that this source will perfectly work on Unix and Windows systems. Take also a look to this examples provided in the CImg package (particularly the one in the file CImg_test.cpp). It will show you how code can be small with the CImg library. 

    \section s2 How to compile ?

    The CImg library is a very light and user-friendly library. No use of huge and complex dependancies is done, in order to ease
    the compilation. The library examples are compiled with many different C++ compilers before each release :
    <i>Microsoft Visual C++, Visual Studio.NET, Intel IA32 compiler, DMC, G++, ...</i>
    It may happens that some compilers has problems. But in this case, please e-mail me, the portability of the library is one of my 
    main goal, so be sure I'll answer quickly.

    To compile the code above (and generally all code using CImg), you need to link it with the standart visualization library 
    (<tt>GDI32</tt> on Windows, and <tt>X11</tt> on Unix/Mac OS). Here are some examples of compilation commands that are valid :

    - <b>Borland compiler</b> : <tt>bcc32 -w- sample.cpp</tt> (linking is done automatically).
    - <b>Intel IA32 compiler</b> : <tt>icl sample.cpp user32.lib gdi32.lib</tt>
    - <b>Microsoft Visual C++ or .NET</b> : Create a new C++ console project, add the file <tt>sample.cpp</tt> and compile
    (linking is done automatically).
    - <b>Dev-C++</b> : Create a new C++ console project, add the file <tt>sample.cpp</tt> and in the Project Options/Parameters add
    the library object <tt>Dev-Cpp/lib/libgdi32.a</tt> and compile.
    - <b>g++ on Unix, using X11</b> : <tt>g++ -I/usr/X11R6/include -L/usr/X11R6/lib -lX11</tt> 

    Well, there are surely plenty of variants, but believe me, that's very simple since only standart C and system libraries are used by CImg. 

    \section s3 Where to find more sources and tutorials ?

    Take a look at the various examples provided in the CImg Library package. There is surely one example that is close of what 
    you want to do (if this is a basic image processing thing of course !).
    A good start will be to look at the file <tt>CImg_test.cpp</tt> which contains small and various examples of what you can do
    with the CImg Library. All CImg classes are used in this source, and the code can be easily modified to see what happens. 

*/
/*@}*/

//------------------------------------------------------------------------------------------------------

/** \addtogroup cimg_exceptions Using Exceptions. */
/*@{
  \page foo4

  \section s4 Exception classes.

  The CImg Library uses the mechanism of C++ \e exceptions to handle error control.
  We will not discuss the concept of exceptions here, but rather describe the CImg exceptions structure
  which is actually quite classical.\n\n
  Basically, there is one base class <tt>CImgException</tt> and four subclasses that may be throwed
  by CImg library functions. These classes are described below :
  
  - <tt><b>CImgException</b></tt> : This is the base class of all CImg exceptions. It can be throwed in the case of
  very general exceptions by the library functions.\n\n
  - <tt><b>CImgArgumentException</b></tt> : This class will be throwed when one passes one or several invalid arguments to
  a library function. This may happen for instance in the following case :\n\n
  <tt>
  &nbsp;&nbsp;CImg<float> img(100,100);&nbsp;&nbsp;&nbsp; // define a 100x100 scalar image with float pixels\n
  &nbsp;&nbsp;img.get_channel(1); &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // trying to retrieve the vector channel v=1
  </tt>\n\n
  As the image <tt>img</tt> is scalar, it has only one vector channel (<tt>img.dim=1</tt>), and you cannot
  retrieve the channel number 1, but only the channel number 0.\n\n
  - <tt><b>CImgInstanceException</b></tt> : This class will be throwed when trying to call a class function from an
  instance which is not defined, or badly defined. Typically, this exception will happen when using empty images :\n\n
  <tt>
  &nbsp;&nbsp;CImg<float> img; &nbsp;&nbsp;// define an empty image\n
  &nbsp;&nbsp;img.blur(10); &nbsp;&nbsp;&nbsp;&nbsp;&nsp;// trying to blur the empty image.
  </tt>\n\n
  - <tt><b>CImgIOException</b></tt> : This class will be throwed when one Input/Output problem has been encountered
  during the execution of a library function. This may particularly happen when using <tt>load()</tt> and <tt>save()</tt>
  functions, with invalid files :\n\n
  <tt>
  &nbsp;&nbsp;CImg<float> img("not_here.jpg");
  </tt>\n\n
  This code will throw such an exception if the file <tt>not_here.jpg</tt> is not in the current directory.\n\n
  - <tt><b>CImgDisplayException</b></tt> : This class will be throwed when a library function encounters a problem
  when trying to open or close a display window. This shouldn't happen very much if your display is working properly.

  \section s5 Handling exceptions

  By default, when an error occurs, the CImg Library will display an error message on the standart error output (Unix),
  or will open an error window containing the error message (Windows), then will throw the exception, leading
  the program to halt. If you want to bypass this behavior by handling the exceptions yourself, you have to define
  a code block <tt>try { ... } catch() { ... }</tt> and define the environment variable <tt>cimg_debug</tt> to 0,
  avoiding then the display of CImg exception messages.

  The <tt>CImgException</tt> class owns a member variable <tt>char* message</tt> which contains the exception message
  corresponding to the precise description of the error that has been encountered.
  The way you can handle exception errors in your own code is then easy.
  The use of a code like this allows to handle CImg exceptions :\n\n
  <tt>
  #define cimg_debug 0 &nbsp;&nbsp;// Disable error message display by CImg.\n
  #define "CImg.h"\n
  int main() {\n  
  &nbsp;try {\n
  &nbsp;&nbsp;&nbsp; ... &nbsp;&nbsp;// Here, do what you want.\n
  &nbsp;}\n
  &nbsp;catch (CImgException &e) {\n
  &nbsp;&nbsp;&nbsp; fprintf(stderr,"CImg Library Error : %s",e.message); &nbsp;&nbsp;// Display error message\n
  &nbsp;&nbsp;&nbsp; ... &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // Do what you want to save the ship !\n
  &nbsp;}\n
  }\n
  </tt>\n\n

  

*/
/*@}*/

//------------------------------------------------------------------------------------------------------


#endif
